/****************************************************************************
              Copyright (c) 2022 Maxima Technologies and Systems
                           All Rights Reserved

   File Name          : Camera_Platformservice.h
   Author             : Adrush
   Version            : V0.0.0.1
   Origination Date   : 25/02/2022
   Description        : Header File.
   Environment        : Independent
   Processor:         : STM32H743IIT6

-----------------------------------------------------------------------------
   Description:

   This file contains the Camera platform service declarations

-----------------------------------------------------------------------------
	File Revision History:
-----------------------------------------------------------------------------
   Version             Date                 Programmer
   0.0.0.1             25/02/2022			Adrush

   Notes:              - Initial Version
-----------------------------------------------------------------------------
*****************************************************************************
THIS FILE CONTAINS INTELLECTUAL PROPERTY OF MAXIMA TECHNOLOGIES AND SYSTEMS.
UNAUTHORIZED REDISTRIBUTION OF THIS SOURCE CODE, IN WHOLE OR PART, WITHOUT
THE EXPRESS WRITTEN PERMISSION OF MAXIMA TECHNOLOGIES IS STRICTLY PROHIBITED.
****************************************************************************/
#ifndef _CAMERA_PLATFORM_SERVICE_H_
#define	_CAMERA_PLATFORM_SERVICE_H_

/*!
 * Camera TxQ and RxQ size.
 */
#define CAMERA_RX_TX_MAX_Q_SIZE       	10

/*!
 * Camera SDK Index.
 */
#define CAMERA_FUNC_INDEX				0
#define CAMERA_RESULT_INDEX				0
#define CAMERA_X_ORG_INDEX				0
#define CAMERA_Y_ORG_INDEX				2
#define CAMERA_WIDTH_INDEX				4
#define CAMERA_HEIGHT_INDEX				6
#define DISPLAY_X_ORG_INDEX				8
#define DISPLAY_Y_ORG_INDEX				10
#define DISPLAY_FLIP         			12
#define STREAM_MODE         			13

/*!
 * Camera validation index.
 */
#define MIN_HEIGHT_VALUE				0
#define MAX_HEIGHT_VALUE				272

#define MIN_WIDTH_VALUE					0
#define MAX_WIDTH_VALUE					500

#define MIN_X_ORIGIN_VALUE				0
#define MAX_X_ORIGIN_VALUE				0

#define MIN_Y_ORIGIN_VALUE				0
#define MAX_Y_ORIGIN_VALUE				0

#define MIN_DISP_X_ORIGIN_VALUE			0
#define MAX_DISP_X_ORIGIN_VALUE			0

#define MIN_DISP_Y_ORIGIN_VALUE			0
#define MAX_DISP_Y_ORIGIN_VALUE			0

/*!
 *  MAX Camera width and Height
 */
#define MAX_CAMERA_WIDTH                720 //PAL Resolution ,600
#define MAX_CAMERA_HEIGHT               576 //PAL Resolution ,236

#define RESIZE_WIDTH_TO_480             LCD_RESOlUTION_X0
#define RESIZE_WIDTH_TO_240             LCD_RESOlUTION_X0/2
#define RESIZE_WIDTH_TO_120             LCD_RESOlUTION_X0/4
#define RESIZE_WIDTH_TO_60              LCD_RESOlUTION_X0/6

#define RESIZE_HEIGTH_TO_136            LCD_RESOlUTION_Y0/2
#define RESIZE_HEIGTH_TO_68             LCD_RESOlUTION_Y0/4
#define RESIZE_HEIGTH_TO_34             LCD_RESOlUTION_Y0/6

/* Framebuffer Address */
#define LAYER_0_FRAMEBUFFER  			0xC0000000
#define LAYER_1_FRAMEBUFFER  			0xC0040000
#define CAMERA_IMAGE_BUFFER  			0x30000000
#define TEMP_FRAMEBUFFER     			0xC0100000


/*!
 * Camera Events
 */
typedef enum __CAMERA_Event
{
	CAMERA_TMR_EVENT,
	CAMERA_MSG_EVENT,
}CAMERA_Event;


/*!
 * Camera Stream mode
 */
typedef enum __Stream_Mode
{
	FULL_SCREEN_ON=1,
	RESIZE_TO_FULL_SCREEN_ON,
	DISPLAY_AS_IT_IS_ON,
}Stream_Mode;

/*!
 * Camera Stream state
 */
typedef enum __Stream_State
{
	CAMERA_STREAM_ON=1,
	CAMERA_STREAM_OFF,
}Stream_State;

/*!
 * Camera Flip state
 */
typedef enum __CAMERA_Flip_State
{
	FLIP_DISABLE,
	FLIP_ENABLE,
}CAMERA_Flip_State;

/*!
 * Camera Flip Mode
 */
typedef enum __CAMERA_Flip_Mode
{
	FLIP_OFF = 1,
	FLIP_VERTICAL,
	FLIP_HORIZONTAL,
}CAMERA_Flip_Mode;

#if (SDK_SERVICE_CAMERA == PS_ENABLE)

HAL_StatusTypeDef SetWindowPosition(LTDC_HandleTypeDef *hltdc,
		                            uint32_t X0,
									uint32_t Y0,
									uint16_t W0,
									uint16_t H0,
									uint32_t LayerIdx);
int8_t Camera_DB_Init(void);
void StopStreaming();
void Set_LCD_RGB(uint32_t rgb,uint16_t skip_pix);
void CM_Delay(uint32_t step);
void Camera_Set_Stream_ON(void);
#endif
#endif //_CAMERA_PLATFORM_SERVICE_H_

/**************** (C) COPYRIGHT 2022 Maxima Technologies *******************/
/***************************** END OF FILE *********************************/
