/****************************************************************************
              Copyright (c) 2022 Maxima Technologies and Systems
                           All Rights Reserved

   File Name          : Fdcan_Platformservice.h
   Author             : Rajesh
   Version            : V0.0.0.1
   Origination Date   : 04/03/2022
   Description        : Header File.
   Environment        : Independent
   Processor:         : STM32H743IIT6

-----------------------------------------------------------------------------
   Description:

   This file contains the FDCAN platform service declarations

-----------------------------------------------------------------------------
	File Revision History:
-----------------------------------------------------------------------------
   Version             Date                 Programmer
   0.0.0.1             04/03/2022			Rajesh

   Notes:              - Initial Version
-----------------------------------------------------------------------------
*****************************************************************************
THIS FILE CONTAINS INTELLECTUAL PROPERTY OF MAXIMA TECHNOLOGIES AND SYSTEMS.
UNAUTHORIZED REDISTRIBUTION OF THIS SOURCE CODE, IN WHOLE OR PART, WITHOUT
THE EXPRESS WRITTEN PERMISSION OF MAXIMA TECHNOLOGIES IS STRICTLY PROHIBITED.
****************************************************************************/
#ifndef _FDCAN_PLATFORMSERVICE_H_
#define _FDCAN_PLATFORMSERVICE_H_

/* USER CODE BEGIN enum */

/*!
 * FDCAN Functionality
 */
typedef enum __FDCAN_Func
{
	FDCAN_OFF,
	FDCAN_ON,

}FDCAN_Func;

/*!
 * FDCAN SDK Message
 */
typedef enum __FDCAN_MSG
{
	FDCAN1_SET,
	FDCAN2_SET,
	FDCAN_ACT,
	FDCAN_GET,
}FDCAN_MSG;

/*!
 * FDCAN ID
 */
typedef enum __FDCAN_id
{
	FDCAN_1,
	FDCAN_2,
}FDCAN_id;

/*!
 * FDCAN SDK Message
 */
typedef enum __FDCAN_INDEX
{
	INDEX0,
	INDEX1
}FDCAN_INDEX;

/*!
 * FDACN Events
 */
typedef enum __FDCAN_Event
{
	FDCAN_TMR_EVENT,
	FDCAN_MSG_EVENT,
}FDCAN_Event;


/*!
 * FDCAN Filter Information
 */
typedef struct __FDCAN_Configuration_Info
{
    uint32_t FDCAN_id;
    uint32_t FDCAN_FilterIndex;
    uint32_t FDCAN_FilterType;
    uint32_t FDCAN_FilterConfig;
    uint32_t FDCAN_FilterId1;
    uint32_t FDCAN_FilterId2;
    uint32_t RxBufferIndex;
    uint8_t  FDCAN_FilterEnable;
}FDCAN_Config_Info_Typedef;


/*!
 * FDCAN_Communication_Event
 */
typedef enum __FDCAN_Communication_state
{
	STATE_EVENT_NONE,
	STATE_EVENT_BUS_OFF,
	STATE_EVENT_BUS_OFF_RECOVERY,
	STATE_EVENT_BUS_ON,
	STATE_EVENT_PASSIVE,
	STATE_EVENT_ACTIVE,
	STATE_EVENT_OVERRUN,
	STATE_EVENT_QUEUE_FULL,
	STATE_EVENT_QUEUE_OVERFLOW,
	STATE_EVENT_QUEUE_EMPTY,
	STATE_EVENT_DRIVER_ERROR
}FDCAN_Communication_state;

/*!
 * FDCAN_State
 */
typedef enum __FDCAN_State
{
	CAN_BUS_OFF,
	CAN_BUS_ON,
	CAN_STATE_PASSIVE,
	CAN_STATE_UNCHANGED,
}FDCAN_State;


/*!
 * FDCAN Tx structure definition
 */
typedef struct __FDCAN_Tx_Configuration_Info
{
	uint32_t FDCAN_Identifier;
	uint32_t FDCAN_IdType;
	uint32_t FDCAN_TxFrameType;
	uint32_t FDCAN_DataLength;
	uint32_t FDCAN_ErrorStateIndicator;
	uint32_t FDCAN_BitRateSwitch;
	uint32_t FDCAN_FDFormate;
	uint32_t FDCAN_TxEventFifoControl;
	uint32_t FDCAN_MessageMarker;
}FDCAN_Tx_Config_Info_Typedef;

/*!
 * FDCAN_Mode_id
 */
typedef enum __FDCAN_Mode_id
{
	CAN_MODE_STANDARD_ID,
	CAN_MODE_EXTENDED_ID
}FDCAN_Mode_id;

/*!
 * FDCAN Supporting BaudRate
 */
#define BAUDRATE_50K							0
#define BAUDRATE_100K							1
#define BAUDRATE_125K							2
#define BAUDRATE_250K							3
#define BAUDRATE_500K							4
#define BAUDRATE_1000K							5

/*!
 * Max Number of FDCAN
 */
#define MAX_NUM_FDCAN            				2

/*!
 * FDCAN Enable
 */
#define   FDCAN_ENABLE							1

/*!
 * FDCAN disable
 */
#define   FDCAN_DISABLE							0

/*!
 * DB init state
 */
#define FDCAN_DB_INIT_DONE						1

/*!
 * FDCAN Platform Service configuration
 */
#define PS_FDCAN_TIMEOUT						100

#define FDCAN_RX_TX_MAX_Q_SIZE					10

/*!
 *
 */
#define FDCAN_RXBUFF_SIZE						40

/*!
 * FDCAN default filter config value
 */
#define FDCAN_DEFAULT_ID						((uint32_t)0x00000000U) //FDCAN_STANDERD_ID
#define FDCAN_DEAULT_INDEX						0
#define FDCAN_DEFAULT_FILTERTYPE				((uint32_t)0x00000001U) // Dual ID filter for FilterID1 or FilterID2
#define FDCAN_DEFAULT_FILTERCONFIG				((uint32_t)0x00000007U) // Store into Rx Buffer, configuration of FilterType ignored.
#define FDCAN_DEAFULT_FILTERID1					(0x111)
#define FDCAN_DEAFULT_FILTERID2					(0x555)
#define FDCAN_DEFAULT_RXBUFFERINDEX				0

/*!
 * FDCAN default config value
 */
#define FDCAN_DEFAULT_IDENTIFIER				(0x111)
#define FDCAN_DEFAULT_IDTYPE					((uint32_t)0x00000000U) //FDCAN_STANDERD_ID
#define FDCAN_DEFAULT_TXFRAME_TYPE				((uint32_t)0x00000000U) // FDCAN Data Frame
#define FDCAN_DEFAULT_DATALENGTH				((uint32_t)0x00080000U) // FDCAN Data length
#define FDCAN_DEFAULT_ERROR_STAE_IDICATOR		((uint32_t)0x00000000U)
#define FDCAN_DEFAULT_BITRATESWITCH				((uint32_t)0x00100000U)
#define FDCAN_DEFULT_FDFORMATE					((uint32_t)0x00200000U)
#define FDCAN_DEFULT_TXEVENT_FIFO_CONTROL		((uint32_t)0x00800000U)
#define FDCAN_DEFAULT_MESSAGE_MAKER				0
/* !
 * FDCAN SDK Field Index
 */
#define FDCAN_FUNC_INDEX						0
#define FDCAN_RESULT_INDEX						0
#define FDCAN_FUNC_SUB_INDEX					1
#define FDCAN_DATA_INDEX						2
#define FDCAN_LEN_INDEX							3


#define MIN_RANGE_FILTER_INDEX_STD				0
#define MAX_RANGE_FILTER_INDEX_STD				32

#define MIN_RANGE_FILTER_INDEX_EXE				0
#define MAX_RANGE_FILTER_INDEX_EXE				64

/*!
 * SDK Function ACT index
 */
#define SET_FDCAN_MODE							1
#define SET_FILTER_INDEX_ENABLE					2
#define SET_FDCAN_RESET							3
#define SET_FDCAN_DRIVER_RESET					4
#define SET_FDCAN_BAUDRATE						5


#define MIN_BYTE_VAL							0
#define MAX_BYTE_VAL							60

#define MIN_DATA_VAL							0
#define MAX_DATA_VAL							60

/* USER CODE BEGIN EFP */
void PS_FDCAN1_Init(uint8_t baud_idx);
void PS_FDCAN2_Init(uint8_t baud_idx);
int8_t FDCAN_DB_Init(void);
int8_t FDCAN_Configuration(void);
/* USER CODE END EFP */

#endif /* _FDCAN_PLATFORMSERVICE_H_ */
