/****************************************************************************
              Copyright (c) 2022 Maxima Technologies and Systems
                           All Rights Reserved

   File Name          : Led_Platformservice.h
   Author             : Adrush
   Version            : V0.0.0.1
   Origination Date   : 04/02/2022
   Description        : Header File.
   Environment        : Independent
   Processor:         : STM32H743IIT6

-----------------------------------------------------------------------------
   Description:

   This file contains the LED platform service declarations

-----------------------------------------------------------------------------
	File Revision History:
-----------------------------------------------------------------------------
   Version             Date                 Programmer
   0.0.0.1             04/02/2022           Adrush

   Notes:              - Initial Version
-----------------------------------------------------------------------------
*****************************************************************************
THIS FILE CONTAINS INTELLECTUAL PROPERTY OF MAXIMA TECHNOLOGIES AND SYSTEMS.
UNAUTHORIZED REDISTRIBUTION OF THIS SOURCE CODE, IN WHOLE OR PART, WITHOUT
THE EXPRESS WRITTEN PERMISSION OF MAXIMA TECHNOLOGIES IS STRICTLY PROHIBITED.
****************************************************************************/
#ifndef _LED_PLATFORM_SERVICE_H_
#define	_LED_PLATFORM_SERVICE_H_

/*!
 * LED TxQ and RxQ size for
 * send and receiving data from SDK API.
 */
#define LED_RX_TX_MAX_Q_SIZE     		10

/*!
 * LED Platform service Task timeout value
 */
#define PS_LED_TIMER_COUNT				250
#define LED_MIN_BLK_TIME     			0
#define LED_MAX_BLK_TIME     			65535

/*!
 * LED Platform Service Task Size
 */
#define LED_TASK_SIZE         			512

/* !
 * LED SDK Field Index
 */
#define LED_FUNC_INDEX             		0
#define LED_RESULT_INDEX           		0
#define LED_NUM_INDEX              		0
#define LED_STATE_INDEX            		1
#define LED_BLK_INDEX              		2

/*!
 * LED Enable /
 * LED disable
 */
#define   LED_ENABLE          			1
#define   LED_DISABLE         			0

/*!
 * LED ON /
 * LED OFF
 */
#define   LED_CONF_ON   	      		2
#define   LED_CONF_OFF    		  		1

/* Thread State */
#define LED_THREAD_STATE_RUN   			0
#define LED_THREAD_STATE_STOP  			1

#define   MAX_PLATFORM_SUPPORTED_LED	2

/*!
 * LED Events
 */
typedef enum __LED_Event
{
	LED_TMR_EVENT,
	LED_MSG_EVENT,
	LED_CAM_STOP_EVENT,
	LED_CAM_START_EVENT
}LED_Event;


/*!
 * LED SDK Message
 */
typedef enum __LED_MSG
{
	LED_ACT,
	LED_SET,
}LED_MSG;

/*!
 * LED Functionality
 */
typedef enum __LED_Func
{
	LED_OFF=1,
	LED_ON
}LED_Func;

/*!
 * LED configuration information
 */
typedef struct __LED_cfg_info
{
	uint8_t  led_enable;    /* Enable / Disable the LED */
	uint8_t  led_state;     /* Current LED State       */
	uint16_t blk_period;    /* Duration to blinking    */
	uint16_t timer_count;   /* Current timer count     */
	uint8_t  blk_state;     /* Blinking State          */
}LED_cfg_info;

/*!
 * Function declaration
 */
int8_t LED_DB_Init(void);

#endif	//_LED_PLATFORM_SERVICE_H_

/**************** (C) COPYRIGHT 2022 Maxima Technologies *******************/
/***************************** END OF FILE *********************************/
