/****************************************************************************
              Copyright (c) 2022 Maxima Technologies and Systems
                           All Rights Reserved

   File Name          : PM_platformservice.h
   Author             : Arjun
   Version            : V0.0.0.1
   Origination Date   : 11/02/2022
   Description        : Header File.
   Environment        : Independent
   Processor:         : STM32H743IIT6

-----------------------------------------------------------------------------
   Description:

   This file contains the Power Management platform service declarations

-----------------------------------------------------------------------------
	File Revision History:
-----------------------------------------------------------------------------
   Version             Date                 Programmer
   0.0.0.1             11/02/2022			Arjun

   Notes:              - Initial Version
-----------------------------------------------------------------------------
*****************************************************************************
THIS FILE CONTAINS INTELLECTUAL PROPERTY OF MAXIMA TECHNOLOGIES AND SYSTEMS.
UNAUTHORIZED REDISTRIBUTION OF THIS SOURCE CODE, IN WHOLE OR PART, WITHOUT
THE EXPRESS WRITTEN PERMISSION OF MAXIMA TECHNOLOGIES IS STRICTLY PROHIBITED.
****************************************************************************/
#ifndef _PM_PLATFORM_SERVICE_H_
#define	_PM_PLATFORM_SERVICE_H_

/* !
 * Power Management SDK Field Index
 */
#define PM_FUNC_INDEX             0
#define PM_RESULT_INDEX           0
#define PM_MODE_INDEX             1

/* !
 * Power Management SDK Field Index
 */
#define PM_RTC_WAKEUP_MIN	      0
#define PM_RTC_WAKEUP_MAX         65535

/*!
 * Function declaration
 */
int8_t PM_DB_Init(void);
void PM_CB_Clear(void);

/*!
 * Power Management Functionality
 */
typedef enum __PM_Func
{
	PM_FUNC
}PM_Func;

/*!
 * Power Management SDK Message
 */
typedef enum __PM_MSG
{
	PM_SET,
	PM_GET,
	PM_ACT,
	PM_SLEEP_MODE,
	PM_STOP_MODE,
	PM_STANDBY_MODE,
	PM_NORMAL_MODE,
}PM_MSG;

/*!
 * Power Management Events
 */
typedef enum __PM_Event
{
	PM_TMR_EVENT,
	PM_MSG_EVENT,
}PM_Event;

/*!
 * Power Management Platform service Task timeout value
 */
#define PS_LS_RAWREAD_TIMER       100

/*!
 * Power Management TX Q size for
 * receiving data from SDK API.
 */
#define PM_RX_TX_MAX_Q_SIZE       10

/*!
 * PM callback register structure
 */
typedef struct __PM_Callbacks
{
	void (*DeInit)();
	void (*WakeupSource)();
	void (*ReInit)();
} PM_Callbacks;


/* Function Declaration */
void PM_Register_Callback(void (*DeInit)(void), void(*WakeupSource)(void),
		                      void (*ReInit)(void));
void PM_DeInit();
void PM_ReInit();
void PM_Conf_Wakeup_Source();

#endif	//_PM_PLATFORM_SERVICE_H_

/**************** (C) COPYRIGHT 2022 Maxima Technologies *******************/
/***************************** END OF FILE *********************************/
