/****************************************************************************
              Copyright (c) 2022 Maxima Technologies and Systems
                           All Rights Reserved

   File Name          : TerminalCommand.h
   Author             :
   Version            : V0.0.0.1
   Origination Date   : 14/02/2022
   Description        : Header File.
   Environment        : Independent
   Processor:         : STM32H743IIT6

-----------------------------------------------------------------------------
   Description:

   This file contains the Terminal command header file

-----------------------------------------------------------------------------
	File Revision History:
-----------------------------------------------------------------------------
   Version             Date                 Programmer
   0.0.0.1             14/02/2022

   Notes:              - Initial Version
-----------------------------------------------------------------------------
*****************************************************************************
THIS FILE CONTAINS INTELLECTUAL PROPERTY OF MAXIMA TECHNOLOGIES AND SYSTEMS.
UNAUTHORIZED REDISTRIBUTION OF THIS SOURCE CODE, IN WHOLE OR PART, WITHOUT
THE EXPRESS WRITTEN PERMISSION OF MAXIMA TECHNOLOGIES IS STRICTLY PROHIBITED.
****************************************************************************/
#ifndef _TERMINALCOMMAND_H_
#define	_TERMINALCOMMAND_H_


#define PKT_MODE_INDEX   0
#define PKT_DADR_INDEX   1
#define PKT_TADR_INDEX   2
#define PKT_SIZE_INDEX   3
#define MSG_ID_INDEX     4

/*!
 * USB Serial command packet structure
 */
typedef struct __baFrame
{
	uint8_t bMode;
	uint8_t bDestAddress;
	uint8_t bToolAddress;
	uint8_t bSize;
}baFrame;

/*!
 * USB commands
 */
typedef enum __cmdtype
{
	readDiagnosticTroubleCodes = 0x13,
	stopRepeatedDataTransmission = 0x25,
	setDataRates = 0x26,
	writeDataByLocalIdentifier = 0x3B,
	dynamicallyDefineLocalIdentifier = 0x2C,
	readDataByLocalIdentifier = 0x21,
	securityAccess = 0x27,
	stopDiagnosticSession = 0x20,
	startDiagnosticSession = 0x10,
	testerPresent = 0x3E,
	accessTimParameters = 0x83,
	writeMemoryByAddress = 0x3D,
	writeDataByCommonIdentifier = 0x2E,
	readMemoryByAddress = 0x23,
	readDataByCommonIdentifier = 0x22,
	readECUIdentification = 0x1A,
	ECUReset = 0x11,
	stopCOM = 0x82,
	startCOM = 0x81,
	userPacket =0x50,


}cmdtype;

#define RXMODE           0x80
#define TXMODE           0x80
#define DEST_ADDRESS     0x61
#define TOOL_ADDRESS     0x51

/*!
 * USB Command packet header
 */
#define DB_MAX_VAL_SIZE                  64

/*!
 * USB Command packet header
 */
#define CMD_PKT_HEADER_SIZE              4

/* !
 * USB Packet macros
 */
#define START_REQ_PKT_SIZE               6
#define START_REQ_PKT_DATA_SIZE          1
#define START_RESP_PKT_SIZE              3
#define START_RESP_TOTAL_PKT_SIZE        8
#define STOP_REQ_PKT_SIZE                6
#define STOP_REQ_PKT_DATA_SIZE           1
#define STOP_RESP_PKT_SIZE               3
#define STOP_RESP_TOTAL_PKT_SIZE         6
#define ECURESET_REQ_PKT_SIZE            6
#define ECURESET_REQ_PKT_DATA_SIZE       1
#define ECURESET_RESP_PKT_SIZE           1
#define ECURESET_RESP_TOTAL_PKT_SIZE     6

/* !
 * STM32H7 Memory Address
 */
#define FLASH_START_ADDR     0x08002000 //0x08000000
#define FLASH_END_ADDR       0x08200000
#define DRCMRAM_START_ADDR   0x20000000
#define DRCMRAM_END_ADDR     0x20020000
#define RAMD1_START_ADDR     0x24000000
#define RAMD1_END_ADDR       0x24080000
#define RAMD2_START_ADDR     0x30000000
#define RAMD2_END_ADDR       0x30048000
#define RAMD3_START_ADDR     0x38000000
#define RAMD3_END_ADDR       0x38010000
#define ITCMRAM_START_ADDR   0x00000000
#define ITCMRAM_END_ADDR     0x00010000
#define SDRAM_START_ADDR     0xC0000000
#define SDRAM_END_ADDR       0xC0100000

/*!
 *  USB Communication Positive and Negative response value
 */
#define POSITIVE_COM_RESP 0x40
#define NEGATIVE_COM_RESP 0x7F

/*!
 *  USB Communication version
 */
#define COMM_VERSION   0x1200

/*!
 * Function declaration
 */
int8_t CalCheckSum(uint8_t *BufferToCal, uint8_t len);
int8_t SetstartCOM(uint8_t *rxpkt, uint8_t *rxlen);
int8_t SetstopCOM(uint8_t *rxpkt, uint8_t *rxlen);
void SetecuReset(uint8_t *rxpkt, uint8_t *rxlen);
void GetreadEcuIdentification(uint8_t *rxpkt, uint8_t *rxlen);
void GetreadDataByCommonIdentifier(uint8_t *rxpkt, uint8_t *rxlen);
void GetreadMemoryByAddress(uint8_t *rxpkt, uint8_t *rxlen);
void SetwriteDataByCommonIdentifier(uint8_t *rxpkt, uint8_t *rxlen);

#endif	//_USB_PLATFORM_SERVICE_H_

/**************** (C) COPYRIGHT 2022 Maxima Technologies *******************/
/***************************** END OF FILE *********************************/
