/****************************************************************************
              Copyright (c) 2022 Maxima Technologies and Systems
                           All Rights Reserved

   File Name          : Warning_Light_platformservice.h
   Author             : Rajesh
   Version            : V0.0.0.1
   Origination Date   : 21/01/2022
   Description        : Header File.
   Environment        : Independent
   Processor:         : STM32H743IIT6

-----------------------------------------------------------------------------
   Description:

   This file contains the Warning Light platform service declerations

-----------------------------------------------------------------------------
	File Revision History:
-----------------------------------------------------------------------------
   Version             Date                 Programmer
   0.0.0.1             21/01/2022           Rajesh

   Notes:              - Initial Version
-----------------------------------------------------------------------------
*****************************************************************************
THIS FILE CONTAINS INTELLECTUAL PROPERTY OF MAXIMA TECHNOLOGIES AND SYSTEMS.
UNAUTHORIZED REDISTRIBUTION OF THIS SOURCE CODE, IN WHOLE OR PART, WITHOUT
THE EXPRESS WRITTEN PERMISSION OF MAXIMA TECHNOLOGIES IS STRICTLY PROHIBITED.
****************************************************************************/
#ifndef _WARNING_LIGHT_PLATFORM_SERVICE_H_
#define	_WARNING_LIGHT_PLATFORM_SERVICE_H_

/* USER CODE BEGIN enum */
/*!
 * Warning Light Functionality
 */
typedef enum __WL_Func
{
	WL_ON=1,
	WL_OFF,
	WL_BLINKING,
	WL_SET_BRIGHTNESS,
}WL_Func;

/*!
 * Warning Light SDK Message
 */
typedef enum __WL_MSG
{
	WL_SET,
	WL_ACT,
}WL_MSG;

/*!
 * Warning Light Events
 */
typedef enum __WL_Event
{
	WL_TMR_EVENT,
	WL_MSG_EVENT,
	WL_CAM_STOP_EVENT,
	WL_CAM_START_EVENT
}WL_Event;
/* USER CODE END enum */

/* USER CODE BEGIN Private defines */
/*!
 * WL peripheral interface
 */
#define WL_I2C1    I2C1

/* Thread State */
#define WL_THREAD_STATE_RUN   0
#define WL_THREAD_STATE_STOP  1

/*!
 * WL Platform service Task timeout value
 */
#define PS_WL_BL_TIMER       250
#define WL_MIN_BLK_TIME      0
#define WL_MAX_BLK_TIME      65535
#define WL_MIN_PWM_DC        0
#define WL_MAX_PWM_DC        100

/*!
 * Warning light Platform Service Task Size
 */
#define WL_TASK_SIZE         512

/*!
 * Return Error value
 */
#define   INVALID_EE_DATA    -1

/*!
 * Return Error value
 */
#define   VALID_EE_DATA      1

/*!
 * Warning Light Enable
 */
#define   WL_ENABLE          1

/*!
 * Warning Light disable
 */
#define   WL_DISABLE         0

/*!
 * Warning Light ON
 */
#define   WL_CONF_ON         1

/*!
 * Warning Light OFF
 */
#define   WL_CONF_OFF        0

/*!
 * Warning Light Frequencies
 */
#define WL_kHz3_FREQ         0
#define WL_kHz22_FREQ        1

/*!
 * Warning Light Frequencies
 */
#define WL_OUT_CURRENT_MAX   	0
#define WL_OUT_CURRENT_MAX_2    1
#define WL_OUT_CURRENT_MAX_4    2
#define WL_OUT_CURRENT_MAX_8    3

/*!
 * Warning Light TX Q size for
 * receiving data from SDK API.
 */
#define WL_RX_TX_MAX_Q_SIZE     10

/*!
 * MAX number of Warning Light supported in this platform
 */
#define MAX_PLATFORM_SUPPORTED_WL 20

/*!
 * DB init state
 */
#define WL_DB_INIT_DONE		      1

/*!
 * PWM Duty cycle PWM value
 */
#define MIN_WL_PWM_DC             0
#define MAX_WL_PWM_DC             100

/*!
 * Warning light blink period
 */
#define MIN_WL_BLINK_PERIOD       0
#define MAX_WL_BLINK_PERIOD       65535

/* !
 * Warning Light SDK Field Index
 */
#define WL_FUNC_INDEX             0
#define WL_RESULT_INDEX           0
#define WL_NUM_INDEX              0
#define WL_STATE_INDEX            1
#define WL_DC_INDEX               2
#define WL_BL_INDEX               3

/* USER CODE END Private defines */

/* USER CODE BEGIN PTD */
/*!
 * warning light blink information
 */
typedef struct __WL_Blinking_info
{
	uint8_t  wl_enable;     /* Enable / Disable the WL */
	uint8_t  wl_state;      /* Current LED State       */
	uint8_t  duty_cyle;     /* PWM duty cycle          */
	uint16_t blk_time;      /* Duration to blinking    */
	uint16_t cur_count;     /* Current timer count     */
	uint8_t  blnk_state;    /* Blinking State          */
}WL_Blinking_info;
/* USER CODE END PTD */

/* USER CODE BEGIN EFP */
int8_t Warning_Light_DB_Init(void);
/* USER CODE END EFP */

#endif	//_WARNING_LIGHT_PLATFORM_SERVICE_H_

/**************** (C) COPYRIGHT 2022 Maxima Technologies *******************/
/***************************** END OF FILE *********************************/
