/****************************************************************************
              Copyright (c) 2022 Maxima Technologies and Systems
                           All Rights Reserved

   File Name          : Watchdog_platformservice.h
   Author             : Zaid
   Version            : V0.0.0.1
   Origination Date   : 04/02/2022
   Description        : Header File.
   Environment        : Independent
   Processor:         : STM32H743IIT6

-----------------------------------------------------------------------------
   Description:

   This file contains the watchdog platform service declarations

-----------------------------------------------------------------------------
	File Revision History:
-----------------------------------------------------------------------------
   Version             Date                 Programmer
   0.0.0.1             04/02/2022           Zaid

   Notes:              - Initial Version
-----------------------------------------------------------------------------
*****************************************************************************
THIS FILE CONTAINS INTELLECTUAL PROPERTY OF MAXIMA TECHNOLOGIES AND SYSTEMS.
UNAUTHORIZED REDISTRIBUTION OF THIS SOURCE CODE, IN WHOLE OR PART, WITHOUT
THE EXPRESS WRITTEN PERMISSION OF MAXIMA TECHNOLOGIES IS STRICTLY PROHIBITED.
****************************************************************************/
#ifndef _WATCHDOG_PLATFORM_SERVICE_H_
#define	_WATCHDOG_PLATFORM_SERVICE_H_

/*!
 * Watchdog maximum TxQ and RxQ size.
 */
#define WD_RX_TX_MAX_Q_SIZE     	10

/*!
 * Watchdog Platform Service Task Size
 */
#define WD_TASK_SIZE        		512

/*!
 * Watchdog max reset count
 */
#define MAX_RESET_COUNT				20

/* !
 * Watchdog SDK Field Index
 */
#define WD_FUNC_INDEX             	0
#define WD_RESULT_INDEX           	0
#define WD_FEED_ID_INDEX		  	1

/*!
 * MAX Watchdog internal ID's
 */
#define MAX_USER_WD_ID				10

/*!
 * Total number of Watchdog ID's
 */
#define MAX_WD_INTERNAL_ID 			14
#define MAX_NUMBER_OF_WD			(MAX_WD_INTERNAL_ID + MAX_USER_WD_ID)

/*!
 * Watchdog internal Task ping_id
 */
#define WD_PING_WL_TASK				0
#define WD_PING_RTC_TASK			1
#define WD_PING_DIO_TASK			2
#define WD_PING_LS_TASK				3
#define WD_PING_LED_TASK			4
#define WD_PING_CI_TASK				5
#define WD_PING_LCD_TASK			6
#define WD_PING_POWERMODE_TASK		7
#define WD_PING_POWERMONITOR_TASK	8
#define WD_PING_EE_TASK     	    9
#define WD_PING_SWTIMER_TASK		10
#define WD_PING_BLE_TASK			11
#define WD_PING_USB_TASK			12
#define WD_PING_CAMERA_TASK			13
#define WD_PING_USER_0				14
#define WD_PING_USER_1				15
#define WD_PING_USER_2				16
#define WD_PING_USER_3				17
#define WD_PING_USER_4				18
#define WD_PING_USER_5				19
#define WD_PING_USER_6				20
#define WD_PING_USER_7				21
#define WD_PING_USER_8				22
#define WD_PING_USER_9				23

#define STOP_RESET_WATCHDOG         1


/*!
 * Base Value
 */
#define WD_BASE_VALUE               2

/*!
 * Watchdog SDK Message
 */
typedef enum __WD_MSG
{
	WD_SET,
}WD_MSG;

/*!
 * Watchdog Events
 */
typedef enum __WD_Event
{
	WD_TMR_EVENT,
	WD_MSG_EVENT,
}WD_Event;

/*!
 * WatchDog Info
 */
typedef struct __WatchDog_info
{
	uint8_t feed_id;
	uint8_t reset_count;
	uint8_t isenabled;
}WatchDog_info;

/*!
 * Function Declaration
 */
int8_t Watchdog_DB_Init(void);

#endif	//_WATCHDOG_PLATFORM_SERVICE_H_

/**************** (C) COPYRIGHT 2022 Maxima Technologies *******************/
/***************************** END OF FILE *********************************/
