/****************************************************************************
              Copyright (c) 2012 Maxima Technologies and Systems
                           All Rights Reserved

   File Name          : Analog_Handler.c
   Author             : Jonathan Serrano
   Version            : V0.0.0.1
   Origination Date   : 07/05/2019
   Description        : C Source File.
   Environment        : Independent
   Processor:         : STM32F746IG

-----------------------------------------------------------------------------
   Description:

   This follow the next specification file:

-----------------------------------------------------------------------------
	File Revision History:
-----------------------------------------------------------------------------
   Version             Date                 Programmer
   0.0.0.0              02/18/2020         Jonathan Serrano

   Notes:              - Initial Version
-----------------------------------------------------------------------------
*****************************************************************************
THIS FILE CONTAINS INTELLECTUAL PROPERTY OF MAXIMA TECHNOLOGIES AND SYSTEMS.
UNAUTHORIZED REDISTRIBUTION OF THIS SOURCE CODE, IN WHOLE OR PART, WITHOUT
THE EXPRESS WRITTEN PERMISSION OF MAXIMA TECHNOLOGIES IS STRICTLY PROHIBITED.
****************************************************************************/
/* Includes ---------------------------------------------------------------*/
#include "../Inc/Common.h"

#if ((SDK_SERVICE_DIGITAL_OUTPUT == PS_ENABLE) ||	\
			(SDK_SERVICE_CFG_INPUT == PS_ENABLE))
/* Public typedef ---------------------------------------------------------*/
/* Public define ----------------------------------------------------------*/
//#define N 255
//uint16_t Samples[N];

/* Public macro -----------------------------------------------------------*/
/* Public variables -------------------------------------------------------*/
/* Public functions declaration -------------------------------------------*/
extern void SetAi(uint8_t ai, INPUT_TYPE type);
extern INPUT_TYPE NVMEM_Get_AI_Type(char idx);
/* Private typedef ---------------------------------------------------------*/
/* Private define ----------------------------------------------------------*/
//#define I (6) // Array to use whitin the Matrix
//#define M (3) // Array to use whitin the Matrix
#define NUM_AI 7 // 6 ai + batt monitor
#define MAX_RESISTANCE_VALUE    ((uint16_t)10000)
#define ANALOG_INPUT_6          (5)
/* Private macro -----------------------------------------------------------*/
/* Private variables -------------------------------------------------------*/
static uint32_t analog_value[NUM_AI] = {0,0,0,0,0,0,0};//current


/* Private functions declaration -------------------------------------------*/
uint32_t ADCtoBattMonitor(uint32_t channel_A, uint32_t channel_B);
void SetAnalogValue(uint32_t value, uint8_t idx);


/** @brief ADC Conversion
 *
 *  This function get the converted data from the ADC
 *
 *
 *  @param  None
 *
 *  @return None
 *
 */

/*This function is just for configurables inputs*/
void Analog_Inputs_Config(void)
{
	uint8_t idx = 0;

	for(idx=0;idx<=6;idx++)
	{
		SetAi(idx, NVMEM_Get_AI_Type(idx));
	}
}

/*This function is just for configurables inputs*/
void Analog_Inputs_Sleep(void)
{
	uint8_t idx = 0;
	for(idx=0;idx<=6;idx++)
	{
		SetAi(idx, INPUT_VOLTAGE_LOW);
	}
}

/*---------------------------------------------------------------------------
 Function:    	ADCtoResistance
 Description:	This function do the conversion for resistance configuration

 Inputs Parameters:		Uint32
 Return Parameters:		Uint32
---------------------------------------------------------------------------*/
uint32_t ADCtoResistance(uint32_t channel_A,uint32_t channel_B, uint8_t analog_input_index)
{
	uint32_t VA=0; //Variable A
	uint32_t VB=0; //Variable B
	uint32_t Rvalue=0;
	uint32_t rtn = 0;

	 if(ANALOG_INPUT_6 == analog_input_index)
	 {
	    VA = ( (channel_A << 4) + (channel_A >> 3) + ( channel_A >> 4) + (channel_A >> 6) );
	    VB = ( (channel_B << 4) + (channel_B >> 3) + ( channel_B >> 4) + (channel_B >> 6) );
	 }
	 else
	 {
    //4 shifting in Cnts of each channel to allow better resolution
    //(ChA<<4)*0.9879 // 0.9879 = (12K7+1M)/(25K1+1M)
        VA =(channel_A<<4)-(channel_A>>3)-(channel_A>>4)-(channel_A>>8)-(channel_A>>9)-(channel_A>>10)-(channel_A>>11);
        VB =channel_B<<4;
	}

    if(VB<VA)
    {
    	// evaluating real operation
    	VA -=VB;
    	Rvalue =(VB<<8)+(VB<<7)+(VB<<2)+(VB<<1); //(ChB<<4)*390
    	Rvalue /=VA;

    	if(Rvalue>MAX_RESISTANCE_VALUE)
    	{
    		// unstable values
    		Rvalue =  MAX_RESISTANCE_VALUE;
    	}
    	else
    	{
    		// real values
    	}

    }
    else if (VB > VA)
    {
    	Rvalue = MAX_RESISTANCE_VALUE;
    }
    else if (VB == 0)
    {
    	Rvalue = 0;
    }
    else
 	 {
    	Rvalue = MAX_RESISTANCE_VALUE;
 	 }

    rtn = Rvalue;

    return rtn;

}

/*---------------------------------------------------------------------------
 Function:    	ADCtoVoltageLow
 Description:	This function do the conversion for Voltage low configuration

 Inputs Parameters:		Uint32
 Return Parameters:		Uint32
---------------------------------------------------------------------------*/
uint32_t ADCtoVLow(uint32_t channel_A,uint32_t channel_B)
{

	float voltage_low = 0;
	//4 shifting in Cnts of each channel to allow better resolution Voltage Low
	//(ChB<<4)8.7842 // 8.7842 = (3300*(12K4+1K27)*(12K7+1M)) / (4095*1K27*1M)
	voltage_low = channel_A * 0.8059f;
	voltage_low = voltage_low * 1.8297f;
	return ((uint32_t)voltage_low);
}

/*---------------------------------------------------------------------------
 Function:     ADCtoVoltageHigh
 Description:  This function do the conversion for Voltage High configuration

 Inputs Parameters:		Uint32
 Return Parameters:		Uint32
---------------------------------------------------------------------------*/
uint32_t ADCtoVHigh (uint32_t channel_A,uint32_t channel_B)
{

	float voltage_high = 0;
	//4 shifting in Cnts of each channel to allow better resolution Voltage Low
	//(ChB<<4)8.7842 // 8.7842 = (3300*(12K4+1K27)*(12K7+1M)) / (4095*1K27*1M)
	voltage_high = channel_B * 0.8059f;
	voltage_high = voltage_high * 10.7637f;
	return ((uint32_t)voltage_high);
}

/*---------------------------------------------------------------------------
 Function:     ADCtoFrequency
 Description:  This function do the conversion for Voltage High configuration

 Inputs Parameters:		none
 Return Parameters:		Uint32
---------------------------------------------------------------------------*/
uint32_t ADCtoFrequency (uint8_t index)
{
	uint32_t freq=0;

	freq = FI_Get_CINx_Frequency(index);/*set the frequency value according index*/
  
	return freq;
  
}

/*---------------------------------------------------------------------------
 Function:     ADCtoCurrent
 Description:  This function do the conversion for Current configuration

 Inputs Parameters:		Uint32
 Return Parameters:		Uint32
---------------------------------------------------------------------------*/
uint32_t ADCtoCurrent(uint32_t channel_A,uint32_t channel_B)
{
	uint32_t VA=0;

	//4 shifting in Cnts of each channel to allow better resolution Voltage High
	VA =(channel_B<<7)+(channel_B<<1)+(channel_B>>1)+(channel_B>>4)+(channel_B>>7); //(ChA<<4)*1.4933 // 1.4933 = (3300*(390+470)*(12K7+1M)) / (4095*470*1M)
	VA = VA>>4;

	return VA;
}


/*---------------------------------------------------------------------------
 Function:     SetAnalogValue
 Description:  This function is used to set the updated value of the analog
 	 	 	   inputs

 Inputs Parameters:		void, Uint32
 Return Parameters:		Uint32
---------------------------------------------------------------------------*/
void SetAnalogValue(uint32_t value, uint8_t idx)
{
	uint32_t val;
	val = value;
	if(idx >NUM_AI) idx = NUM_AI;
	analog_value[idx] = val;
}

/*---------------------------------------------------------------------------
 Function:     GetAnalogValue
 Description:  This function is used to Get the updated value of the analog
 	 	 	   inputs

 Inputs Parameters:		Uint8
 Return Parameters:		Uint32
---------------------------------------------------------------------------*/
uint32_t GetAnalogValue(uint8_t idx)
{
	if(idx >NUM_AI) idx = NUM_AI;

	return analog_value[idx];
}

/*---------------------------------------------------------------------------
 Function:    	STB_convertion
 Description:	This function do the conversion for DIGITAL STB

 Inputs Parameters:		Uint32
 Return Parameters:		Uint32
---------------------------------------------------------------------------*/
uint32_t STB_convertion(uint32_t channel_A,uint32_t channel_B)
{
	uint8_t status=0;

	if( (channel_A<=100) && (channel_B<=100) )
	{
		status=0x00;
	}
	else if( (channel_A>200) && (channel_B>200) )
	{
		status=0x01;
	}
	else
	{
		// do nothing;
	}

	return status;
}

/*---------------------------------------------------------------------------
 Function:    	STB_convertion
 Description:	This function do the conversion for DIGITAL STG

 Inputs Parameters:		Uint32
 Return Parameters:		Uint32
---------------------------------------------------------------------------*/
uint32_t STG_convertion(uint32_t channel_A,uint32_t channel_B)
{
	uint8_t status=0;

	if( (channel_A>=3800) && (channel_B) )
	{
		status=0x00;
	}
	else if( (channel_A<3800) && (channel_B<2300) )
	{
		status=0x01;
	}
	else
	{
		// do nothing;
	}

	return status;
}

/*---------------------------------------------------------------------------
 Function:    	ADCtoBattMonitor BAT_IN
 Description:	This function do the conversion for battery system configuration

 Inputs Parameters:		Uint32
 Return Parameters:		Uint32
---------------------------------------------------------------------------*/
uint32_t ADCtoBattMonitor(uint32_t channel_A, uint32_t channel_B)
{
	uint32_t value;

	//value = channel_A *10.12;// resolution three decimals
	value = channel_A *9.99;// resolution three decimals

	return value;
}
#endif

/**************** (C) COPYRIGHT 2012 Maxima Technologies *******************/
/***************************** END OF FILE *********************************/
