/****************************************************************************
              Copyright (c) 2022 Maxima Technologies and Systems
                           All Rights Reserved

   File Name          : Camera_Platformservice.c
   Author             : Adrush
   Version            : V0.0.0.1
   Origination Date   : 25/02/2022
   Description        : C Source File.
   Environment        : Independent
   Processor:         : STM32H743IIT6

-----------------------------------------------------------------------------
   Description:

   This file contains the Camera platform service initialization

-----------------------------------------------------------------------------
	File Revision History:
-----------------------------------------------------------------------------
   Version             Date                 Programmer
   0.0.0.1             25/02/2022			Adrush

   Notes:              - Initial Version
-----------------------------------------------------------------------------
*****************************************************************************
THIS FILE CONTAINS INTELLECTUAL PROPERTY OF MAXIMA TECHNOLOGIES AND SYSTEMS.
UNAUTHORIZED REDISTRIBUTION OF THIS SOURCE CODE, IN WHOLE OR PART, WITHOUT
THE EXPRESS WRITTEN PERMISSION OF MAXIMA TECHNOLOGIES IS STRICTLY PROHIBITED.
****************************************************************************/

/* USER CODE BEGIN Includes */
#include "../Inc/common.h"
#include "../../Drivers/Maximatecc/inc/camera.h"
/* USER CODE END Includes */

/* USER CODE BEGIN Macros */
#if (SDK_SERVICE_CAMERA == PS_ENABLE)
extern pvt_variable pv;
uint8_t   SetStreamOn=0;

#if (SDK_SERVICE_PM == PS_ENABLE)
/** @brief Camera module deinit
 *
 *  This function will make the Camera RESET pin to low.
 *
 *  @param  None
 *
 *  @return  None
 *
 */
void Camera_deinit(void)
{
	/*Configure CAMERA Reset pin to low for the Power saving */
	BSP_CAMERA_PwrDown();
}

/** @brief Camera reinit
 *
 *  This function will make the Camera RESET pin to high.
 *
 *  @param  None
 *
 *  @return  None
 *
 */
void Camera_reinit(void)
{
	/*Configure CAMERA Reset pin to low for the Power saving */
	HAL_GPIO_WritePin(LCD_ENABLE_PORT, LCD_ENABLE_PIN, GPIO_PIN_SET);

	/* Initialize the Camera Interface */
	BSP_CAMERA_Init();

}
#endif	//SDK_SERVICE_PM

#if ((SDK_CAMERA_STOP_KEY1_KEYPAD == PS_ENABLE)  || \
     (SDK_CAMERA_STOP_KEY2_KEYPAD == PS_DISABLE) || \
     (SDK_CAMERA_STOP_KEY3_KEYPAD == PS_DISABLE) || \
	 (SDK_CAMERA_STOP_KEY4_KEYPAD == PS_DISABLE))
/** @brief Stop streaming camera
 *
 *  This function stops streaming the camera
 *
 *  @param  None
 *
 *  @return  None
 *
 */
void StopStreaming()
{
	/* Stopped the Camera streaming */
	if (DISPLAY_NONE != pv.cameramode)
		pv.stopstreaming = 1;
}

#endif

/** @brief Stop/Start Register function
 *
 *  This function gets the individual SDK platform service
 *  start/stop function and store it in the global variable.
 *
 *  @param  None
 *
 *  @return  0 : PS_SUCCESS
 *          -1 : PS_FAILURE
 *
 */

void StartStopSDKServiceReg(uint8_t (*cb)(uint8_t state))
{
	pv.stop_start_cb[pv.num_reg_cb++] = cb;
}

/** @brief Stop/Start SDK service
 *
 *  This function stops/starts the SDK Platform device thread.
 *
 *  @param  None
 *
 *  @return  0 : PS_SUCCESS
 *          -1 : PS_FAILURE
 *
 */

void StartStopSDKService(uint8_t state)
{
	uint8_t index;

	for (index = 0; index < pv.num_reg_cb; index++)
	{
		pv.stop_start_cb[index](state);
	}
}

#if 0
void StartStopSDKService(uint8_t state)
{
	uint8_t index;

	for (index = 0; index < pv.num_reg_cb++; index++)
	{
		pv.stop_start_cb[pv.num_reg_cb++](state);
	}
}
#endif

/* USER CODE END Macros */


/* USER CODE BEGIN Global Variables */


/** @brief Camera DB Initialization
 *
 *  This function takes the default configuration from the config header
 *  file and store the data into the DB.
 *
 *  @param  None
 *
 *  @return  0 : PS_SUCCESS
 *          -1 : PS_FAILURE
 *
 */
int8_t Camera_DB_Init(void)
{
	uint8_t  val8;
	uint16_t val16;

	/* Update the default configuration into the database */
	val8 = PS_CAMERA_MODE;
	Set_DL(CAMERA_VIDEO_MODE, (uint8_t *)&val8);

	val16 = PS_CAMERA_X0_DISP_ORIGIN;
	Set_DL(DISPLAY_X0_ORIGIN, (uint8_t *)&val16);

	val16 = PS_CAMERA_Y0_DISP_ORIGIN;
	Set_DL(DISPLAY_Y0_ORIGIN, (uint8_t *)&val16);
//NTSC
	val16 = PS_NTSC_CAMERA_X0_ORIGIN;
	Set_DL(NTSC_CAMERA_X0_ORIGIN, (uint8_t *)&val16);

	val16 = PS_NTSC_CAMERA_Y0_ORIGIN;
	Set_DL(NTSC_CAMERA_Y0_ORIGIN, (uint8_t *)&val16);

	val16 = (uint16_t)PS_NTSC_CAMERA_X0_WIDTH;
	Set_DL(NTSC_CAMERA_X0_WIDTH, (uint8_t *)&val16);

	val16 = (uint16_t) PS_NTSC_CAMERA_Y0_HEIGHT;
	Set_DL(NTSC_CAMERA_Y0_HEIGHT, (uint8_t *)&val16);
//PAL
	val16 = PS_PAL_CAMERA_X0_ORIGIN;
	Set_DL(PAL_CAMERA_X0_ORIGIN, (uint8_t *)&val16);

	val16 = PS_PAL_CAMERA_Y0_ORIGIN;
	Set_DL(PAL_CAMERA_Y0_ORIGIN, (uint8_t *)&val16);

	val16 = (uint16_t)PS_PAL_CAMERA_X0_WIDTH;
	Set_DL(PAL_CAMERA_X0_WIDTH, (uint8_t *)&val16);

	val16 = (uint16_t) PS_PAL_CAMERA_Y0_HEIGHT;
	Set_DL(PAL_CAMERA_Y0_HEIGHT, (uint8_t *)&val16);

	/* Initialize the Camera Interface */
	BSP_CAMERA_Init();

	val8 = CAMERA_STREAM_OFF;
	Set_DL(CAMERA_VIDEO_STREAM, (uint8_t *)&val8);

	return PS_SUCCESS;
}

void Camera_Set_Stream_ON(void)
{
	SetStreamOn=1;
}

void Set_LCD_RGB(uint32_t rgb,uint16_t skip_pix)
{
  uint32_t  *ptrLcd;
  ptrLcd = (uint32_t*)(LAYER_1_FRAMEBUFFER);
  uint32_t size = 0;
  size = LCD_RESOlUTION_X0*LCD_RESOlUTION_Y0*4;
  for (uint32_t i=0; i<(size); i+=skip_pix) /*480*272*2  x*y*bpp*/
  {
    ptrLcd[i]=rgb;
  }
}

void CM_Delay(uint32_t step)
{
  uint32_t max_cycles = 2000000;
  for (uint32_t i=0; i<(max_cycles); i+=step);
}

/** @brief Camera Platform service task
 *
 *  This function starts the Camera Platform Service
 *  task.
 *
 *  @param  pvParameters : Task Private structure
 *
 *  @return none
 *
 */
void Camera_ServiceStart(void *pvParameters)
{
	uint32_t  cNotifiedValue;
	uint8_t   *camerabuf;
	uint8_t   res;
	uint8_t   func;
	uint8_t   cam_state;

#if (SDK_SERVICE_PM == PS_ENABLE)
	/* Register the Power Mode callback */
	PM_Register_Callback(Camera_deinit,NULL,Camera_reinit);
#endif
	Set_DL(DISPLAY_X0_ORIGIN, (uint8_t *)&uiLCDOffset_X);
	Set_DL(DISPLAY_Y0_ORIGIN, (uint8_t *)&uiLCDOffset_Y);

	while(1)
	{

#if (SDK_SERVICE_WATCHDOG == PS_ENABLE)
		/* Ping the watchdog timer */
		res = SetWD(WD_PING_CAMERA_TASK);

		if(res != SUCCESS)
		{
			LOGERROR("Camera Task failed to ping the WD Timer\r\n");
		}
#endif
		/* Wait for receiving the Event from the SDK API */
		res = xTaskNotifyWait( CLEAR_BITS_ONENTRY, ULONG_MAX , &cNotifiedValue, PS_CAMERA_TASK_TIMEOUT);

		if(res != pdTRUE)
		{
#if ((SDK_CAMERA_STOP_KEY1_KEYPAD == PS_ENABLE)  || \
     (SDK_CAMERA_STOP_KEY2_KEYPAD == PS_DISABLE) || \
     (SDK_CAMERA_STOP_KEY3_KEYPAD == PS_DISABLE) || \
	 (SDK_CAMERA_STOP_KEY4_KEYPAD == PS_DISABLE))
	 	 	if ((1 == pv.stopstreaming) &&
	 	 			(DISPLAY_NONE != pv.cameramode))
			{
				/* Stop the Platform service thread */
				//StartStopSDKService(STATE_START);
                ReStart();
				/* Stop the Camera Streaming */
				BSP_CAMERA_Stop();
				HAL_LTDC_SetAlpha(pv.hltdc, 0, 1);
				pv.cameramode = DISPLAY_NONE;

				res = CAMERA_STREAM_OFF;
				Set_DL(CAMERA_VIDEO_STREAM, (uint8_t *)&res);
			}
			pv.stopstreaming = 0;
#endif
			continue;
		}

		if (cNotifiedValue & (1<<CAMERA_MSG_EVENT))
		{
				/* Receive Message from the Camera SDK API */
				res = xQueueReceive(pv.cameratxq, &camerabuf , ( TickType_t ) Q_TICKSTOWAIT );

				if(res != pdTRUE)
				{
					break;
				}

				func = camerabuf[CAMERA_FUNC_INDEX];

				/* Update the result */
				camerabuf[CAMERA_RESULT_INDEX] = SUCCESS;

				/* Stop the Camera Streaming */
				BSP_CAMERA_Stop();

				cam_state = camerabuf[STREAM_MODE+1];

				if (SetStreamOn)
					{
					   cam_state = CAMERA_STREAM_ON;
					   SetStreamOn=0;
					}

				/* Camera Stream is ON */
				if (CAMERA_STREAM_ON == cam_state)
				{
					//VideoStandardID();
					HAL_NVIC_DisableIRQ(LTDC_IRQn); //TurnOff TouchGFX
					Get_DL(CAMERA_VIDEO_FLIP_VERTICAL, &bLCD_Hor_Flip);
					Get_DL(CAMERA_VIDEO_FLIP_HORIZONTAL, &bLCD_Ver_Flip);
					switch(func)
					{
						case FULL_SCREEN_ON:
						case RESIZE_TO_FULL_SCREEN_ON:
							/* Clear the Framebuffer */
							memset((void *)LAYER_1_FRAMEBUFFER, 0xFF, (LCD_RESOlUTION_X0*LCD_RESOlUTION_Y0*4));

							/* Get all the parameteres */
							memcpy(&pv.cX0_Origin,&camerabuf[CAMERA_X_ORG_INDEX+1], sizeof(uint16_t));
							memcpy(&pv.cY0_Origin,&camerabuf[CAMERA_Y_ORG_INDEX+1], sizeof(uint16_t));
							memcpy(&pv.X_W0, &camerabuf[CAMERA_WIDTH_INDEX+1], sizeof(uint16_t));
							memcpy(&pv.Y_H0,&camerabuf[CAMERA_HEIGHT_INDEX+1], sizeof(uint16_t));
							memcpy(&pv.LCD_X0_Origin,&camerabuf[DISPLAY_X_ORG_INDEX+1], sizeof(uint16_t));
							memcpy(&pv.LCD_Y0_Origin,&camerabuf[DISPLAY_Y_ORG_INDEX+1], sizeof(uint16_t));
							/* Set the Flip mode */
							memcpy(&pv.flip_mode,&camerabuf[DISPLAY_FLIP+1], sizeof(uint8_t));

							pv.cameramode = RESIZE_IMAGE_TO_FULLSCREEN;

							BSP_LCD_LayerDefaultInit(1, LAYER_1_FRAMEBUFFER, 1,
							pv.LCD_X0_Origin, pv.LCD_Y0_Origin, LCD_RESOlUTION_X0, LCD_RESOlUTION_Y0);
							CropCameraImageResizetoLCDSize(pv.cX0_Origin, pv.cY0_Origin,
															pv.X_W0, pv.Y_H0,
															pv.LCD_X0_Origin, pv.LCD_Y0_Origin);
														/* Set the Alpha blending */
							HAL_LTDC_SetAlpha(pv.hltdc, 255, 1);

							break;

						case DISPLAY_AS_IT_IS_ON:
							/* Clear the Framebuffer */
							memset((void *)LAYER_1_FRAMEBUFFER, 0xFF, (LCD_RESOlUTION_X0*LCD_RESOlUTION_Y0*4));

							/* Get all the parameteres */
							memcpy(&pv.cX0_Origin,&camerabuf[CAMERA_X_ORG_INDEX+1], sizeof(uint16_t));
							memcpy(&pv.cY0_Origin,&camerabuf[CAMERA_Y_ORG_INDEX+1], sizeof(uint16_t));
							memcpy(&pv.X_W0, &camerabuf[CAMERA_WIDTH_INDEX+1], sizeof(uint16_t));
							memcpy(&pv.Y_H0,&camerabuf[CAMERA_HEIGHT_INDEX+1], sizeof(uint16_t));
							memcpy(&pv.LCD_X0_Origin,&camerabuf[DISPLAY_X_ORG_INDEX+1], sizeof(uint16_t));
							memcpy(&pv.LCD_Y0_Origin,&camerabuf[DISPLAY_Y_ORG_INDEX+1], sizeof(uint16_t));
							/* Set the Flip mode */
							memcpy(&pv.flip_mode,&camerabuf[DISPLAY_FLIP+1], sizeof(uint8_t));
							pv.cameramode = DISPLAY_CROPPED_IMAGE;
							BSP_LCD_LayerDefaultInit(1, LAYER_1_FRAMEBUFFER, 1,
									pv.LCD_X0_Origin, pv.LCD_Y0_Origin, LCD_RESOlUTION_X0, LCD_RESOlUTION_Y0);

							CropCameraImageResizetoLCDSize(pv.cX0_Origin, pv.cY0_Origin,
															pv.X_W0, pv.Y_H0,
															pv.LCD_X0_Origin, pv.LCD_Y0_Origin);

							/* Set the Alpha blending */
							HAL_LTDC_SetAlpha(pv.hltdc, 255, 1);
							break;
						default:
							camerabuf[CAMERA_RESULT_INDEX] = FAILURE;
							break;
					}
				}
				else if (CAMERA_STREAM_OFF == cam_state)
				{
					if (DISPLAY_NONE != pv.cameramode)
					{
						/* Stop the Camera Streaming */
						BSP_CAMERA_Stop();
						HAL_LTDC_SetAlpha(pv.hltdc, 0, 1);
					}
					pv.cameramode = DISPLAY_NONE;
					HAL_NVIC_EnableIRQ(LTDC_IRQn);

				}
				else
				{
					camerabuf[CAMERA_RESULT_INDEX] = FAILURE;
				}
				/* Send the response to the SDK API */
				xQueueSend( pv.camerarxq, ( void * )&camerabuf, ( TickType_t ) Q_TICKSTOWAIT );
		}
	}
}

/** @brief Start Camera Platform service
 *
 *  This function initializes the following modules
 *     ===> Camera DB
 *     ===> Camera platform driver
 *     ===> Camera software resources
 *     ===> Start the Camera service
 *
 *  @param  None
 *
 *  @return 0 : PS_SUCCESS
 *          1 : PS_FAILURE
 *
 */
int8_t Camera_PS_Start(void)
{
	BaseType_t xReturned;

	/* Create Camera TxQ for receiving value from the SDK API */
	pv.cameratxq = xQueueCreate( CAMERA_RX_TX_MAX_Q_SIZE, sizeof(uint32_t*));
	if(NULL == pv.cameratxq)
	{
		LOGERROR("Unable to initialize the Camera TxQ\r\n");
		return PS_FAILURE;
	}

	/* Create Camera RxQ for sending return value to the SDK API */
	pv.camerarxq = xQueueCreate( CAMERA_RX_TX_MAX_Q_SIZE, sizeof(uint32_t*));
	if(NULL == pv.camerarxq)
	{
		LOGERROR("Unable to initialize the Camera RxQ\r\n");
		return PS_FAILURE;
	}

	/* Start the Camera Platform service thread */
	xReturned = xTaskCreate(Camera_ServiceStart, "Camera",
			512, NULL, PS_CAMERA_TASK_PRIORITY,   &pv.camera_task);

	if (xReturned != pdPASS)
	{
		LOGERROR("Unable to initialize the Camera Thread\r\n");
		return PS_FAILURE;
	}

	LOGERROR("Camera Platform Service task created Successfully \r\n");

	return PS_SUCCESS;
}
#endif	//SDK_SERVICE_CAMERA

/**************** (C) COPYRIGHT 2022 Maxima Technologies *******************/
/***************************** END OF FILE *********************************/
