/****************************************************************************
              Copyright (c) 2022 Maxima Technologies and Systems
                           All Rights Reserved

   File Name          : ConfigurableInput_platformservice.c
   Author             : Zaid
   Version            : V0.0.0.1
   Origination Date   : 04/02/2022
   Description        : C Source File.
   Environment        : Independent
   Processor:         : STM32H743IIT6

-----------------------------------------------------------------------------
   Description:

   This file contains the ConfigurableInput platform service initialization

-----------------------------------------------------------------------------
	File Revision History:
-----------------------------------------------------------------------------
   Version             Date                 Programmer
   0.0.0.1             04/02/2022           Zaid

   Notes:              - Initial Version
-----------------------------------------------------------------------------
*****************************************************************************
THIS FILE CONTAINS INTELLECTUAL PROPERTY OF MAXIMA TECHNOLOGIES AND SYSTEMS.
UNAUTHORIZED REDISTRIBUTION OF THIS SOURCE CODE, IN WHOLE OR PART, WITHOUT
THE EXPRESS WRITTEN PERMISSION OF MAXIMA TECHNOLOGIES IS STRICTLY PROHIBITED.
****************************************************************************/

/* USER CODE BEGIN Includes */
#include "../Inc/common.h"

/* USER CODE END Includes */

/* USER CODE BEGIN Macros */

/* USER CODE END Macros */

#if (SDK_SERVICE_CFG_INPUT == PS_ENABLE)
/* USER CODE BEGIN Global Variables */

extern pvt_variable pv;


/*!
 * Configuration Input Default configuration
 */
ci_conf_param ci_conf[MAX_NUM_CFG_INPUT]=
{
    {CFG_INPUT_01_TYPE, CFG_INPUT_01_NUMB_SAMPLES,
    		CONF_INPUT_TYPE_01, CONF_INPUT_01_NUMB_SAMPLES, 0, ANLG_MAP_01_A, ANLG_MAP_01_B},
	{CFG_INPUT_02_TYPE, CFG_INPUT_02_NUMB_SAMPLES,
			CONF_INPUT_TYPE_02, CONF_INPUT_02_NUMB_SAMPLES, 0, ANLG_MAP_02_A, ANLG_MAP_02_B},
	{CFG_INPUT_03_TYPE, CFG_INPUT_03_NUMB_SAMPLES,
			CONF_INPUT_TYPE_03, CONF_INPUT_03_NUMB_SAMPLES, 0, ANLG_MAP_03_A, ANLG_MAP_03_B},
	{CFG_INPUT_04_TYPE, CFG_INPUT_04_NUMB_SAMPLES,
			CONF_INPUT_TYPE_04, CONF_INPUT_04_NUMB_SAMPLES, 0, ANLG_MAP_04_A, ANLG_MAP_04_B},
	{CFG_INPUT_05_TYPE, CFG_INPUT_05_NUMB_SAMPLES,
			CONF_INPUT_TYPE_05, CONF_INPUT_05_NUMB_SAMPLES, 0, ANLG_MAP_05_A, ANLG_MAP_05_B},
	{CFG_INPUT_06_TYPE, CFG_INPUT_06_NUMB_SAMPLES,
			CONF_INPUT_TYPE_06, CONF_INPUT_06_NUMB_SAMPLES, 0, ANLG_MAP_06_A, ANLG_MAP_06_B},
};


#if (SDK_SERVICE_PM == PS_ENABLE)
/** @brief Configuration Input deinit
 *
 *  This function will make the CI pins to low.
 *
 *  @param  None
 *
 *  @return  None
 *
 */
void CI_deinit(void)
{
	uint8_t index;
#if 1
	/* Make the IO pins to LOW */
    for (index = 0; index < MAX_NUM_CFG_INPUT; index++)
    {
		/* Configure IO pins to lows */
		SetAI(pv.hi2c, index, EXP_IO_POWER_DOWN);
    }
#endif
}

/** @brief Configuration Input reinit
 *
 *  This function will make the CI pins to reinit state.
 *
 *  @param  None
 *
 *  @return  None
 *
 */
void CI_reinit(void)
{
#if 1
	uint8_t index;
	/* Make the IO pins to LOW */
    for (index = 0; index < MAX_NUM_CFG_INPUT; index++)
    {
		/* Configure IO pins to lows */
		SetAI(pv.hi2c, index, ci_conf[index].type);
    }
#endif
}
#endif

/** @brief ConfigurableInput DB Initialization
 *
 *  This function takes the default configuration from the config header
 *  file and store the data into the DB.
 *
 *  @param  None
 *
 *  @return None
 *
 */
void CI_DB_Init(void)
{
	uint8_t  index;

	/* Set the default configuration into DB */
	for (index = 0; index < MAX_NUM_CFG_INPUT; index++)
	{
		/* Check the boundary value of Input */
		if (!((ci_conf[index].type > START_CI_INPUT) &&
				(ci_conf[index].type < END_CI_INPUT)))
			ci_conf[index].type = CI_INPUT_OFF;

		/* Set the input type into the DB */
		Set_DL(ci_conf[index].db_type_index, &ci_conf[index].type);

		/* Configure the input into the hardware */
		SetAI(pv.hi2c, index, ci_conf[index].type);

		if (!((ci_conf[index].num_sample >= MIN_NUM_SAMPLE) &&
				(ci_conf[index].num_sample <= MAX_NUM_SAMPLE)))
			ci_conf[index].num_sample = DEFAULT_NUM_SAMPLES;

		/* Set the Number of samples into the DB */
		Set_DL(ci_conf[index].db_sample_index, &ci_conf[index].num_sample);
	}

    for (index = 0; index < MAX_NUM_CFG_INPUT; index++)
    {
    	pv.channelA_value[index] = 0xFFFF;
    	pv.channelB_value[index] = 0xFFFF;
    }
}

/** @brief CI Read the sample
 *
 *  This function reads the data from the ADC and
 *  update into the DB.
 *
 *  @param  None
 *
 *  @return None
 *
 */
int8_t CI_Read_Sample(void)
{
	uint8_t index;
    uint32_t converted_ADC_value = 0;
    uint32_t u8_start_ci_idx = 0;
    uint32_t u16_start_ci_idx = 0;
    uint32_t u32_start_ci_idx = 0;
#if ((SDK_CAMERA_STOP_CI_STG == CONF_NONE) && (SDK_CAMERA_STOP_CI_STB != CONF_NONE))
	uint8_t stb;
#endif

#if ((SDK_CAMERA_STOP_CI_STG != CONF_NONE) && (SDK_CAMERA_STOP_CI_STB == CONF_NONE))
	uint8_t stg;
#endif

    for (index = 0; index < MAX_NUM_CFG_INPUT; index++)
    {
    	ci_conf[index].elapse_count++;

        /*	Get ADC_Results for Section A1	*/
    	if (0xFFFF == pv.channelA_value[index])
    	{
    		pv.channelA_value[index] = ((uint16_t)GetADC_Results(ci_conf[index].AChanADC));
    	}
    	else
    	{
    		pv.channelA_value[index] = (pv.channelA_value[index] +
    				((uint16_t)GetADC_Results(ci_conf[index].AChanADC)))/2;
    	}

        /*	Get ADC_Results for Section A2	*/
    	if (0xFFFF == pv.channelB_value[index])
    	{
            pv.channelB_value[index] = ((uint16_t)GetADC_Results(ci_conf[index].BChanADC));
    	}
    	else
    	{
    		pv.channelB_value[index] = (pv.channelB_value[index] +
    				((uint16_t)GetADC_Results(ci_conf[index].BChanADC)))/2;
    	}

        if (ci_conf[index].num_sample == ci_conf[index].elapse_count)
        {
        	/* Read the data */
            switch(ci_conf[index].type)
            {

            	case CI_INPUT_CURRENT:
            		converted_ADC_value = ADCtoCurrent(pv.channelA_value[index], pv.channelB_value[index]);

            		Set_DL(CFG_INPUT_06_CURRENT, (uint8_t *)&converted_ADC_value);
            		break;
#if 0
            	case CI_INPUT_VOLTAGE_LOW:
            		converted_ADC_value = ADCtoVLow(pv.channelA_value[index], pv.channelB_value[index]);

            		u16_start_ci_idx = CFG_INPUT_01_VOLTAGE_3_3V;
            		u16_start_ci_idx += (index * 2);
            		Set_DL(u16_start_ci_idx, (uint8_t *)&converted_ADC_value);
            		break;
#endif
            	case CI_INPUT_RESISTANCE:
            		converted_ADC_value = ADCtoResistance(pv.channelA_value[index], pv.channelB_value[index], index);

            		//u16_start_ci_idx = CFG_INPUT_01_VOLTAGE_3_3V;
            		u16_start_ci_idx = CFG_INPUT_01_RESISTANCE;
            		//u16_start_ci_idx += (index * 2) + 1;
            		u16_start_ci_idx += index;
            		Set_DL(u16_start_ci_idx, (uint8_t *)&converted_ADC_value);
            		break;

            	case CI_DIGITAL_STG:
            		converted_ADC_value = STG_convertion(pv.channelA_value[index], pv.channelB_value[index]);

            		u8_start_ci_idx = CFG_INPUT_01_DIGITAL_STG;
            		u8_start_ci_idx += (index * 4);
            		Set_DL(u8_start_ci_idx, (uint8_t *)&converted_ADC_value);
            		break;

            	case CI_DIGITAL_STB:
            		converted_ADC_value = STB_convertion(pv.channelA_value[index], pv.channelB_value[index]);

            		u8_start_ci_idx = CFG_INPUT_01_DIGITAL_STG;
            		u8_start_ci_idx += (index * 4) + 1;
            		Set_DL(u8_start_ci_idx, (uint8_t *)&converted_ADC_value);
            		break;

            	case CI_INPUT_VOLTAGE_HIGH:
            		converted_ADC_value = ADCtoVHigh(pv.channelA_value[index], pv.channelB_value[index]);

            		u16_start_ci_idx = CFG_INPUT_01_VOLTAGE_32V;
            		u16_start_ci_idx += (index * 2);
            		Set_DL(u16_start_ci_idx, (uint8_t *)&converted_ADC_value);
            		break;

            	case CI_INPUT_VOLTAGE_LOW_6V:
            		converted_ADC_value = ADCtoVLow(pv.channelA_value[index], pv.channelB_value[index]);

            		u16_start_ci_idx = CFG_INPUT_01_VOLTAGE_32V;
            		u16_start_ci_idx += (index * 2) + 1;
            		Set_DL(u16_start_ci_idx, (uint8_t *)&converted_ADC_value);
            		break;

            	default:
            		break;
            }

        	pv.channelA_value[index] = 0xFFFF;
        	pv.channelB_value[index] = 0xFFFF;

            /* Re-Initialize the elapse count */
           	ci_conf[index].elapse_count = 0;
        }

        if (CI_INPUT_FREQUENCY == ci_conf[index].type)
        {
			converted_ADC_value = ADCtoFrequency(index);

			u32_start_ci_idx = CFG_INPUT_01_FREQUENCY;
			u32_start_ci_idx += index;
			Set_DL(u32_start_ci_idx, (uint8_t *)&converted_ADC_value);
        }
    }

#if ((SDK_CAMERA_STOP_KEY1_KEYPAD == PS_DISABLE) &&  \
     (SDK_CAMERA_STOP_KEY2_KEYPAD == PS_DISABLE) &&  \
	 (SDK_CAMERA_STOP_KEY3_KEYPAD == PS_DISABLE) &&  \
	 (SDK_CAMERA_STOP_KEY4_KEYPAD == PS_DISABLE))

    /* Camera ON / OFF functionality */
#if ((SDK_CAMERA_STOP_CI_STG != CONF_NONE) && (SDK_CAMERA_STOP_CI_STB == CONF_NONE))
    converted_ADC_value = 0;
	u8_start_ci_idx = CFG_INPUT_01_DIGITAL_STG;
	u8_start_ci_idx += (SDK_CAMERA_STOP_CI_STG * 4);
	Get_DL(u8_start_ci_idx, (uint8_t *)&stg);

	/* Camera is ON */
	if (TRUE == stg)
	{
		/* Check whether the camera is ON or NOT */
		Get_DL(CAMERA_VIDEO_STREAM, (uint8_t *)&stg);
		if ( CAMERA_STREAM_OFF == stg)
		{
			/* Camera if is off, start the streaming */
			stg = CAMERA_STREAM_ON;
			Set_DL(CAMERA_VIDEO_STREAM, (uint8_t *)&stg);
		}
	}

#endif

#if ((SDK_CAMERA_STOP_CI_STG == CONF_NONE) && (SDK_CAMERA_STOP_CI_STB != CONF_NONE))
	converted_ADC_value = 0;
	u8_start_ci_idx = CFG_INPUT_01_DIGITAL_STB;
	u8_start_ci_idx += (SDK_CAMERA_STOP_CI_STB * 4);
	Get_DL(u8_start_ci_idx, (uint8_t *)&stb);

	/* Camera is ON */
	if (1 == stb)
	{
		stb = 0;
		/* Check whether the camera is ON or NOT */
		Get_DL(CAMERA_VIDEO_STREAM, (uint8_t *)&stb);
		if ( CAMERA_STREAM_OFF == stb)
		{
			/* Camera if is off, start the streaming */
			stb = CAMERA_STREAM_ON;
			Set_DL(CAMERA_VIDEO_STREAM, (uint8_t *)&stb);
		}
	}
#endif
#endif
	return PS_SUCCESS;
}
/** @CI Check Zero Frequency Input
 *
 *  This function check monitor flags to set frequency to zero when no input frequency is feeding by each input
 *
 *  @param  None
 *
 *  @return None
 *
 */
static void CI_Zero_Mon_Freq_Input(void)
{
	static bool setMonFlagFreq=false;
	//CIN1
	if (CONF_INPUT_TYPE_01== CI_INPUT_FREQUENCY)
		{ if (setMonFlagFreq)
			  FI_Mon_CIN1_Frequency();
		  else
		  {
			  if (FI_Chk_CIN1_Frequency()!=1) FI_Set_CIN1_Frequency(0);
		  }
		}
	//CIN2
		if (CONF_INPUT_TYPE_02== CI_INPUT_FREQUENCY)
			{ if (setMonFlagFreq)
				  FI_Mon_CIN2_Frequency();
			  else
			  {
				  if (FI_Chk_CIN2_Frequency()!=1) FI_Set_CIN2_Frequency(0);
			  }
			}
	//CIN3
		if (CONF_INPUT_TYPE_03== CI_INPUT_FREQUENCY)
			{ if (setMonFlagFreq)
				 FI_Mon_CIN3_Frequency();
			  else
			  {
				  if (FI_Chk_CIN3_Frequency()!=1) FI_Set_CIN3_Frequency(0);
			  }
			}
    //CIN4
		if (CONF_INPUT_TYPE_04== CI_INPUT_FREQUENCY)
			{ if (setMonFlagFreq)
				FI_Mon_CIN4_Frequency();
			  else
			  {
				  if (FI_Chk_CIN4_Frequency()!=1) FI_Set_CIN4_Frequency(0);
			  }
			}
	//CIN5
	if (CONF_INPUT_TYPE_05== CI_INPUT_FREQUENCY)
	{ if (setMonFlagFreq)
		  FI_Mon_CIN5_Frequency();
	  else
	  {
		  if (FI_Chk_CIN5_Frequency()!=1) FI_Set_CIN5_Frequency(0);
	  }
	}
	//CIN6
	if (CONF_INPUT_TYPE_06== CI_INPUT_FREQUENCY)
	{ if (setMonFlagFreq)
		  FI_Mon_CIN6_Frequency();
	  else
	  {
		 if (FI_Chk_CIN6_Frequency()!=1) FI_Set_CIN6_Frequency(0);
	  }
	}
	setMonFlagFreq=!setMonFlagFreq;
}

/** @brief CI Platform service task
 *
 *  This function starts the CI Platform Service
 *  task.
 *
 *  @param  None
 *
 *  @return 0 : PS_SUCCESS
 *          1 : PS_FAILURE
 *
 */
static void CI_ServiceStart(void *pvParameters)
{
	uint32_t cNotifiedValue;
	uint8_t *cibuf;
	uint8_t res;
	uint8_t func;

	/* STart the timer for the CI update */
	xTimerStart(pv.xTimers[ PS_CI_TIMER ], 0);
#if (SDK_SERVICE_PM == PS_ENABLE)
	/* Register the Power Mode callback */
	PM_Register_Callback(CI_deinit,NULL,CI_reinit);
#endif

	while(1)
	{

#if (SDK_SERVICE_WATCHDOG == PS_ENABLE)
		/* Ping the watchdog timer */
		res = SetWD(WD_PING_CI_TASK);

		if(res != SUCCESS)
		{
			LOGERROR("CI Task failed to ping the WD Timer\r\n");
		}
#endif

		/* Wait for receiving the Event from the SDK API and CI timer */
		res = xTaskNotifyWait( CLEAR_BITS_ONENTRY, ULONG_MAX , &cNotifiedValue, PS_CFG_INPUT_TASK_TIMEOUT);

		if(res != pdTRUE)
		{
			continue;
		}

		/* Event from SDK Configuration Input API */
		if (cNotifiedValue & (1<<CI_TMR_EVENT))
		{
			/* read and update the configurable data to DB */
			CI_Read_Sample();
			CI_Zero_Mon_Freq_Input();
		}

		if (cNotifiedValue & (1<<CI_MSG_EVENT))
		{
				/* Receive Message from the CI SDK API */
				res = xQueueReceive(pv.citxq, &cibuf , ( TickType_t ) Q_TICKSTOWAIT );
				if(res != pdTRUE)
				{
					break;
				}

				func = cibuf[CI_FUNC_INDEX];

				/* Update the result */
				cibuf[CI_RESULT_INDEX] = SUCCESS;

				switch(func)
				{
					case CI_SET:
						/* Validate the channel parameter */
						if (!((cibuf[CI_TYPE_INDEX] >= MIN_NUM_CFG_INPUT) &&
								(cibuf[CI_TYPE_INDEX] <= MAX_NUM_CFG_INPUT)))
						{
							/* Update the result */
							cibuf[CI_RESULT_INDEX] = FAILURE;
							break;
						}

						/* Process the function parameter */
						switch(cibuf[CI_SET_ID_INDEX])
						{
							case CI_SET_INPUT_TYPE:
								/* Validate type parameter */
								if (!((cibuf[CI_VALUE_INDEX] > START_CI_INPUT) &&
										(cibuf[CI_VALUE_INDEX] < END_CI_INPUT)))
								{
									/* Update the result */
									cibuf[CI_RESULT_INDEX] = FAILURE;
									break;
								}

								/* CI 6 did not support freq, validate this parameter */
								if (((cibuf[CI_TYPE_INDEX] == MAX_NUM_CFG_INPUT) &&
										(cibuf[CI_VALUE_INDEX] == CI_INPUT_FREQUENCY)))
								{
									/* Update the result */
									cibuf[CI_RESULT_INDEX] = FAILURE;
									break;
								}
								//TakeI2CBusLock();
								/* Configure the input into the hardware */
								SetAI(pv.hi2c, (cibuf[CI_TYPE_INDEX] - 1), cibuf[CI_VALUE_INDEX]);
								//GiveI2CBusLock();

								/* Update into the configuration structure */
								ci_conf[(cibuf[CI_TYPE_INDEX] - 1)].type = cibuf[CI_VALUE_INDEX];
								break;

							case CI_SET_NUM_SAMPLE:
								/* Update into the configuration structure */
								ci_conf[cibuf[CI_TYPE_INDEX] - 1].num_sample = cibuf[CI_VALUE_INDEX];

								/* Reinitialize the elapse counter */
								ci_conf[cibuf[CI_TYPE_INDEX] - 1].elapse_count = 0x00;
								break;

							default:
								/* Update the result */
								cibuf[CI_RESULT_INDEX] = FAILURE;
								break;
						}
						break;

					case CI_GET:
						/* Process the function parameter */
						switch(cibuf[CI_GET_ID_INDEX])
						{
							case CI_GET_INPUT_TYPE:
								cibuf[CI_GET_ID_INDEX] = ci_conf[cibuf[CI_TYPE_INDEX]].type;
								break;

							case CI_GET_NUM_SAMPLE:
								cibuf[CI_GET_ID_INDEX] = ci_conf[cibuf[CI_TYPE_INDEX]].num_sample;
								break;

							default:
								/* Update the result */
								cibuf[CI_RESULT_INDEX] = FAILURE;
								break;
						}
						break;

					default:
						/* Update the result */
						cibuf[CI_RESULT_INDEX] = FAILURE;
						break;
				}

				/* Send the response to the SDK API */
				xQueueSend( pv.cirxq, ( void * )&cibuf, ( TickType_t ) Q_TICKSTOWAIT );
		}
		/* Clear the Notify Value */
		cNotifiedValue = 0;
	}
}

/** @brief Configurable Input Timer Callback
 *
 *  This function is used for the Configurable Input Sampling time
 *  functionality
 *
 *  @param  xTimer : Configurable Input timer handle
 *
 *  @return 0 : PS_SUCCESS
 *          1 : PS_FAILURE
 *
 */
void CITimerCallback( TimerHandle_t xTimer )
{
	/* Notify to the CI task with CI timer callback */
	xTaskNotify( pv.ci_task, (1<<CI_TMR_EVENT), eSetBits );
}

/** @brief Start CI Platform service
 *
 *  This function initializes the following modules
 *     ===> CI DB
 *     ===> CI platform driver
 *     ===> CI software resources
 *     ===> Start the CI service
 *  the CI platform service layer.
 *
 *  @param  None
 *
 *  @return 0 : PS_SUCCESS
 *          1 : PS_FAILURE
 *
 */
int8_t CI_PS_Start(void)
{
	BaseType_t xReturned;

	/* Create the configurable input timer */
	pv.xTimers[ PS_CI_TIMER ] = xTimerCreate
	                   ( "CI_Timer",
	                	 PS_CI_TIMEOUT,
	                     pdTRUE,
	                     ( void * ) 0,
	                     CITimerCallback
	                   );

	if (NULL == pv.xTimers[ PS_CI_TIMER ])
	{
		LOGERROR("Unable to initialize the ConfigurableInput timer\r\n");
		return PS_FAILURE;
	}

	/* Create CI TxQ for receiving value from the SDK API */
	pv.citxq = xQueueCreate( CI_RX_TX_MAX_Q_SIZE, sizeof(uint32_t*));
	if(NULL == pv.citxq)
	{
		LOGERROR("Unable to initialize the ConfigurableInput\r\n");
		return PS_FAILURE;
	}

	/* Create CI RxQ for sending return value to the SDK API */
	pv.cirxq = xQueueCreate( CI_RX_TX_MAX_Q_SIZE, sizeof(uint32_t*));
	if(NULL == pv.cirxq)
	{
		LOGERROR("Unable to initialize the ConfigurableInput\r\n");
		return PS_FAILURE;
	}

	xReturned = xTaskCreate(CI_ServiceStart, "CfgInput",
			CI_TASK_SIZE, NULL, PS_CFG_INPUT_TASK_PRIORITY,   &pv.ci_task);

	/* Start the Configurable Input Platform service thread */
	if (xReturned != pdPASS)
	{
		LOGERROR("Unable to initialize the ConfigurableInput\r\n");
		return PS_FAILURE;
	}

	LOGERROR("ConfigurableInput Platform Service task created Successfully \r\n");

	return PS_SUCCESS;
}

#endif

/**************** (C) COPYRIGHT 2022 Maxima Technologies *******************/
/***************************** END OF FILE *********************************/
