/****************************************************************************
              Copyright (c) 2022 Maxima Technologies and Systems
                           All Rights Reserved

   File Name          : DL_Eeprom_Validation.c
   Author             : Rajesh
   Version            : V0.0.0.1
   Origination Date   : 21/01/2022
   Description        : C Source File.
   Environment        : Independent
   Processor:         : STM32H743IIT6

-----------------------------------------------------------------------------
   Description:

   This file contains the Data Layer EEPROM field validation function

-----------------------------------------------------------------------------
	File Revision History:
-----------------------------------------------------------------------------
   Version             Date                 Programmer
   0.0.0.1             21/01/2022           Rajesh

   Notes:              - Initial Version
-----------------------------------------------------------------------------
*****************************************************************************
THIS FILE CONTAINS INTELLECTUAL PROPERTY OF MAXIMA TECHNOLOGIES AND SYSTEMS.
UNAUTHORIZED REDISTRIBUTION OF THIS SOURCE CODE, IN WHOLE OR PART, WITHOUT
THE EXPRESS WRITTEN PERMISSION OF MAXIMA TECHNOLOGIES IS STRICTLY PROHIBITED.
****************************************************************************/

/* USER CODE BEGIN Includes */
#include "../../Maximatecc/Inc/Data_Layer.h"
#include "../Inc/Common.h"
#include "../Inc/Dl_Eeprom_Validation.h"
/* USER CODE END Includes */

/* USER CODE BEGIN Global Variables */
int8_t (*dl_eeprom_func[1])(uint16_t, uint8_t*);

extern pvt_variable pv;
/* USER CODE END Global Variables */

/** @brief DB Validation function
 *
 *  This function initialize the DB Validation function for the each
 *  unsinged 8 bits field.
 *
 *  @param  None
 *
 *  @return None
 *
 */
void DB_EEPROM_Field_Validation_Reg(void)
{
	/* Clear the Callback memory */
	memset (&dl_eeprom_func[0], 0x00, sizeof(dl_eeprom_func));

#if (SDK_SERVICE_EEPROM == PS_ENABLE)

    /* EE_PLACEHOLDER_X Validation */
	dl_eeprom_func[0] = validate_placeholder;
#endif

}

#if (SDK_SERVICE_EEPROM == PS_ENABLE)
/** @brief EEPROM Placeholder field validation
 *
 *  This function validates the EEPROM Placeholder and send the data
 *  to platform service.
 *
 *  @param  placeholder [IN]  : EEPROM PlaceHolder
 *          eeprom_data  [IN] : EEPROM data
 *
 *  @return  0 : FAILURE
 *  		 1 : SUCCESS
 *
 */
int8_t validate_placeholder(uint16_t placeholder, uint8_t *eeprom_data)
{
	int8_t res;

	if (!((EE_MAX_PH >= placeholder) && (EE_MIN_PH <= placeholder)))
		return FAILURE;

	res = SetERM(placeholder, eeprom_data);

	return res;
}

/** @brief Get the EEPROM Placeholder data
 *
 *  This function gets the EEPROM data from the
 *  EEPROM Placeholder and send the data
 *  to platform service.
 *
 *  @param  placeholder [IN]  : EEPROM PlaceHolder
 *          eeprom_data  [IN] : EEPROM data
 *
 *  @return  0 : FAILURE
 *  		 1 : SUCCESS
 *
 */
int8_t Get_placeholderdata(uint16_t placeholder, uint8_t *eeprom_data)
{
	int8_t res;

	if (!((EE_MAX_PH >= placeholder) && (EE_MIN_PH <= placeholder)))
		return FAILURE;

	res = GetERM(placeholder, eeprom_data);

	return res;
}
#endif	//SDK_SERVICE_EEPROM

/**************** (C) COPYRIGHT 2022 Maxima Technologies *******************/
/***************************** END OF FILE *********************************/
