/****************************************************************************
              Copyright (c) 2021 Maxima Technologies and Systems
                           All Rights Reserved

   File Name          : DL_float_Validation.c
   Author             : Rajesh
   Version            : V0.0.0.1
   Origination Date   : 01/01/2022
   Description        : C Source File.
   Environment        : Independent
   Processor:         : STM32H743IIT6

-----------------------------------------------------------------------------
   Description:

   This file contains the Data Layer float field validation function

-----------------------------------------------------------------------------
	File Revision History:
-----------------------------------------------------------------------------
   Version             Date                 Programmer
   0.0.0.0              04/01/2022          Rajesh

   Notes:              - Initial Version
-----------------------------------------------------------------------------
*****************************************************************************
THIS FILE CONTAINS INTELLECTUAL PROPERTY OF MAXIMA TECHNOLOGIES AND SYSTEMS.
UNAUTHORIZED REDISTRIBUTION OF THIS SOURCE CODE, IN WHOLE OR PART, WITHOUT
THE EXPRESS WRITTEN PERMISSION OF MAXIMA TECHNOLOGIES IS STRICTLY PROHIBITED.
****************************************************************************/

/* USER CODE BEGIN Includes */
#include "../Inc/Common.h"
#include "../Inc/Data_Layer.h"
#include "../Inc/Dl_Float_Validation.h"
/* USER CODE END Includes */

/* USER CODE BEGIN Macros */


/* USER CODE END Macros */


/* USER CODE BEGIN Global Variables */
int8_t (*dl_float_func[(END_FLOAT    -1)  - START_FLOAT     ])(uint16_t, float);

extern pvt_variable pv;
/* USER CODE END Global Variables */

/** @brief DB Validation function
 *
 *  This function initialize the DB Validation function for the each
 *  float bits field.
 *
 *  @param  None
 *
 *  @return None
 *
 */
void DB_float_Field_Validation_Reg(void)
{
	/* Clear the Callback memory */
	memset (&dl_float_func[0], 0x00, ((END_FLOAT    -1) - START_FLOAT));

#if (SDK_SERVICE_POWER_MONITOR == PS_ENABLE)

	/* THERMOSTAT_LEVEL Validation */
	dl_float_func[THERMOSTAT_LEVEL - (START_FLOAT + 1)] = validate_thermostat_level;

#else

	/* THERMOSTAT_LEVEL_DUMMY Validation */
	dl_float_func[THERMOSTAT_LEVEL_DUMMY - (START_FLOAT + 1)] = validate_float_dummy_field;

#endif	//SDK_SERVICE_POWER_MONITOR
}

#if (SDK_SERVICE_POWER_MONITOR == PS_ENABLE)

/** @brief validation of thermostat level
 *
 *  This function validates the thermostat level and
 *  sends the thermostat level value to platform service.
 *
 *  @param  thermostat_idx [IN]   : Thermostat DB Field
 *          thermostat_level [IN] : Thermostat Level
 *
 *  @return  1 : SUCCESS
 *           0 : FAILURE
 *
 */
int8_t validate_thermostat_level(uint16_t thermostat_idx, float thermostat_level)
{
	/* Battery level value */
	if (!((TEMP_SENSOR_MIN_VAL <= thermostat_level) &&
			(TEMP_SENSOR_MAX_VAL >= thermostat_level)))
		return FAILURE;

	return SUCCESS;
}

#endif

/** @brief float Dummy Field validation
 *
 *  This function validates the dummy field in the DB.
 *
 *  @param  dummy1 [IN] : Not used
 *          dummy2 [IN] : Not used
 *
 *  @return 0 : FAILURE
 *
 */
int8_t validate_float_dummy_field(uint16_t dummy1, float dummy2)
{
	return FAILURE;
}


/**************** (C) COPYRIGHT 2022 Maxima Technologies *******************/
/***************************** END OF FILE *********************************/
