/****************************************************************************
              Copyright (c) 2022 Maxima Technologies and Systems
                           All Rights Reserved

   File Name          : Watchdog_Platformservice.c
   Author             : Zaid
   Version            : V0.0.0.1
   Origination Date   : 04/02/2022
   Description        : C Source File.
   Environment        : Independent
   Processor:         : STM32H743IIT6

-----------------------------------------------------------------------------
   Description:

   This file contains the Watchdog platform service initialization

-----------------------------------------------------------------------------
	File Revision History:
-----------------------------------------------------------------------------
   Version             Date                 Programmer
   0.0.0.1             04/02/2022           Zaid

   Notes:              - Initial Version
-----------------------------------------------------------------------------
*****************************************************************************
THIS FILE CONTAINS INTELLECTUAL PROPERTY OF MAXIMA TECHNOLOGIES AND SYSTEMS.
UNAUTHORIZED REDISTRIBUTION OF THIS SOURCE CODE, IN WHOLE OR PART, WITHOUT
THE EXPRESS WRITTEN PERMISSION OF MAXIMA TECHNOLOGIES IS STRICTLY PROHIBITED.
****************************************************************************/

/* USER CODE BEGIN Includes */
#include "../Inc/Common.h"
/* USER CODE END Includes */

#if (SDK_SERVICE_WATCHDOG == PS_ENABLE)
extern pvt_variable pv;
WatchDog_info WD_conf[MAX_NUMBER_OF_WD];


/** @brief WatchDog Refresh
 *
 *  This function will Refresh the WatchDog timer.
 *
 *  @param  None
 *
 *  @return None
 *
 */
void watchdog_refresh(void)
{
	for(uint8_t index = 0; index < MAX_NUMBER_OF_WD; index++)
	{
		/* Check for the WD Enable */
		if (FALSE == WD_conf[index].isenabled)
            continue;

		/* Increment reset count */
		WD_conf[index].reset_count++;

		if(MAX_RESET_COUNT == WD_conf[index].reset_count)
		{
			/* if reset_count reaches max count stop feeding the WatchDog */
			pv.WatchDog_reset = STOP_RESET_WATCHDOG;
		}
	}

	if(STOP_RESET_WATCHDOG != pv.WatchDog_reset)
	{
		/* Refresh the IWDG */
		HAL_IWDG_Refresh(pv.iwdg);
	}

}

/** @brief Watchdog DB Initialization
 *
 *  This function takes the default configuration from the config header
 *  file and store the data into the DB.
 *
 *  @param  None
 *
 *  @return 0 : PS_SUCCESS
 *         -1 : PS_FAILURE
 *
 */
int8_t Watchdog_DB_Init(void)
{
	uint8_t index;

	/* Clear all the Watchdog values */
	for(index = 0; index < MAX_NUMBER_OF_WD; index++)
	{
		WD_conf[index].reset_count = 0;
		WD_conf[index].isenabled = TRUE;
	}

#if (SDK_SERVICE_WARNING_LIGHT == PS_DISABLE)
	WD_conf[WD_PING_WL_TASK].isenabled = FALSE;
#endif

#if (SDK_SERVICE_RTC == PS_DISABLE)
	WD_conf[WD_PING_RTC_TASK].isenabled = FALSE;
#endif

#if (SDK_SERVICE_DIGITAL_OUTPUT == PS_DISABLE)
	WD_conf[WD_PING_DIO_TASK].isenabled = FALSE;
#endif

#if (SDK_SERVICE_LIGHT_SENSOR == PS_DISABLE)
	WD_conf[WD_PING_LS_TASK].isenabled = FALSE;
#endif

#if (SDK_SERVICE_LED == PS_DISABLE)
	WD_conf[WD_PING_LED_TASK].isenabled = FALSE;
#endif

#if (SDK_SERVICE_CFG_INPUT == PS_DISABLE)
	WD_conf[WD_PING_CI_TASK].isenabled = FALSE;
#endif

#if (SDK_SERVICE_LCD == PS_DISABLE)
	WD_conf[WD_PING_LCD_TASK].isenabled = FALSE;
#endif

#if (SDK_SERVICE_PM == PS_DISABLE)
	WD_conf[WD_PING_POWERMODE_TASK].isenabled = FALSE;
#endif

#if (SDK_SERVICE_POWER_MONITOR == PS_DISABLE)
	WD_conf[WD_PING_POWERMONITOR_TASK].isenabled = FALSE;
#endif

#if (SDK_SERVICE_EEPROM == PS_DISABLE)
	WD_conf[WD_PING_EE_TASK].isenabled = FALSE;
#endif

#if (SDK_SERVICE_SWTIMER == PS_DISABLE)
	WD_conf[WD_PING_SWTIMER_TASK].isenabled = FALSE;
#endif

#if (SDK_SERVICE_BLE == PS_DISABLE)
	WD_conf[WD_PING_BLE_TASK].isenabled = FALSE;
#endif

#if (SDK_SERVICE_USB == PS_DISABLE)
	WD_conf[WD_PING_USB_TASK].isenabled = FALSE;
#endif

#if (SDK_SERVICE_CAMERA == PS_DISABLE)
	WD_conf[WD_PING_CAMERA_TASK].isenabled = FALSE;
#endif

	index = WD_PING_CAMERA_TASK + 1;
	/* Check for the USER Defined WD configuration */
#if (USER_TASK_WD0 == PS_DISABLE)
	WD_conf[index].isenabled = FALSE;
#endif
	index++;

#if (USER_TASK_WD1 == PS_DISABLE)
	WD_conf[index].isenabled = FALSE;
#endif
	index++;

#if (USER_TASK_WD2 == PS_DISABLE)
	WD_conf[index].isenabled = FALSE;
#endif
	index++;

#if (USER_TASK_WD3 == PS_DISABLE)
	WD_conf[index].isenabled = FALSE;
#endif
	index++;

#if (USER_TASK_WD4 == PS_DISABLE)
	WD_conf[index].isenabled = FALSE;
#endif
	index++;

#if (USER_TASK_WD5 == PS_DISABLE)
	WD_conf[index].isenabled = FALSE;
#endif
	index++;

#if (USER_TASK_WD6 == PS_DISABLE)
	WD_conf[index].isenabled = FALSE;
#endif
	index++;

#if (USER_TASK_WD7 == PS_DISABLE)
	WD_conf[index].isenabled = FALSE;
#endif
	index++;

#if (USER_TASK_WD8 == PS_DISABLE)
	WD_conf[index].isenabled = FALSE;
#endif
	index++;

#if (USER_TASK_WD9 == PS_DISABLE)
	WD_conf[index].isenabled = FALSE;
#endif

	return PS_SUCCESS;
}
/** @brief Watchdog Platform service task
 *
 *  This function starts the Watchdog Platform Service
 *  task.
 *
 *  @param  pvParameters - Private data pointer
 *
 *  @return none
 *
 */
void Watchdog_ServiceStart(void *pvParameters)
{
	uint8_t *watchdogbuf;
	uint8_t res = 0;
	uint32_t cNotifiedValue;
	uint8_t  func;

	/* Start the Watchdog reset timer */
	xTimerStart( pv.xTimers[ PS_WD_TIMER ], 0 );

	while(1)
	{

		/* Wait for receiving the Event from the SDK API */
		res = xTaskNotifyWait( CLEAR_BITS_ONENTRY, ULONG_MAX , &cNotifiedValue, PS_WD_TASK_TIMEOUT);

		if(res != pdTRUE)
		{
			continue;
		}

		if (cNotifiedValue & (1<<WD_TMR_EVENT))
		{
			/* WatchDog Refresh */
			watchdog_refresh();
		}

		if (cNotifiedValue & (1<<WD_MSG_EVENT))
		{
			while(1)
			{
				/* Receive Message from the WatchDog SDK API */
				res = xQueueReceive(pv.wdtxq, &watchdogbuf , ( TickType_t ) 20 );

				if(res != pdTRUE)
				{
					break;
				}

				func = watchdogbuf[WD_FUNC_INDEX];

				/* Update the result */
				watchdogbuf[WD_RESULT_INDEX] = SUCCESS;

				switch(func)
				{
					case WD_SET:
						/* update the reset count value */
						WD_conf[watchdogbuf[WD_FEED_ID_INDEX]].reset_count = 0;
						break;

					default:
						watchdogbuf[WD_RESULT_INDEX] = FAILURE;
						break;
				}

				/* Send the response to the SDK API */
				xQueueSend( pv.wdrxq, ( void * )&watchdogbuf, ( TickType_t ) 10 );
			}
		}
	}
}

/** @brief Watchdog Timer Callback
 *
 *  This function is used for the resetting the Watchdog
 *  timer for all the registered tasks
 *
 *  @param  xTimer : Watchdog timer handle
 *
 *  @return none
 *
 */
void WDTimerCallback( TimerHandle_t xTimer )
{
	/* Notify to the WD task to blink the WD */
	xTaskNotify( pv.wd_task, (1<<WD_TMR_EVENT), eSetBits );
}

/** @brief Start Watchdog Platform service
 *
 *  This function initializes the following modules
 *     ===> Watchdog DB
 *     ===> Watchdog platform driver
 *     ===> Watchdog software resources
 *     ===> Start the Watchdog service
 *
 *  @param  None
 *
 *  @return 0 : PS_SUCCESS
 *         -1 : PS_FAILURE
 *
 */
int8_t Watchdog_PS_Start(void)
{
	BaseType_t xReturned;

	/* Create WD TxQ for receiving value from the SDK API */
	pv.wdtxq = xQueueCreate( WD_RX_TX_MAX_Q_SIZE, sizeof(uint32_t*));
	if(NULL == pv.wdtxq)
	{
		LOGERROR("Unable to initialize the Watchdog TxQ \r\n");
		return PS_FAILURE;
	}

	/* Create WD RxQ for sending return value to the SDK API */
	pv.wdrxq = xQueueCreate( WD_RX_TX_MAX_Q_SIZE, sizeof(uint32_t*));
	if(NULL == pv.wdrxq)
	{
		LOGERROR("Unable to initialize the Watchdog RxQ \r\n");
		return PS_FAILURE;
	}

	/* Create the WatchDog timer for the WD Feeding */
	pv.xTimers[ PS_WD_TIMER ] = xTimerCreate
	                   ( "WD_Timer",
	                	 pdMS_TO_TICKS(PS_WD_RESET_TIMER),
	                     pdTRUE,
	                     ( void * ) 0,
	                     WDTimerCallback
	                   );

	if (NULL == pv.xTimers[ PS_WD_TIMER ])
	{
		LOGERROR("Unable to initialize the Warning Light timer\r\n");
		return PS_FAILURE;
	}

	/* Create the Watchdog task */
	xReturned = xTaskCreate(Watchdog_ServiceStart, "Watchdog",
			WD_TASK_SIZE, NULL, PS_WD_TASK_PRIORITY,   &pv.wd_task);

    if( pdPASS != xReturned)
    {
		LOGERROR("Unable to initialize the Watchdog Task\r\n");
		return PS_FAILURE;
    }

    LOGERROR("Watchdog Platform Service task created Successfully \r\n");

	return PS_SUCCESS;
}

#endif	//SDK_SERVICE_WATCHDOG

/**************** (C) COPYRIGHT 2022 Maxima Technologies *******************/
/***************************** END OF FILE *********************************/
