/****************************************************************************
              Copyright (c) 2022 Maxima Technologies and Systems
                           All Rights Reserved

   File Name          : AI430_Config.h
   Author             : Rajesh
   Version            : V0.0.0.1
   Origination Date   : 21/01/2022
   Description        : Header File.
   Environment        : Independent
   Processor:         : STM32H743IIT6

-----------------------------------------------------------------------------
   Description:

   This file contains the AI430 board configuration macros

-----------------------------------------------------------------------------
	File Revision History:
-----------------------------------------------------------------------------
   Version             Date                 Programmer
   0.0.0.1              21/01/2022          Rajesh

   Notes:              - Initial Version
-----------------------------------------------------------------------------
*****************************************************************************
THIS FILE CONTAINS INTELLECTUAL PROPERTY OF MAXIMA TECHNOLOGIES AND SYSTEMS.
UNAUTHORIZED REDISTRIBUTION OF THIS SOURCE CODE, IN WHOLE OR PART, WITHOUT
THE EXPRESS WRITTEN PERMISSION OF MAXIMA TECHNOLOGIES IS STRICTLY PROHIBITED.
****************************************************************************/

#ifndef _AI430_CONFIG_H_
#define	_AI430_CONFIG_H_

/* ########################## Module Selection ############################## */
/**
  * @brief USER has to change only these for enabling and disabling
  *        Platform services software components. And also the USER can
  *        configure the Platform service driver configuration.
  */


/*****************************************************************************
 *
 *                      Warning Light Module Configuration
 *
 *****************************************************************************/

/*!
 * Warning light Platform service Enable(PS_ENABLE) / Disable(PS_DISABLE) Macros
 */
#define SDK_SERVICE_WARNING_LIGHT                PS_ENABLE

#if (SDK_SERVICE_WARNING_LIGHT == PS_ENABLE)

/*!
 * Warning Light Task Periodicity	100ms
 */
#define PS_WL_TASK_TIMEOUT   					 100

/*!
 * Warning Light Task Priority
 * osPriorityNone          =  0,
 * osPriorityIdle          =  1,
 * osPriorityLow           =  8,
 * osPriorityLow1          =  8+1,
 * 		,,					  ,,
 * 		,,					  ,,
 * osPriorityISR           = 56,
 * osPriorityError         = -1,
 * osPriorityReserved      = 0x7FFFFFFF
 */
#define PS_WL_TASK_PRIORITY   					 osPriorityIdle

/*!
 * Maximum Number of Warning Light required for this application
 *
 * MACOR Supported
 *
 * MAX_NUM_WARNING_LIGHT  : This hardware support maximum of 20 Warning lights
 *                          USER can choose between 0 to 20
 *
 * WARNING_LIGHT_FREQ WL_kHz3_FREQ / WL_kHz22_FREQ
 *
 * CONF_WARNING_LIGHT_xx_ENABLE             PS_ENABLE
 *                                          PS_DISABLE
 *
 * CONF_WARNING_LIGHT_xx_CURRENT            WL_OUT_CURRENT_MAX
 *                                          WL_OUT_CURRENT_MAX_2
 *                                          WL_OUT_CURRENT_MAX_4
 *                                          WL_OUT_CURRENT_MAX_8
 *
 * CONF_WARNING_LIGHT_xx_POWERON_STATE      WL_CONF_OFF
 *                                          WL_CONF_ON
 *
 * CONF_WARNING_LIGHT_xx_PWM_DC             <0 - 100>
 *
 * CONF_WARNING_LIGHT_xx_BLINKING_MS        <0-65535>
 *
 *
 */

/* Private defines -----------------------------------------------------------*/
/* USER CODE BEGIN Private defines */
#define MAX_NUM_WARNING_LIGHT                    20
#define WARNING_LIGHT_FREQ                       WL_kHz3_FREQ

#define CONF_WARNING_LIGHT_01_ENABLE             PS_ENABLE
#define CONF_WARNING_LIGHT_01_CURRENT            WL_OUT_CURRENT_MAX
#define CONF_WARNING_LIGHT_01_POWERON_STATE      WL_CONF_OFF
#define CONF_WARNING_LIGHT_01_PWM_DC             100
#define CONF_WARNING_LIGHT_01_BLINKING_MS        0

#define CONF_WARNING_LIGHT_02_ENABLE             PS_ENABLE
#define CONF_WARNING_LIGHT_02_CURRENT            WL_OUT_CURRENT_MAX
#define CONF_WARNING_LIGHT_02_POWERON_STATE      WL_CONF_OFF
#define CONF_WARNING_LIGHT_02_PWM_DC             100
#define CONF_WARNING_LIGHT_02_BLINKING_MS        0

#define CONF_WARNING_LIGHT_03_ENABLE             PS_ENABLE
#define CONF_WARNING_LIGHT_03_CURRENT            WL_OUT_CURRENT_MAX
#define CONF_WARNING_LIGHT_03_POWERON_STATE      WL_CONF_OFF
#define CONF_WARNING_LIGHT_03_PWM_DC             100
#define CONF_WARNING_LIGHT_03_BLINKING_MS        0

#define CONF_WARNING_LIGHT_04_ENABLE             PS_ENABLE
#define CONF_WARNING_LIGHT_04_CURRENT            WL_OUT_CURRENT_MAX
#define CONF_WARNING_LIGHT_04_POWERON_STATE      WL_CONF_OFF
#define CONF_WARNING_LIGHT_04_PWM_DC             100
#define CONF_WARNING_LIGHT_04_BLINKING_MS        0

#define CONF_WARNING_LIGHT_05_ENABLE             PS_ENABLE
#define CONF_WARNING_LIGHT_05_CURRENT            WL_OUT_CURRENT_MAX
#define CONF_WARNING_LIGHT_05_POWERON_STATE      WL_CONF_OFF
#define CONF_WARNING_LIGHT_05_PWM_DC             100
#define CONF_WARNING_LIGHT_05_BLINKING_MS        0

#define CONF_WARNING_LIGHT_06_ENABLE             PS_ENABLE
#define CONF_WARNING_LIGHT_06_CURRENT            WL_OUT_CURRENT_MAX
#define CONF_WARNING_LIGHT_06_POWERON_STATE      WL_CONF_OFF
#define CONF_WARNING_LIGHT_06_PWM_DC             100
#define CONF_WARNING_LIGHT_06_BLINKING_MS        0

#define CONF_WARNING_LIGHT_07_ENABLE             PS_ENABLE
#define CONF_WARNING_LIGHT_07_CURRENT            WL_OUT_CURRENT_MAX
#define CONF_WARNING_LIGHT_07_POWERON_STATE      WL_CONF_OFF
#define CONF_WARNING_LIGHT_07_PWM_DC             100
#define CONF_WARNING_LIGHT_07_BLINKING_MS        0

#define CONF_WARNING_LIGHT_08_ENABLE             PS_ENABLE
#define CONF_WARNING_LIGHT_08_CURRENT            WL_OUT_CURRENT_MAX_2
#define CONF_WARNING_LIGHT_08_POWERON_STATE      WL_CONF_OFF
#define CONF_WARNING_LIGHT_08_PWM_DC             100
#define CONF_WARNING_LIGHT_08_BLINKING_MS        0

#define CONF_WARNING_LIGHT_09_ENABLE             PS_ENABLE
#define CONF_WARNING_LIGHT_09_CURRENT            WL_OUT_CURRENT_MAX_4
#define CONF_WARNING_LIGHT_09_POWERON_STATE      WL_CONF_OFF
#define CONF_WARNING_LIGHT_09_PWM_DC             100
#define CONF_WARNING_LIGHT_09_BLINKING_MS        0

#define CONF_WARNING_LIGHT_10_ENABLE             PS_ENABLE
#define CONF_WARNING_LIGHT_10_CURRENT            WL_OUT_CURRENT_MAX_8
#define CONF_WARNING_LIGHT_10_POWERON_STATE      WL_CONF_OFF
#define CONF_WARNING_LIGHT_10_PWM_DC             100
#define CONF_WARNING_LIGHT_10_BLINKING_MS        0

#define CONF_WARNING_LIGHT_11_ENABLE             PS_ENABLE
#define CONF_WARNING_LIGHT_11_CURRENT            WL_OUT_CURRENT_MAX
#define CONF_WARNING_LIGHT_11_POWERON_STATE      WL_CONF_OFF
#define CONF_WARNING_LIGHT_11_PWM_DC             100
#define CONF_WARNING_LIGHT_11_BLINKING_MS        0

#define CONF_WARNING_LIGHT_12_ENABLE             PS_ENABLE
#define CONF_WARNING_LIGHT_12_CURRENT            WL_OUT_CURRENT_MAX
#define CONF_WARNING_LIGHT_12_POWERON_STATE      WL_CONF_OFF
#define CONF_WARNING_LIGHT_12_PWM_DC             100
#define CONF_WARNING_LIGHT_12_BLINKING_MS        0

#define CONF_WARNING_LIGHT_13_ENABLE             PS_ENABLE
#define CONF_WARNING_LIGHT_13_CURRENT            WL_OUT_CURRENT_MAX
#define CONF_WARNING_LIGHT_13_POWERON_STATE      WL_CONF_OFF
#define CONF_WARNING_LIGHT_13_PWM_DC             100
#define CONF_WARNING_LIGHT_13_BLINKING_MS        0

#define CONF_WARNING_LIGHT_14_ENABLE             PS_ENABLE
#define CONF_WARNING_LIGHT_14_CURRENT            WL_OUT_CURRENT_MAX
#define CONF_WARNING_LIGHT_14_POWERON_STATE      WL_CONF_OFF
#define CONF_WARNING_LIGHT_14_PWM_DC             100
#define CONF_WARNING_LIGHT_14_BLINKING_MS        0

#define CONF_WARNING_LIGHT_15_ENABLE             PS_ENABLE
#define CONF_WARNING_LIGHT_15_CURRENT            WL_OUT_CURRENT_MAX
#define CONF_WARNING_LIGHT_15_POWERON_STATE      WL_CONF_OFF
#define CONF_WARNING_LIGHT_15_PWM_DC             100
#define CONF_WARNING_LIGHT_15_BLINKING_MS        0

#define CONF_WARNING_LIGHT_16_ENABLE             PS_ENABLE
#define CONF_WARNING_LIGHT_16_CURRENT            WL_OUT_CURRENT_MAX
#define CONF_WARNING_LIGHT_16_POWERON_STATE      WL_CONF_OFF
#define CONF_WARNING_LIGHT_16_PWM_DC             100
#define CONF_WARNING_LIGHT_16_BLINKING_MS        0

#define CONF_WARNING_LIGHT_17_ENABLE             PS_ENABLE
#define CONF_WARNING_LIGHT_17_CURRENT            WL_OUT_CURRENT_MAX
#define CONF_WARNING_LIGHT_17_POWERON_STATE      WL_CONF_OFF
#define CONF_WARNING_LIGHT_17_PWM_DC             100
#define CONF_WARNING_LIGHT_17_BLINKING_MS        0

#define CONF_WARNING_LIGHT_18_ENABLE             PS_ENABLE
#define CONF_WARNING_LIGHT_18_CURRENT            WL_OUT_CURRENT_MAX
#define CONF_WARNING_LIGHT_18_POWERON_STATE      WL_CONF_OFF
#define CONF_WARNING_LIGHT_18_PWM_DC             100
#define CONF_WARNING_LIGHT_18_BLINKING_MS        0

#define CONF_WARNING_LIGHT_19_ENABLE             PS_ENABLE
#define CONF_WARNING_LIGHT_19_CURRENT            WL_OUT_CURRENT_MAX
#define CONF_WARNING_LIGHT_19_POWERON_STATE      WL_CONF_OFF
#define CONF_WARNING_LIGHT_19_PWM_DC             100
#define CONF_WARNING_LIGHT_19_BLINKING_MS        0

#define CONF_WARNING_LIGHT_20_ENABLE             PS_ENABLE
#define CONF_WARNING_LIGHT_20_CURRENT            WL_OUT_CURRENT_MAX
#define CONF_WARNING_LIGHT_20_POWERON_STATE      WL_CONF_OFF
#define CONF_WARNING_LIGHT_20_PWM_DC             100
#define CONF_WARNING_LIGHT_20_BLINKING_MS        0
/* USER CODE END Private defines */

#endif	//SDK_SERVICE_WARNING_LIGHT


/*****************************************************************************
 *
 *                      Keypad Module Configuration
 *
 *****************************************************************************/

/*!
 * Keypad Platform service Enable(PS_ENABLE) / Disable(PS_DISABLE) Macros
 */
#define SDK_SERVICE_KEYPAD           			 PS_ENABLE

#if (SDK_SERVICE_KEYPAD == PS_ENABLE)

/*!
 * Keypad Task Priority
 * osPriorityNone          =  0,
 * osPriorityIdle          =  1,
 * osPriorityLow           =  8,
 * osPriorityLow1          =  8+1,
 * 		,,					  ,,
 * 		,,					  ,,
 * osPriorityISR           = 56,
 * osPriorityError         = -1,
 * osPriorityReserved      = 0x7FFFFFFF
 */
#define PS_KPD_TASK_PRIORITY   					 osPriorityIdle

/*!
 * MACRO Supported
 *
 * Keypad backlight configuration state
 * 1: KEY_BACKLIGHT_ON /
 * 0: KEY_BACKLIGHT_OFF
 */
#define KEYPAD_BACKLIGHT_CFG_STATE				 KEY_BACKLIGHT_ON

/*!
 * SHORT Press timeout (Millisecond)
 */
#define SHORT_PRESS_TIMEOUT         10

/*!
 * MACOR Supported
 *
 * Keypad configuration state
 * 1: PS_ENABLE /
 * 0: PS_DISABLE
 */
#define CONF_KEYPAD_01_STATE            		 PS_ENABLE
#define CONF_KEYPAD_02_STATE            		 PS_ENABLE
#define CONF_KEYPAD_03_STATE            		 PS_ENABLE
#define CONF_KEYPAD_04_STATE            		 PS_ENABLE

#endif	//SDK_SERVICE_KEYPAD

/*****************************************************************************
 *
 *                      RTC Module Configuration
 *
 *****************************************************************************/

/*!
 * RTC Platform service Enable(PS_ENABLE) / Disable(PS_DISABLE) Macros
 */
#define SDK_SERVICE_RTC           				 PS_ENABLE

#if (SDK_SERVICE_RTC == PS_ENABLE)

/*!
 * RTC Task Periodicity	100ms
 */
#define PS_RTC_TASK_TIMEOUT         			 100

/*!
 * RTC Task Priority
 * osPriorityNone          =  0,
 * osPriorityIdle          =  1,
 * osPriorityLow           =  8,
 * osPriorityLow1          =  8+1,
 * 		,,					  ,,
 * 		,,					  ,,
 * osPriorityISR           = 56,
 * osPriorityError         = -1,
 * osPriorityReserved      = 0x7FFFFFFF
 */
#define PS_RTC_TASK_PRIORITY   					 osPriorityIdle

#endif	//SDK_SERVICE_RTC


/*****************************************************************************
 *
 *                      Digital Output Module Configuration
 *
 *****************************************************************************/

/*!
 * DIO Platform service Enable(PS_ENABLE) / Disable(PS_DISABLE) Macros
 */
#define SDK_SERVICE_DIGITAL_OUTPUT           	 PS_ENABLE

#if (SDK_SERVICE_DIGITAL_OUTPUT == PS_ENABLE)

/*!
 * DIO Task Periodicity	100ms
 */
#define PS_DIO_TASK_TIMEOUT         			 100

/*!
 * DIO Task Priority
 * osPriorityNone          =  0,
 * osPriorityIdle          =  1,
 * osPriorityLow           =  8,
 * osPriorityLow1          =  8+1,
 * 		,,					  ,,
 * 		,,					  ,,
 * osPriorityISR           = 56,
 * osPriorityError         = -1,
 * osPriorityReserved      = 0x7FFFFFFF
 */
#define PS_DIO_TASK_PRIORITY   					 osPriorityIdle

/*!
 * Select the DIGITAL_OUTPUT_CFG 00 : CONF_LOW_SIDE_DRIVER
 *                               01 : CONF_HIGH_SIDE_DRIVER
 *                               02 : CONF_OPEN_DRIVE_DRIVER
 */
#define DIGITAL_OUTPUT_CFG     					 CONF_OPEN_DRIVE_DRIVER

#endif	//SDK_SERVICE_DIGITAL_OUTPUT


/*****************************************************************************
 *
 *                      Light Sensor Module Configuration
 *
 *****************************************************************************/

/*!
 * Light Sensor Platform service Enable(PS_ENABLE) / Disable(PS_DISABLE) Macros
 */
#define SDK_SERVICE_LIGHT_SENSOR      			 PS_DISABLE//PS_ENABLE

#if (SDK_SERVICE_LIGHT_SENSOR == PS_ENABLE)

/*!
 * Light Sensor Task Periodicity		100ms
 */
#define PS_LS_TASK_TIMEOUT        				 100

/*!
 * light sensor Task Priority
 * osPriorityNone          =  0,
 * osPriorityIdle          =  1,
 * osPriorityLow           =  8,
 * osPriorityLow1          =  8+1,
 * 		,,					  ,,
 * 		,,					  ,,
 * osPriorityISR           = 56,
 * osPriorityError         = -1,
 * osPriorityReserved      = 0x7FFFFFFF
 */
#define PS_LS_TASK_PRIORITY   					 osPriorityIdle

/*!
 * MACOR Supported
 *
 * Light Sensor conversion time
 * 0: LS_CONVERSION_TIME_100 /
 * 1: LS_CONVERSION_TIME_800
 */
#define CONF_LIGHT_SENSOR_CONVERSION_TIME        LS_CONVERSION_TIME_100

/*!
 * MACOR Supported
 *
 * Light Sensor conversion mode
 * 0: LS_SHUTDOWN /
 * 1: LS_SINGLE_MODE_CONV/
 * 2: LS_CONTINUOUS_MODE_CONV
 */
#define CONF_LIGHT_SENSOR_CONVERSION_MODE        LS_CONTINUOUS_MODE_CONV
#endif	//SDK_SERVICE_LIGHT_SENSOR


/*****************************************************************************
 *
 *                      LED Configuration
 *
 *****************************************************************************/

/*!
 * LED Platform service Enable(PS_ENABLE) / Disable(PS_DISABLE) Macros
 */
#define SDK_SERVICE_LED      			 		 PS_DISABLE

#if (SDK_SERVICE_LED == PS_ENABLE)

/*!
 * LED Task Periodicity		100ms
 */
#define PS_LED_TASK_TIMEOUT        				 100

/*!
 * LED Task Priority
 * osPriorityNone          =  0,
 * osPriorityIdle          =  1,
 * osPriorityLow           =  8,
 * osPriorityLow1          =  8+1,
 * 		,,					  ,,
 * 		,,					  ,,
 * osPriorityISR           = 56,
 * osPriorityError         = -1,
 * osPriorityReserved      = 0x7FFFFFFF
 */
#define PS_LED_TASK_PRIORITY   					 osPriorityIdle

/*!
 * Maximum Number of LED required for this application
 *
 * MACOR Supported
 *
 * MAX_LED_NUM  : This hardware support maximum of 2 LED's
 *
 * CONF_xx_LED_PS_STATE			            PS_ENABLE
 *                                          PS_DISABLE
 *
 * CONF_xx_LED_STATE      					LED_CONF_OFF
 *                                          LED_CONF_ON
 *
 * CONF_xx_LED_BLINKING_MS        			<0-65535>
 *
 */
#define MAX_LED_NUM                    			 2

#define CONF_RED_LED_PS_STATE		             PS_ENABLE
#define CONF_RED_LED_STATE             			 LED_CONF_ON
#define CONF_RED_LED_BLINKING_MS       			 0

#define CONF_AMB_LED_PS_STATE		             PS_ENABLE
#define CONF_AMB_LED_STATE             			 LED_CONF_ON
#define CONF_AMB_LED_BLINKING_MS       			 0

#endif	//SDK_SERVICE_LED

/*****************************************************************************
 *
 *                      Configurable Input Module Configuration
 *
 *****************************************************************************/

/*!
 * Config input Platform service Enable(PS_ENABLE) / Disable(PS_DISABLE) Macros
 */
#define SDK_SERVICE_CFG_INPUT      				 PS_ENABLE

#if (SDK_SERVICE_CFG_INPUT == PS_ENABLE)

/*!
 * Config_input Task Periodicity		100ms
 */
#define PS_CFG_INPUT_TASK_TIMEOUT        		 100

/*!
 * CFG Input Task Priority
 * osPriorityNone          =  0,
 * osPriorityIdle          =  1,
 * osPriorityLow           =  8,
 * osPriorityLow1          =  8+1,
 * 		,,					  ,,
 * 		,,					  ,,
 * osPriorityISR           = 56,
 * osPriorityError         = -1,
 * osPriorityReserved      = 0x7FFFFFFF
 */
#define PS_CFG_INPUT_TASK_PRIORITY   			 osPriorityIdle

#define CONF_INPUT_TYPE_01       				 CI_INPUT_FREQUENCY
#define CONF_INPUT_TYPE_02       				 CI_INPUT_FREQUENCY
#define CONF_INPUT_TYPE_03       				 CI_INPUT_FREQUENCY
#define CONF_INPUT_TYPE_04       				 CI_INPUT_FREQUENCY
#define CONF_INPUT_TYPE_05       				 CI_INPUT_FREQUENCY
#define CONF_INPUT_TYPE_06       				 CI_INPUT_CURRENT

#define CONF_INPUT_01_NUMB_SAMPLES   			 1
#define CONF_INPUT_02_NUMB_SAMPLES   			 1
#define CONF_INPUT_03_NUMB_SAMPLES   			 1
#define CONF_INPUT_04_NUMB_SAMPLES   			 1
#define CONF_INPUT_05_NUMB_SAMPLES   			 1
#define CONF_INPUT_06_NUMB_SAMPLES   			 1

#endif	//SDK_SERVICE_CFG_INPUT

/*****************************************************************************
 *
 *                      LCD Module Configuration
 *
 *****************************************************************************/

/*!
 * LCD Platform service Enable(PS_ENABLE) / Disable(PS_DISABLE) Macros
 */
#define SDK_SERVICE_LCD           			 	 PS_ENABLE

#if (SDK_SERVICE_LCD == PS_ENABLE)

/*!
 * LCD Task Periodicity	100ms
 */
#define PS_LCD_TASK_TIMEOUT					 	 100

/*!
 * LCD Task Priority
 * osPriorityNone          =  0,
 * osPriorityIdle          =  1,
 * osPriorityLow           =  8,
 * osPriorityLow1          =  8+1,
 * 		,,					  ,,
 * 		,,					  ,,
 * osPriorityISR           = 56,
 * osPriorityError         = -1,
 * osPriorityReserved      = 0x7FFFFFFF
 */
#define PS_LCD_TASK_PRIORITY   					 osPriorityIdle

/*!
 * MACOR Supported
 *
 * CONF_LCD_STATE		         LCD_CONF_ON/
 * 								 LCD_CONF_OFF
 *
 * CONF_LCD_BRIGHTNESS           <0 - 100>
 *
 */
#define CONF_LCD_STATE             				 LCD_CONF_ON
#define CONF_LCD_BRIGHTNESS             		 100

#endif	//SDK_SERVICE_LCD

/*****************************************************************************
 *
 *                      Power Management Module Configuration
 *
 *****************************************************************************/

/*!
 * Power Management Platform service Enable(PS_ENABLE) / Disable(PS_DISABLE) Macros
 */
#define SDK_SERVICE_PM                         	 PS_ENABLE

#if (SDK_SERVICE_PM == PS_ENABLE)

/*!
 * Power Management Task Periodicity	100ms
 */
#define PS_PM_TASK_TIMEOUT                     	 100

/*!
 * PM Task Priority
 * osPriorityNone          =  0,
 * osPriorityIdle          =  1,
 * osPriorityLow           =  8,
 * osPriorityLow1          =  8+1,
 * 		,,					  ,,
 * 		,,					  ,,
 * osPriorityISR           = 56,
 * osPriorityError         = -1,
 * osPriorityReserved      = 0x7FFFFFFF
 */
#define PS_PM_TASK_PRIORITY   					 osPriorityIdle

/*!
 * GPIO Wake up Source
 */
#define KEYPAD01_WAKEUP_SOURCE                 	 PS_ENABLE
#define KEYPAD02_WAKEUP_SOURCE                 	 PS_ENABLE
#define KEYPAD03_WAKEUP_SOURCE                 	 PS_ENABLE
#define KEYPAD04_WAKEUP_SOURCE                 	 PS_ENABLE

/*!
 * RTC Wake up Source
 */
#define RTC_WAKEUP_SOURCE_STATE                	 PS_DISABLE
#define RTC_WAKEUP_SOURCE_TIMEOUT	           	 10000

/*!
 * IGN Wake up Source
 */
#define IGN_WAKEUP_SOURCE					   	 PS_ENABLE

/*!
 * CAN Wake up Source
 */
#define CAN_WAKEUP_SOURCE					   	 PS_ENABLE


#endif //SDK_SERVICE_PM

/*****************************************************************************
 *
 *                      Watchdog Module Configuration
 *
 *****************************************************************************/

/*!
 * Watchdog Platform service Enable(PS_ENABLE) / Disable(PS_DISABLE) Macros
 */
#define SDK_SERVICE_WATCHDOG      			 	 PS_ENABLE

#if (SDK_SERVICE_WATCHDOG == PS_ENABLE)

/*!
 * Watchdog Task Periodicity		100ms
 */
#define PS_WD_TASK_TIMEOUT        				 100

/*!
 * Watchdog Task Priority
 * osPriorityNone          =  0,
 * osPriorityIdle          =  1,
 * osPriorityLow           =  8,
 * osPriorityLow1          =  8+1,
 * 		,,					  ,,
 * 		,,					  ,,
 * osPriorityISR           = 56,
 * osPriorityError         = -1,
 * osPriorityReserved      = 0x7FFFFFFF
 */
#define PS_WD_TASK_PRIORITY   					 osPriorityIdle

/*!
 * Watchdog Reset timer
 */
#define PS_WD_RESET_TIMER           			 850

/*
 * IWDG_PRESCALER_4
 * IWDG_PRESCALER_8
 * IWDG_PRESCALER_16
 * IWDG_PRESCALER_32
 * IWDG_PRESCALER_64
 * IWDG_PRESCALER_128
 * IWDG_PRESCALER_256
 */
#define WATCHDOG_FEED_TIME					 	 IWDG_PRESCALER_256
/*!
 * Watchdog external task ping_id
 *
 * MAX Supported USER Watchdog is 10
 */
#define USER_TASK_WD0                            PS_DISABLE
#define USER_TASK_WD1                            PS_DISABLE
#define USER_TASK_WD2                            PS_DISABLE
#define USER_TASK_WD3                            PS_DISABLE
#define USER_TASK_WD4                            PS_DISABLE
#define USER_TASK_WD5                            PS_DISABLE
#define USER_TASK_WD6                            PS_DISABLE
#define USER_TASK_WD7                            PS_DISABLE
#define USER_TASK_WD8                            PS_DISABLE
#define USER_TASK_WD9                            PS_DISABLE


#endif	//SDK_SERVICE_WATCHDOG

/*****************************************************************************
 *
 *                      SDRAM Module Configuration
 *
 *****************************************************************************/

/*!
 * SDRAM Platform service Enable(PS_ENABLE) / Disable(PS_DISABLE) Macros
 */
#define SDK_SERVICE_SDRAM      			 	 	 PS_ENABLE

#if (SDK_SERVICE_SDRAM == PS_ENABLE)


#endif	//SDK_SERVICE_SDRAM

/*****************************************************************************
 *
 *                      Power Monitor Module Configuration
 *
 *****************************************************************************/

/*!
 * Power Monitor Platform service Enable(PS_ENABLE) / Disable(PS_DISABLE) Macros
 */
#define SDK_SERVICE_POWER_MONITOR      			 PS_ENABLE

#if (SDK_SERVICE_POWER_MONITOR == PS_ENABLE)

#if ((SDK_SERVICE_POWER_MONITOR == PS_ENABLE) && (SDK_SERVICE_CFG_INPUT == PS_DISABLE))
#undef SDK_SERVICE_POWER_MONITOR
#define SDK_SERVICE_POWER_MONITOR				PS_DISABLE
#endif

/*!
 * Power Monitor Task Priority
 * osPriorityNone          =  0,
 * osPriorityIdle          =  1,
 * osPriorityLow           =  8,
 * osPriorityLow1          =  8+1,
 * 		,,					  ,,
 * 		,,					  ,,
 * osPriorityISR           = 56,
 * osPriorityError         = -1,
 * osPriorityReserved      = 0x7FFFFFFF
 */
#define PS_POWER_MONITOR_TASK_PRIORITY   		 osPriorityIdle

/*!
 * Power Monitor Task Periodicity	100ms
 */
#define PS_POWER_MONITOR_TASK_TIMEOUT            100


#endif	//SDK_SERVICE_POWER_MONITOR

/*****************************************************************************
 *
 *                      EEPROM Module Configuration
 *
 *****************************************************************************/

/*!
 * EEPROM Platform service Enable(PS_ENABLE) / Disable(PS_DISABLE) Macros
 */
#define SDK_SERVICE_EEPROM      			 	 PS_DISABLE

#if (SDK_SERVICE_EEPROM == PS_ENABLE)
/*!
 * EEPROM Task Priority
 * osPriorityNone          =  0,
 * osPriorityIdle          =  1,
 * osPriorityLow           =  8,
 * osPriorityLow1          =  8+1,
 * 		,,					  ,,
 * 		,,					  ,,
 * osPriorityISR           = 56,
 * osPriorityError         = -1,
 * osPriorityReserved      = 0x7FFFFFFF
 */
#define PS_EE_TASK_PRIORITY   					 osPriorityIdle

/*!
 * EEPROM Task Periodicity	100ms
 */
#define PS_EE_TASK_TIMEOUT					 	 100

/*!
 * EEPROM Place holder size
 */
#define SIZE_OF_PLACEHOLDER                      100
#endif	//SDK_SERVICE_EEPROM

/*****************************************************************************
 *
 *                      SWTIMER Module Configuration
 *
 *****************************************************************************/

/*!
 * SWTIMER Platform service Enable(PS_ENABLE) / Disable(PS_DISABLE) Macros
 */
#define SDK_SERVICE_SWTIMER      			 	 PS_ENABLE

#if (SDK_SERVICE_SWTIMER == PS_ENABLE)
/*!
 * SWTIMER Task Priority
 * osPriorityNone          =  0,
 * osPriorityIdle          =  1,
 * osPriorityLow           =  8,
 * osPriorityLow1          =  8+1,
 * 		,,					  ,,
 * 		,,					  ,,
 * osPriorityISR           = 56,
 * osPriorityError         = -1,
 * osPriorityReserved      = 0x7FFFFFFF
 */
#define PS_SWT_TASK_PRIORITY   					 osPriorityIdle

/*!
 * SWTimer Task Periodicity	100ms
 */
#define PS_SWT_TASK_TIMEOUT					 	 100

#endif	//SDK_SERVICE_SWTIMER

/*****************************************************************************
 *
 *                      BLE Module Configuration
 *
 *****************************************************************************/

/*!
 * BLE Platform service Enable(PS_ENABLE) / Disable(PS_DISABLE) Macros
 */
#define SDK_SERVICE_BLE      			 	 	 PS_ENABLE

#if (SDK_SERVICE_BLE == PS_ENABLE)
/*!
 * BLE Task Priority
 * osPriorityNone          =  0,
 * osPriorityIdle          =  1,
 * osPriorityLow           =  8,
 * osPriorityLow1          =  8+1,
 * 		,,					  ,,
 * 		,,					  ,,
 * osPriorityISR           = 56,
 * osPriorityError         = -1,
 * osPriorityReserved      = 0x7FFFFFFF
 */
#define PS_BLE_TASK_PRIORITY   			 		 osPriorityIdle

/*!
 * BLE Task Periodicity	100ms
 */
#define PS_BLE_TASK_TIMEOUT				 		 100

/*!
 * BLE Device Name
 */
#define BLE_DEVICE_NAME 						 "maxAIVidSample"

#endif	//SDK_SERVICE_BLE

/*****************************************************************************
 *
 *                      USB Module Configuration
 *
 *****************************************************************************/

/*!
 * USB Platform service Enable(PS_ENABLE) / Disable(PS_DISABLE) Macros
 */
#define SDK_SERVICE_USB      			 		 PS_ENABLE

#if (SDK_SERVICE_USB == PS_ENABLE)

/*!
 * USB Task Periodicity		100ms
 */
#define PS_USB_TASK_TIMEOUT        			 	 100

/*!
 * USB Task Priority
 * osPriorityNone          =  0,
 * osPriorityIdle          =  1,
 * osPriorityLow           =  8,
 * osPriorityLow1          =  8+1,
 * 		,,					  ,,
 * 		,,					  ,,
 * osPriorityISR           = 56,
 * osPriorityError         = -1,
 * osPriorityReserved      = 0x7FFFFFFF
 */
#define PS_USB_TASK_PRIORITY   				 	 osPriorityIdle

/*!
 * EcuIdentification commands
 */
#endif	//SDK_SERVICE_USB
#if ((SDK_SERVICE_USB == PS_ENABLE) || (SDK_SERVICE_BLE == PS_ENABLE) )
#define VEHICLE_MANU_ECU_SW_NUM			 		 0x88
#define VEHICLE_MANU_ECU_SW_VER          		 0x89
#define ECU_MANU_DATE                    		 0x8B
#define ECU_SERIAL_NUM                   		 0x8C
#define PROGRAMMING_DATE                 		 0x99
#define PS_USB_USER_PACKET                       0x50

#endif

/*****************************************************************************
 *
 *                      Camera Module Configuration
 *
 *****************************************************************************/

/*!
 * Camera Platform service Enable(PS_ENABLE) / Disable(PS_DISABLE) Macros
 */
#define SDK_SERVICE_CAMERA      			 	 PS_ENABLE

#if (SDK_SERVICE_CAMERA == PS_ENABLE)

/*!
 * Camera Task Periodicity		100ms
 */
#define PS_CAMERA_TASK_TIMEOUT        			 100

/*!
 * Camera Task Priority
 * osPriorityNone          =  0,
 * osPriorityIdle          =  1,
 * osPriorityLow           =  8,
 * osPriorityLow1          =  8+1,
 * 		,,					  ,,
 * 		,,					  ,,
 * osPriorityISR           = 56,
 * osPriorityError         = -1,
 * osPriorityReserved      = 0x7FFFFFFF
 */
#define PS_CAMERA_TASK_PRIORITY   				  osPriorityIdle //osPriorityLow4//

/*!
 *  PS_CAMERA_MODE
 *  =============
 *  FULL_SCREEN_ON
 * 	RESIZE_TO_FULL_SCREEN_ON
 * 	DISPLAY_AS_IT_IS_ON
 */
#define PS_CAMERA_MODE                              FULL_SCREEN_ON

/*!
 *
 */

#define PS_CAMERA_X0_DISP_ORIGIN                      0
#define PS_CAMERA_Y0_DISP_ORIGIN                      0
//(NTSC 612x238, MAX_X0_DISP_ORG 264, MIN_X0_DISP_ORG=4) visible FOV area (547x238)
#define PS_NTSC_CAMERA_X0_ORIGIN                      4 //12 //4 // 4/4=1Pixels
#define PS_NTSC_CAMERA_Y0_ORIGIN                      0
#define PS_NTSC_CAMERA_X0_WIDTH						  612 //612
#define PS_NTSC_CAMERA_Y0_HEIGHT                      238 //222 //238
//(PAL 540,273,  MAX_X0_DISP_ORG 120, MIN_X0_DISP_ORG=76) visible FOV area (529x273)
#define PS_PAL_CAMERA_X0_ORIGIN                       76 // 76/4=9Pixels
#define PS_PAL_CAMERA_Y0_ORIGIN                       0
#define PS_PAL_CAMERA_X0_WIDTH						  540
#define PS_PAL_CAMERA_Y0_HEIGHT                       273

/*  PS_ENABLE : During the Camera streaming, if the key
 *  is enabled and pressed, the camera streaming will be stopped */
#define SDK_CAMERA_STOP_KEY1_KEYPAD              PS_DISABLE
#define SDK_CAMERA_STOP_KEY2_KEYPAD              PS_ENABLE
#define SDK_CAMERA_STOP_KEY3_KEYPAD              PS_DISABLE
#define SDK_CAMERA_STOP_KEY4_KEYPAD              PS_DISABLE

#if ((SDK_CAMERA_STOP_KEY1_KEYPAD == PS_DISABLE) &&  \
     (SDK_CAMERA_STOP_KEY2_KEYPAD == PS_DISABLE) &&  \
	 (SDK_CAMERA_STOP_KEY3_KEYPAD == PS_DISABLE) &&  \
	 (SDK_CAMERA_STOP_KEY4_KEYPAD == PS_DISABLE))
/*
 * Configuration Input AI1 ===> CONF_AI1
 * Configuration Input AI2 ===> CONF_AI2
 * Configuration Input AI3 ===> CONF_AI3
 * Configuration Input AI4 ===> CONF_AI4
 * Configuration Input AI5 ===> CONF_AI5
 * Configuration Input None ===> CONF_NONE
 */
#define SDK_CAMERA_STOP_CI_STB                   CONF_NONE
#define SDK_CAMERA_STOP_CI_STG                   CONF_NONE
#endif

#if ((SDK_CAMERA_STOP_KEY1_KEYPAD == PS_DISABLE) &&  \
     (SDK_CAMERA_STOP_KEY2_KEYPAD == PS_DISABLE) &&  \
	 (SDK_CAMERA_STOP_KEY3_KEYPAD == PS_DISABLE) &&  \
	 (SDK_CAMERA_STOP_KEY4_KEYPAD == PS_DISABLE) &&  \
	 (SDK_CAMERA_STOP_CI_STB      == CONF_NONE)  &&  \
	 (SDK_CAMERA_STOP_CI_STG      == CONF_NONE))

/*
 *  Camera ON/OFF based on the CAN packet
 *  PS_ENABLE : Enable the CAMERA Stream ON/OFF through CAN Message
 *
 */
#define SDK_CAMERA_STOP_CAN                      PS_ENABLE
#endif


/* SDL Module to be stopped */
/*  PS_ENABLE : Stopped the SDK Service during the Camera Streaming */
#define SDK_CAM_WARNING_LIGHT_STOP             	PS_DISABLE
#define SDK_CAM_LED_STOP                       	PS_DISABLE
#define SDK_CAM_LS_STOP                        	PS_DISABLE
#define SDK_CAM_USB_STOP                       	PS_DISABLE
#define SDK_CAM_DIO_STOP                       	PS_DISABLE
#define SDK_CAM_BLE_STOP                       	PS_DISABLE

#endif	//SDK_SERVICE_CAMERA

/*****************************************************************************
 *
 *                      QSPI Module
 *
 *****************************************************************************/

/*!
 * QSPI SDK API (PS_ENABLE) / Disable(PS_DISABLE) Macros
 */
#define SDK_SERVICE_QSPI      			 		 PS_ENABLE


/*****************************************************************************
 *
 *                      FDCAN Module
 *
 *****************************************************************************/

/*!
 * FDCAN Platform service (PS_ENABLE) / Disable(PS_DISABLE) Macros
 */
#define SDK_SERVICE_FDCAN      			 		 PS_DISABLE

#if (SDK_SERVICE_FDCAN == PS_ENABLE)

#define FDCAN1_ENABLE        			 		 PS_ENABLE
#define FDCAN2_ENABLE       			 		 PS_ENABLE

/*!
 * FDCAN Task Periodicity		100ms
 */
#define PS_FDCAN_TASK_TIMEOUT        			 100

/*!
 * FDCAN Task Priority
 * osPriorityNone          =  0,
 * osPriorityIdle          =  1,
 * osPriorityLow           =  8,
 * osPriorityLow1          =  8+1,
 * 		,,					  ,,
 * 		,,					  ,,
 * osPriorityISR           = 56,
 * osPriorityError         = -1,
 * osPriorityReserved      = 0x7FFFFFFF
 */
#define PS_FDCAN_TASK_PRIORITY   				 osPriorityIdle

#if (FDCAN1_ENABLE == PS_ENABLE)

/*
 * BAUDRATE_50K
 * BAUDRATE_100K
 * BAUDRATE_125K
 * BAUDRATE_250K
 * BAUDRATE_500K
 * BAUDRATE_1000K
 */
#define FDCAN1_BAUDRATE							BAUDRATE_50K

#define FDCAN1_IDENTIFIER						0x19FEFCFE //0x111
#define FDCAN1_IDTYPE							FDCAN_EXTENDED_ID
#define FDCAN1_TXFRAMETYPE						FDCAN_DATA_FRAME
#define FDCAN1_DATALENGTH						FDCAN_DLC_BYTES_8
#define FDCAN1_ERRORSTATEIND					FDCAN_ESI_ACTIVE
#define FDCAN1_BITRATESWITCH					FDCAN_BRS_ON
#define FDCAN1_FDFORMATE						FDCAN_FD_CAN
#define FDCAN1_TXEVENTFIFOCONTROL				FDCAN_STORE_TX_EVENTS
#define FDCAN1_MESSAGEMARKER					0

#define FDCAN1_ID								FDCAN_STANDARD_ID
#define FDCAN1_FILTERINDEX						0
#define FDCAN1_FILTERTYPE						FDCAN_FILTER_DUAL
#define FDCAN1_FILTERCONFIG						FDCAN_FILTER_TO_RXBUFFER
#define FDCAN1_FILTERID1						FDCAN_DEAFULT_FILTERID1
#define FDCAN1_FILTERID2						FDCAN_DEAFULT_FILTERID2
#define FDCAN1_RXBUFFERIND						1

/* CAN1 Receive Task Delay */
#define FDCAN1_RECEIVE_TASK_DELAY				100

#endif	/*	FDCAN1_ENABLE	*/

#if (FDCAN2_ENABLE == PS_ENABLE)

/*
 * BAUDRATE_50K
 * BAUDRATE_100K
 * BAUDRATE_125K
 * BAUDRATE_250K
 * BAUDRATE_500K
 * BAUDRATE_1000K
 */
#define FDCAN2_BAUDRATE							BAUDRATE_50K

#define FDCAN2_IDENTIFIER						0x18FEFCFE //0x222
#define FDCAN2_IDTYPE							FDCAN_EXTENDED_ID
#define FDCAN2_TXFRAMETYPE						FDCAN_DATA_FRAME
#define FDCAN2_DATALENGTH						FDCAN_DLC_BYTES_8
#define FDCAN2_ERRORSTATEIND					FDCAN_ESI_ACTIVE
#define FDCAN2_BITRATESWITCH					FDCAN_BRS_ON
#define FDCAN2_FDFORMATE						FDCAN_FD_CAN
#define FDCAN2_TXEVENTFIFOCONTROL				FDCAN_STORE_TX_EVENTS
#define FDCAN2_MESSAGEMARKER					0

#define FDCAN2_ID								FDCAN_STANDARD_ID
#define FDCAN2_FILTERINDEX						0
#define FDCAN2_FILTERTYPE						FDCAN_FILTER_DUAL
#define FDCAN2_FILTERCONFIG						FDCAN_FILTER_TO_RXBUFFER
#define FDCAN2_FILTERID1						FDCAN_DEAFULT_FILTERID1
#define FDCAN2_FILTERID2						FDCAN_DEAFULT_FILTERID2
#define FDCAN2_RXBUFFERIND						1

/* CAN2 Receive Task Delay */
#define FDCAN2_RECEIVE_TASK_DELAY				100

#endif	/*	FDCAN2_ENABLE	*/

#endif	//SDK_SERVICE_FDCAN

/*****************************************************************************
 *
 *                      J1939 Module
 *
 *****************************************************************************/

/*!
 * J1939 Platform service (PS_ENABLE) / Disable(PS_DISABLE) Macros
 */
#define SDK_SERVICE_J1939      			 		 PS_ENABLE //PS_DISABLE

#if ((SDK_SERVICE_J1939 == PS_ENABLE) && (SDK_SERVICE_FDCAN == PS_ENABLE))
#undef SDK_SERVICE_FDCAN
#define SDK_SERVICE_FDCAN      			 		 PS_DISABLE
#endif

#if (SDK_SERVICE_J1939 == PS_ENABLE)

/*!
 * J1939 Task Periodicity		100ms
 */
#define PS_J1939_TASK_TIMEOUT        			 100

/*!
 * J1939 Task Priority
 * osPriorityNone          =  0,
 * osPriorityIdle          =  1,
 * osPriorityLow           =  8,
 * osPriorityLow1          =  8+1,
 * 		,,					  ,,
 * 		,,					  ,,
 * osPriorityISR           = 56,
 * osPriorityError         = -1,
 * osPriorityReserved      = 0x7FFFFFFF
 */
#define PS_J1939_TASK_PRIORITY   				 osPriorityIdle

/*!
 * J1939_CLAIM_ADDRESS
 */
#define J1939_CLAIM_ADDRESS                      PS_ENABLE

/*
 * State BLE J1939 Debug data
 */
#define BLE_J1939_DEBUG_DATA					 PS_ENABLE

#define J1939_CAN0_ENABLE                        PS_ENABLE
#define J1939_CAN1_ENABLE                        PS_ENABLE

/*!
 * J1939_CLAIM_ADDRESS will be between 0 to 255
 */
#define J1939_CAN0_CLAIM_ADDRESS                 23
#define J1939_CAN1_CLAIM_ADDRESS				 85

/*!
 * J1939_ADDRESS_CLAIM_DYNAMIC will search address when set to 1, Fixed = 0
 */
#define J1939_CAN0_ADDRESS_CLAIM_DYNAMIC	     1
#define J1939_CAN1_ADDRESS_CLAIM_DYNAMIC		 1

/*!
 * J1939_ADDRESS_CLAIM_NEXT_ADDRESS When dynamic set to 1, search will starting from this value and up
 */
#define J1939_CAN0_ADDRESS_CLAIM_NEXT_ADDRESS	 80
#define J1939_CAN1_ADDRESS_CLAIM_NEXT_ADDRESS	 90

/*!
 * J1939_CAN0_BITRATE will be between 0/250/500/1000
 */
#define J1939_CAN0_BITRATE                       250
#define J1939_CAN1_BITRATE                       250

/*!
 * J1939 ENABLE/Disable DMx
 */
#define EMTOS_J1939_DM1							 PS_ENABLE
#define EMTOS_J1939_DM2							 PS_ENABLE
#define EMTOS_J1939_DM3							 PS_ENABLE
//#define EMTOS_J1939_DM11						 PS_ENABLE
/*!
 * J1939 Set number of PGN'S to receive and transmit
 */
#define CAN0_NUMBER_PGNS_RX                      18u
#define CAN1_NUMBER_PGNS_RX                      3u
#define CAN0_NUMBER_PGNS_TX                      1u
#define CAN1_NUMBER_PGNS_TX                      1u
/*!
 * J1939 Set COB handlers
 */
#define CAN0_CO_COB_COUNTS                       36u
#define CAN1_CO_COB_COUNTS                       21u
#endif	//SDK_SERVICE_J1939

/*****************************************************************************
 *
 *                      Through Put Service
 *
 *****************************************************************************/

/*!
 * Through Put service (PS_ENABLE) / Disable(PS_DISABLE) Macros
 */
#define THROUGH_PUT_SERVICE							PS_ENABLE

#if (THROUGH_PUT_SERVICE == PS_ENABLE)

#endif	//THROUGH_PUT_SERVICE


/*****************************************************************************
 *
 *                      		USER Task
 *
 *****************************************************************************/

/*!
 * User Task (PS_ENABLE) / Disable(PS_DISABLE) Macros
 */
#define USER_TASK_STATE								PS_ENABLE

#if (USER_TASK_STATE == PS_ENABLE)

#define USER_TASK1									PS_ENABLE
#define USER_TASK2									PS_ENABLE
#define USER_TASK3									PS_ENABLE

#endif	//USER_TASK_STATE

#endif	//_AI430_CONFIG_H_

/**************** (C) COPYRIGHT 2022 Maxima Technologies *******************/
/***************************** END OF FILE *********************************/
