/****************************************************************************
              Copyright (c) 2022 Maxima Technologies and Systems
                           All Rights Reserved

   File Name          : OPT3002.h
   Author             : Zaid
   Version            : V0.0.0.1
   Origination Date   : 28/01/2022
   Description        : Header File.
   Environment        : Independent
   Processor:         : STM32H743IIT6

-----------------------------------------------------------------------------
   Description:

   This file contains the OPT3002 configuration macros

-----------------------------------------------------------------------------
	File Revision History:
-----------------------------------------------------------------------------
   Version             Date                 Programmer
   0.0.0.1             28/01/2022           Zaid

   Notes:              - Initial Version
-----------------------------------------------------------------------------
*****************************************************************************
THIS FILE CONTAINS INTELLECTUAL PROPERTY OF MAXIMA TECHNOLOGIES AND SYSTEMS.
UNAUTHORIZED REDISTRIBUTION OF THIS SOURCE CODE, IN WHOLE OR PART, WITHOUT
THE EXPRESS WRITTEN PERMISSION OF MAXIMA TECHNOLOGIES IS STRICTLY PROHIBITED.
****************************************************************************/

#ifndef OPT3002_H
#define OPT3002_H


typedef enum
{
	OPTERR_ERROR_OK		  = 0x00FF,
	OPTERR_NOT_READY      = 0x0000,
	OPTERR_SET_OPERATION  = 0x0001,
	OPTERR_SET_LL		  = 0x0002,
	OPTERR_SET_HL   	  = 0x0003,
	OPTERR_SET_ADD        = 0x0004,
	OPTERR_RESULT         = 0x0005,
	OPTERR_MANUF_ID  	  = 0x0006,
}opt_drv_t;

/*!
 * Configuation Reg : Conversion time Mask and bit position
 */
#define OPT_CONF_CONVERSION_TIME_MASK            	0x08
#define OPT_CONF_CONVERSION_TIME_BIT_POSITION    	3
#define OPT_CONF_MODE_MASK                       	0x06
#define OPT_CONF_MODE_BIT_POSITION               	1
#define OPT_CONF_DEVICE_SHUTDOWN                 	0x00

#define Addr_OPT3002 								0x88

/*!
 * Configuation Reg : Automatic full scale value
 */
#define AUTOMATIC_FULL_SCALE     				 	0xC

/*!
 * Field Bit Position fields
 */
#define LS_CFG_BIT_POSITION        				 	1
#define MODE_BIT_POSITION        				 	3
#define CONV_TIME_BIT_POSITION    				 	4


/*!
 * Light Sensor Min and Max Conversion time
 */
#define CONF_REG_CONVERSION_TIME_100  			 	0
#define CONF_REG_CONVERSION_TIME_800  			 	1

#define CONF_TIME_IN_100MS                          1
#define CONF_TIME_IN_800MS                          8
/*!
 * Light Sensor Mode Configuration
 */
#define CONF_REG_SHUTDOWN                           00
#define CONF_REG_SINGLE_MODE_CONV      			 	01
#define CONF_REG_CONTINUOUS_MODE_CONV  			 	02

uint16_t SOPTdriver_Manufacturer_ID(I2C_HandleTypeDef *hi2c);
uint8_t SOPTdriver_GetCovntime(I2C_HandleTypeDef *hi2c, uint8_t *convtime);
uint8_t SOPTdriver_GetMode(I2C_HandleTypeDef *hi2c, uint8_t *mode);
uint8_t SOPTdriver_Shutdown(I2C_HandleTypeDef *hi2c);
extern opt_drv_t SOPTdriver_Init(I2C_HandleTypeDef *hi2c, uint8_t convtime, uint8_t mode);
extern opt_drv_t SOPTdriver_LoLimit(I2C_HandleTypeDef *hi2c, uint16_t lowlimit);
extern opt_drv_t SOPTdriver_HiLimit(I2C_HandleTypeDef *hi2c, uint16_t highlimit);
extern uint32_t AdjustLCDContrast(I2C_HandleTypeDef *hi2c, uint8_t MaxOut,uint8_t MinOut,float MaxAdjRead,float MinAdjRead);
extern opt_drv_t SOPTdriver_Result(I2C_HandleTypeDef *hi2c, uint32_t *val);
extern opt_drv_t SOPTdriver_RawResult(I2C_HandleTypeDef *hi2c, uint8_t *rsp_data);

extern void updateLightSensor(void);
extern uint8_t Get_flag_night_mode(void);

#endif /* OPT3002_H */

/**************** (C) COPYRIGHT 2022 Maxima Technologies *******************/
/***************************** END OF FILE *********************************/
