/****************************************************************************
              Copyright (c) 2022 Maxima Technologies and Systems
                           All Rights Reserved

   File Name          : Warning_light.h
   Author             : Rajesh
   Version            : V0.0.0.1
   Origination Date   : 21/01/2022
   Description        : Header File.
   Environment        : Independent
   Processor:         : STM32H743IIT6

-----------------------------------------------------------------------------
   Description:

   This file contains the board configuration structures and macros

-----------------------------------------------------------------------------
	File Revision History:
-----------------------------------------------------------------------------
   Version             Date                 Programmer
   0.0.0.1             21/01/2022           Rajesh

   Notes:              - Initial Version
-----------------------------------------------------------------------------
*****************************************************************************
THIS FILE CONTAINS INTELLECTUAL PROPERTY OF MAXIMA TECHNOLOGIES AND SYSTEMS.
UNAUTHORIZED REDISTRIBUTION OF THIS SOURCE CODE, IN WHOLE OR PART, WITHOUT
THE EXPRESS WRITTEN PERMISSION OF MAXIMA TECHNOLOGIES IS STRICTLY PROHIBITED.
****************************************************************************/
#ifndef _WARNING_LIGHT_H_
#define	_WARNING_LIGHT_H_

/* USER CODE BEGIN Macros */

#define I2C_TIMEOUT 				1000

/*	IS31FL3235 Slave Address */
#define IS31FL3235_IIC_ADDRESS  	0x78 /*	AD == GND */


#define Addr_VCC 0x7e

#define IS31FL3235_REG_WORK			0x00 /*	SSD */

/* PWM Control registers */
#define IS31FL3235_REG_PWM_OUT1 	0x05 /*	PWM */
#define IS31FL3235_REG_PWM_OUT2 	0x06
#define IS31FL3235_REG_PWM_OUT3 	0x07
#define IS31FL3235_REG_PWM_OUT4 	0x08
#define IS31FL3235_REG_PWM_OUT5 	0x09
#define IS31FL3235_REG_PWM_OUT6 	0x0A
#define IS31FL3235_REG_PWM_OUT7 	0x0B
#define IS31FL3235_REG_PWM_OUT8 	0x0C
#define IS31FL3235_REG_PWM_OUT9 	0x0D
#define IS31FL3235_REG_PWM_OUT10 	0x0E
#define IS31FL3235_REG_PWM_OUT11 	0x0F
#define IS31FL3235_REG_PWM_OUT12 	0x10
#define IS31FL3235_REG_PWM_OUT13 	0x11
#define IS31FL3235_REG_PWM_OUT14 	0x12
#define IS31FL3235_REG_PWM_OUT15 	0x13
#define IS31FL3235_REG_PWM_OUT16 	0x14
#define IS31FL3235_REG_PWM_OUT17 	0x15
#define IS31FL3235_REG_PWM_OUT18 	0x16
#define IS31FL3235_REG_PWM_OUT19 	0x17
#define IS31FL3235_REG_PWM_OUT20 	0x18
#define IS31FL3235_REG_PWM_OUT21 	0x19
#define IS31FL3235_REG_PWM_OUT22 	0x1A
#define IS31FL3235_REG_PWM_OUT23 	0x1B
#define IS31FL3235_REG_PWM_OUT24 	0x1C
#define IS31FL3235_REG_PWM_OUT25 	0x1D
#define IS31FL3235_REG_PWM_OUT26 	0x1E
#define IS31FL3235_REG_PWM_OUT27 	0x1F
#define IS31FL3235_REG_PWM_OUT28 	0x20

/*	PWM and CTRL update commend */
#define	IS31FL3235_REG_DATA_UPDATE	0x25


/* LED Control registers */
#define IS31FL3235_REG_CTRL_OUT1 	0x2A /*	LED control */
#define IS31FL3235_REG_CTRL_OUT2 	0x2B
#define IS31FL3235_REG_CTRL_OUT3 	0x2C
#define IS31FL3235_REG_CTRL_OUT4 	0x2D
#define IS31FL3235_REG_CTRL_OUT5 	0x2E
#define IS31FL3235_REG_CTRL_OUT6 	0x2F
#define IS31FL3235_REG_CTRL_OUT7 	0x30
#define IS31FL3235_REG_CTRL_OUT8 	0x31
#define IS31FL3235_REG_CTRL_OUT9 	0x32
#define IS31FL3235_REG_CTRL_OUT10 	0x33
#define IS31FL3235_REG_CTRL_OUT11 	0x34
#define IS31FL3235_REG_CTRL_OUT12 	0x35
#define IS31FL3235_REG_CTRL_OUT13 	0x36
#define IS31FL3235_REG_CTRL_OUT14 	0x37
#define IS31FL3235_REG_CTRL_OUT15 	0x38
#define IS31FL3235_REG_CTRL_OUT16 	0x39
#define IS31FL3235_REG_CTRL_OUT17 	0x3A
#define IS31FL3235_REG_CTRL_OUT18 	0x3B
#define IS31FL3235_REG_CTRL_OUT19 	0x3C
#define IS31FL3235_REG_CTRL_OUT20 	0x3D
#define IS31FL3235_REG_CTRL_OUT21 	0x3E
#define IS31FL3235_REG_CTRL_OUT22 	0x3F
#define IS31FL3235_REG_CTRL_OUT23 	0x40
#define IS31FL3235_REG_CTRL_OUT24 	0x41
#define IS31FL3235_REG_CTRL_OUT25 	0x42
#define IS31FL3235_REG_CTRL_OUT26 	0x43
#define IS31FL3235_REG_CTRL_OUT27 	0x44
#define IS31FL3235_REG_CTRL_OUT28 	0x45


#define IS31FL3235_REG_LED_SYNC 	0x4A
#define IS31FL3235A_REG_FREQ     	0x4B /*	PWM fre */

#define IS31FL3235_REG_RST 			0x4F /*	reset */

#define IS31FL3235_REG_CTRL_OUT_CURR_BIT 1

/*!
 * Warning light macros
 */
#define WL_LED_ON           		0x01
#define WL_LED_OFF          		0x00
#define WL_MIN_LED_NUMBER   		0
#define WL_MAX_LED_NUMBER   		20
#define WL_MIN_CURRENT      		0
#define WL_MAX_CURRENT      		3

#define WL_MIN_INTENSITY    		0
#define WL_MAX_INTENSITY    		256

#define kHz3_FREQ           		0
#define kHz22_FREQ          		1

/*!
 * Shutdown all LEDs
 */
#define GC_SHUTDOWN_ALL_LEDS   		1
#define GC_NORMAL_OPERATION    		0

/*!
 * Software Shutdown Enable
 */
#define SOFTWARE_SHUTDOWN_MODE 		0
#define SOFTWARE_NORMAL_MODE   		1

/*!
 * WL current configuration
 */
#define WL_REG_CURRENT_FIELD 		2

/*!
 * Warning Light Shutdown Hardware Pin control
 */
#define HW_WLSTB_Pin 				GPIO_PIN_1
#define HW_WLSTB_GPIO_Port 			GPIOG

/* USER CODE END Macros */

/*!
 * Function declaration
 */
int8_t Warning_Light_On(I2C_HandleTypeDef *hi2c, uint8_t led_num, uint8_t current);
int8_t Warning_Light_Off(I2C_HandleTypeDef *hi2c, uint8_t led_num);
int8_t Set_Warning_Light_Intensity(I2C_HandleTypeDef *hi2c, uint8_t led_num, uint8_t intenval);
void Turn_Off_Warning_Light(I2C_HandleTypeDef *hi2c);
int8_t Warning_Light_Init(I2C_HandleTypeDef *hi2c, uint8_t freq);


#endif

/**************** (C) COPYRIGHT 2022 Maxima Technologies *******************/
/***************************** END OF FILE *********************************/
