/****************************************************************************
              Copyright (c) 2022 Maxima Technologies and Systems
                           All Rights Reserved

   File Name          : Adc.c
   Author             : Zaid
   Version            : V0.0.0.1
   Origination Date   : 28/01/2022
   Description        : C Source File.
   Environment        : Independent
   Processor:         : STM32H743IIT6

-----------------------------------------------------------------------------
   Description:

   This file contains the ADC HAL driver functions

-----------------------------------------------------------------------------
	File Revision History:
-----------------------------------------------------------------------------
   Version             Date                 Programmer
   0.0.0.1             28/01/2022           Zaid

   Notes:              - Initial Version
-----------------------------------------------------------------------------
*****************************************************************************
THIS FILE CONTAINS INTELLECTUAL PROPERTY OF MAXIMA TECHNOLOGIES AND SYSTEMS.
UNAUTHORIZED REDISTRIBUTION OF THIS SOURCE CODE, IN WHOLE OR PART, WITHOUT
THE EXPRESS WRITTEN PERMISSION OF MAXIMA TECHNOLOGIES IS STRICTLY PROHIBITED.
****************************************************************************/

/* USER CODE BEGIN Includes */
#include "../../../Core/Maximatecc/Inc/Common.h"
/* USER CODE END Includes */

#if (SDK_SERVICE_CFG_INPUT == PS_ENABLE)

uint32_t ADC_resultsCH1[CHANNELS_ADC_1];
uint32_t ADC_resultsCH3[CHANNELS_ADC_3];

uint32_t ADC_results[MAX_ADC];

extern pvt_variable pv;

extern ci_conf_param ci_conf[MAX_NUM_CFG_INPUT];


/**
  * @brief  Conversion complete callback in non blocking mode
  * @param  AdcHandle : AdcHandle handle
  * @note   This example shows a simple way to report end of conversion, and
  *         you can add your own implementation.
  * @retval None
  */
void HAL_ADC_ConvCpltCallback(ADC_HandleTypeDef* hadc)
{
#if ((SDK_CAMERA_STOP_CI_STG != CONF_NONE) && (SDK_CAMERA_STOP_CI_STB == CONF_NONE))
	uint16_t val1 =0;
	uint16_t val2 =0;
	uint8_t stg = 0;
#endif

#if ((SDK_CAMERA_STOP_CI_STG == CONF_NONE) && (SDK_CAMERA_STOP_CI_STB != CONF_NONE))
	uint16_t val1 =0;
	uint16_t val2 =0;
	uint8_t stb = 0;
#endif

	if(hadc->Instance==ADC3)
	{
		for(int i =0;i<CHANNELS_ADC_3;i++)
		{
			ADC_results[i+CHANNELS_ADC_1] = ADC_resultsCH3[i];
		}
	}
	else if(hadc->Instance == ADC1)
	{
		for(int i =0;i<CHANNELS_ADC_1;i++)
		{
			ADC_results[i] = ADC_resultsCH1[i];
		}
	}
#if ((SDK_CAMERA_STOP_KEY1_KEYPAD == PS_DISABLE) &&  \
     (SDK_CAMERA_STOP_KEY2_KEYPAD == PS_DISABLE) &&  \
	 (SDK_CAMERA_STOP_KEY3_KEYPAD == PS_DISABLE) &&  \
	 (SDK_CAMERA_STOP_KEY4_KEYPAD == PS_DISABLE))

#if ((SDK_CAMERA_STOP_CI_STG == CONF_NONE) && (SDK_CAMERA_STOP_CI_STB != CONF_NONE))

	val1 = GetADC_Results(ci_conf[SDK_CAMERA_STOP_CI_STB].AChanADC);
	val2 = GetADC_Results(ci_conf[SDK_CAMERA_STOP_CI_STB].BChanADC);

	stb = STB_convertion(val1, val2);

	if (0 == stb)
		StopStreaming();

#endif

#if ((SDK_CAMERA_STOP_CI_STG != CONF_NONE) && (SDK_CAMERA_STOP_CI_STB == CONF_NONE))
	val1 = GetADC_Results(ci_conf[SDK_CAMERA_STOP_CI_STG].AChanADC);
	val2 = GetADC_Results(ci_conf[SDK_CAMERA_STOP_CI_STG].BChanADC);

	stg = STG_convertion(val1, val2);
	if (FALSE == stg)
		StopStreaming();
#endif
#endif
}

/*---------------------------------------------------------------------------
 Function:     ADC_start
 Description:  This function is used to start the DMA to get the ADC value.

 Inputs Parameters:		void
 Return Parameters:		None
---------------------------------------------------------------------------*/
void ADC_start(void)
{
	HAL_ADCEx_Calibration_Start(pv.hadc1, ADC_CALIB_OFFSET, ADC_SINGLE_ENDED);
	HAL_ADCEx_Calibration_Start(pv.hadc3, ADC_CALIB_OFFSET, ADC_SINGLE_ENDED);
	if(HAL_ADC_Start_DMA(pv.hadc1,(uint32_t*)ADC_resultsCH1,CHANNELS_ADC_1) != HAL_OK)
	{
		/* Start Conversation Error */
		Error_Handler();
	}

	if(HAL_ADC_Start_DMA(pv.hadc3,(uint32_t *) ADC_resultsCH3, CHANNELS_ADC_3) != HAL_OK)
	{
		/* Start Conversation Error */
		Error_Handler();
	}
}

/*---------------------------------------------------------------------------
 Function:     GetADC_Results
 Description:  This function is used to Get the updated value of the analog

 Inputs Parameters:		int
 Return Parameters:		uint32_t
---------------------------------------------------------------------------*/
uint32_t GetADC_Results(int i)
{
	if(i<MAX_ADC)
	{
		/*AI_Conversions manage this results*/
		return ADC_results[i];
	}
	else
		return 0;

}

#endif
/**************** (C) COPYRIGHT 2022 Maxima Technologies *******************/
/***************************** END OF FILE *********************************/
