/****************************************************************************
              Copyright (c) 2022 Maxima Technologies and Systems
                           All Rights Reserved

   File Name          : Led.c
   Author             : Adrush
   Version            : V0.0.0.1
   Origination Date   : 04/02/2022
   Description        : C Source File.
   Environment        : Independent
   Processor:         : STM32H743IIT6

-----------------------------------------------------------------------------
   Description:

   This file contains the LED HAL driver functions

-----------------------------------------------------------------------------
	File Revision History:
-----------------------------------------------------------------------------
   Version             Date                 Programmer
   0.0.0.1             04/02/2022           Adrush

   Notes:              - Initial Version
-----------------------------------------------------------------------------
*****************************************************************************
THIS FILE CONTAINS INTELLECTUAL PROPERTY OF MAXIMA TECHNOLOGIES AND SYSTEMS.
UNAUTHORIZED REDISTRIBUTION OF THIS SOURCE CODE, IN WHOLE OR PART, WITHOUT
THE EXPRESS WRITTEN PERMISSION OF MAXIMA TECHNOLOGIES IS STRICTLY PROHIBITED.
****************************************************************************/

/* USER CODE BEGIN Includes */
#include "../../../Core/Maximatecc/Inc/Common.h"
#include "../../Drivers/Maximatecc/inc/IO_Expander_TCA9535.h"
#include "../../Drivers/Maximatecc/inc/LED.h"
/* USER CODE END Includes */
#define   MAX_PLATFORM_SUPPORTED_LED	2


/** @brief Led Initialize
 *
 *  This function iniitializes the LED IO Expander
 *  IO pins as Output.
 *
 *  @param  led_num [IN]: LED number
 *
 *
 *  @return 0 : PS_SUCCESS
 *          1 : PS_FAILURE
 */
int8_t Led_Init(I2C_HandleTypeDef *hi2c)
{
	HAL_StatusTypeDef ret;
	TCA9535Regs Regs;

	/* Get the IO Expander Semaphore */
	TakeIOExpanderLock();

	/* Read the Config reg from the IO Expander */
	ret = TCA9535ReadConfig(hi2c, &Regs, TCA9535_2_ADDRESS);

	if(HAL_ERROR == ret)
		return PS_FAILURE;

	/* Make the LED Pins as output */
   	Regs.Config.Port.P1.bit.B3 = 0;
   	Regs.Config.Port.P1.bit.B4 = 0;

	/* Set the LED IO pins status into the IO Expander */
	ret = TCA9535WriteConfig(hi2c, &Regs, TCA9535_2_ADDRESS);

	/* Read the Polarity Reg from the IO Expander */
	ret = TCA9535ReadPolarity(hi2c, &Regs, TCA9535_2_ADDRESS);
	if(HAL_ERROR == ret)
		return PS_FAILURE;

	/* Make the LED Pins as output */
   	Regs.PolarityInversion.Port.P1.bit.B3 = 0;
   	Regs.PolarityInversion.Port.P1.bit.B4 = 0;

	/* Set the LED IO pins status into the IO Expander */
	ret = TCA9535WritePolarity(hi2c, &Regs, TCA9535_2_ADDRESS);

	/* Release the Semaphore */
	GiveIOExpanderLock();

	return PS_SUCCESS;
}

/** @brief Led Off
 *
 *  This function is used for turn off LED.
 *
 *  @param  led_num [IN]: LED number
 *
 *
 *  @return 0 : PS_SUCCESS
 *          1 : PS_FAILURE
 */
int8_t Led_Off(I2C_HandleTypeDef *hi2c, uint8_t led_num)
{
	HAL_StatusTypeDef ret;
	TCA9535Regs Regs;

	if (!((AMBER_LED == led_num) || (RED_LED == led_num)))
		return PS_FAILURE;

	memset(&Regs, 0x00, sizeof(Regs));
	/* Get the IO Expander Semaphore */
	TakeIOExpanderLock();

	/* Read the Output from the IO Expander */
	ret = TCA9535ReadOutput(hi2c, &Regs, TCA9535_2_ADDRESS);

	if(HAL_ERROR == ret)
		return PS_FAILURE;
	HAL_Delay(10);

	/* Set the Amber / Red LED GPIO in IO Expander pin */
    if (AMBER_LED == led_num)
    	Regs.Output.Port.P1.bit.B4 = 0;
    else
    if (RED_LED == led_num)
    	Regs.Output.Port.P1.bit.B3 = 0;

	/* Set the LED IO pins status into the IO Expander */
    ret = TCA9535WriteOutput(hi2c, &Regs, TCA9535_2_ADDRESS);
	if(HAL_ERROR == ret)
		return PS_FAILURE;
	HAL_Delay(10);

	/* Release the Semaphore */
	GiveIOExpanderLock();

	return PS_SUCCESS;
}

/** @brief Led On
 *
 *  This function is used for turn on LED.
 *
 *  @param  led_num [IN]: LED number
 *
 *
 *  @return 0 : PS_SUCCESS
 *          1 : PS_FAILURE
 */
int8_t Led_On(I2C_HandleTypeDef *hi2c, uint8_t led_num)
{
	HAL_StatusTypeDef ret;
	TCA9535Regs Regs;

	if (!((AMBER_LED == led_num) || (RED_LED == led_num)))
		return PS_FAILURE;

	memset(&Regs, 0x00, sizeof(Regs));
	/* Get the IO Expander Semaphore */
	TakeIOExpanderLock();

	/* Read the Output from the IO Expander */
	ret = TCA9535ReadOutput(hi2c, &Regs, TCA9535_2_ADDRESS);
	if(HAL_ERROR == ret)
		return PS_FAILURE;

	HAL_Delay(10);

    if (AMBER_LED == led_num)
    	Regs.Output.Port.P1.bit.B4 = 1;
    else
    if (RED_LED == led_num)
    	Regs.Output.Port.P1.bit.B3 = 1;

	/* Set the LED IO pins status into the IO Expander */
    ret = TCA9535WriteOutput(hi2c, &Regs, TCA9535_2_ADDRESS);

	if(HAL_ERROR == ret)
		return PS_FAILURE;
	HAL_Delay(10);
	memset(&Regs, 0x00, sizeof(Regs));

	/* Read the Output from the IO Expander */
	ret = TCA9535ReadOutput(hi2c, &Regs, TCA9535_2_ADDRESS);
	if(HAL_ERROR == ret)
		return PS_FAILURE;
	HAL_Delay(10);

	/* Release the Semaphore */
	GiveIOExpanderLock();

	return PS_SUCCESS;
}

/**************** (C) COPYRIGHT 2022 Maxima Technologies *******************/
/***************************** END OF FILE *********************************/
