/****************************************************************************
              Copyright (c) 2022 Maxima Technologies and Systems
                           All Rights Reserved

   File Name          : Power_Mode.c
   Author             : Arjun
   Version            : V0.0.0.1
   Origination Date   : 11/02/2022
   Description        : C Source File.
   Environment        : Independent
   Processor:         : STM32H743IIT6

-----------------------------------------------------------------------------
   Description:

   This file contains the Power Mode platform service initialization

-----------------------------------------------------------------------------
	File Revision History:
-----------------------------------------------------------------------------
   Version             Date                 Programmer
   0.0.0.1             11/02/2022			Arjun

   Notes:              - Initial Version
-----------------------------------------------------------------------------
*****************************************************************************
THIS FILE CONTAINS INTELLECTUAL PROPERTY OF MAXIMA TECHNOLOGIES AND SYSTEMS.
UNAUTHORIZED REDISTRIBUTION OF THIS SOURCE CODE, IN WHOLE OR PART, WITHOUT
THE EXPRESS WRITTEN PERMISSION OF MAXIMA TECHNOLOGIES IS STRICTLY PROHIBITED.
****************************************************************************/

/* USER CODE BEGIN Includes */
#include "../../../Core/Maximatecc/Inc/Common.h"
/* USER CODE END Includes */
#if (SDK_SERVICE_PM == PS_ENABLE)

extern void SystemClock_Config(void);
extern void PeriphCommonClock_Config(void);

void PowerDown_pin_reset()
{
	HAL_GPIO_WritePin(GPIOD, GPIO_PIN_7, GPIO_PIN_RESET);
}

void PowerDown_pin_set()
{
	HAL_GPIO_WritePin(GPIOD, GPIO_PIN_7, GPIO_PIN_SET);
}

void fdcan_wakeup()
{

	GPIO_InitTypeDef GPIO_InitStruct = {0};

	__HAL_RCC_GPIOA_CLK_ENABLE();
	__HAL_RCC_GPIOB_CLK_ENABLE();
	__HAL_RCC_GPIOG_CLK_ENABLE();

	GPIO_InitStruct.Pin = GPIO_PIN_11;
	GPIO_InitStruct.Mode = GPIO_MODE_IT_RISING_FALLING;
	GPIO_InitStruct.Pull = GPIO_NOPULL;
	GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_LOW;
	HAL_GPIO_Init(GPIOA, &GPIO_InitStruct);

	GPIO_InitStruct.Pin = GPIO_PIN_12;
	GPIO_InitStruct.Mode = GPIO_MODE_IT_RISING_FALLING;
	GPIO_InitStruct.Pull = GPIO_NOPULL;
	//GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_LOW;
	HAL_GPIO_Init(GPIOB, &GPIO_InitStruct);

	/*Configure GPIO pin : PG5 */
	GPIO_InitStruct.Pin = GPIO_PIN_5;
	GPIO_InitStruct.Mode = GPIO_MODE_OUTPUT_PP;
	GPIO_InitStruct.Pull = GPIO_NOPULL;
	GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_LOW;
	HAL_GPIO_Init(GPIOG, &GPIO_InitStruct);

	/*Configure GPIO pin Output Level */
	HAL_GPIO_WritePin(GPIOG, GPIO_PIN_5, GPIO_PIN_SET);

}

/** @brief Sleep Mode
 *
 *  This function takes the controller to sleep mode.
 *
 *  @param  None
 *
 *  @return  None
 *
 */
void SleepMode(void)
{
	/* De Initialize the Platform Service Driver */
	PM_DeInit();

	/* Configure the Wakeup Source */
	PM_Conf_Wakeup_Source();

    /* Suspend Tick increment to prevent wakeup by Systick interrupt.
     Otherwise the Systick interrupt will wake up the device within 1ms (HAL time base) */
    HAL_SuspendTick();

    /* Request to enter SLEEP mode */
    HAL_PWR_EnterSLEEPMode(PWR_LOWPOWERREGULATOR_ON, PWR_SLEEPENTRY_WFI);

    /* Resume Tick interrupt if disabled prior to sleep mode entry*/
    HAL_ResumeTick();

    HAL_Delay(2000);
    /* Re Initialize the Platform Service Driver */
    PM_ReInit();

}

/** @brief Stop Mode
 *
 *  This function takes the controller to stop mode.
 *
 *  @param  None
 *
 *  @return  None
 *
 */
void StopMode(void)
{
	/* De Initialize the Platform Service Driver */
	PM_DeInit();

	/* Configure the Wakeup Source */
	PM_Conf_Wakeup_Source();

#if(CAN_WAKEUP_SOURCE == PS_ENABLE)
	fdcan_wakeup();
#endif

	HAL_PWREx_EnableFlashPowerDown();

	/* Suspend Tick increment to prevent wakeup by Systick interrupt.
	 * Otherwise the Systick interrupt will wake up the device within 1ms (HAL time base) */
	HAL_SuspendTick();

	/* Request to enter SLEEP mode */
	HAL_PWR_EnterSTOPMode(PWR_LOWPOWERREGULATOR_ON, PWR_SLEEPENTRY_WFI);

	/* Configure the system clock */
	SystemClock_Config();

    PeriphCommonClock_Config();
#if 0
	/* Resume Tick interrupt if disabled prior to sleep mode entry*/
	HAL_ResumeTick();

	/* Reinitialize Platform Drivers*/
	PM_ReInit();
#endif

	/* Resume Tick interrupt if disabled prior to sleep mode entry*/
	HAL_ResumeTick();

	/* Give some delay before software reset */
	HAL_Delay(10);

	/* Software Reset */
	NVIC_SystemReset();

	while(1)
	{

	}
}


/** @brief Standby Mode
 *
 *  This function takes the controller to standby mode.
 *
 *  @param  None
 *
 *  @return  None
 *
 */
void StandbyMode(void)
{
	/* De Initialize the Platform Service Driver */
	PM_DeInit();

	/* Configure the Wakeup Source */
	PM_Conf_Wakeup_Source();

	/* Enter STANDBY mode */
	HAL_PWR_EnterSTANDBYMode();
}
#endif	//SDK_SERVICE_PM

/**************** (C) COPYRIGHT 2022 Maxima Technologies *******************/
/***************************** END OF FILE *********************************/
