/****************************************************************************
Copyright (c) 2012 Maxima Technologies and Systems
All Rights Reserved
File Name : stm32746g_discovery_camera.c
Author : upender chennoju
Version : V0.0.0.1
Origination Date : 10/15/2019
Description : C Source File.
Environment : Independant
Processor: : Independant
-----------------------------------------------------------------------------
Description:

-----------------------------------------------------------------------------
File Revision History:
-----------------------------------------------------------------------------
Version Date Programmer
0.0.0.0 04/04/2011 Name Last_Name
Notes: - Initial Version
-----------------------------------------------------------------------------
Version Date Programmer
0.0.0.1 31/03/2011 Name Last_Name
Notes: - Second Version
-----------------------------------------------------------------------------
*****************************************************************************
THIS FILE CONTAINS INTELLECTUAL PROPERTY OF MAXIMA TECHNOLOGIES AND SYSTEMS.
UNAUTHORIZED REDISTRIBUTION OF THIS SOURCE CODE, IN WHOLE OR PART, WITHOUT
THE EXPRESS WRITTEN PERMISSION OF MAXIMA TECHNOLOGIES IS STRICTLY PROHIBITED.
****************************************************************************/
/* Includes ---------------------------------------------------------------*/
#include "../../Core/Maximatecc/Inc/common.h"
#include "../inc/camera.h"

#if (SDK_SERVICE_CAMERA == PS_ENABLE)
extern pvt_variable pv;


#define CAMERA_X0_ORIGIN_AFTER_BLANK_PERIOD   272
#define CAMERA_Y0_ORIGIN_AFTER_BLANK_PERIOD   15


#define CAMERA_FULL_X0_SIZE  600
#define CAMERA_FULL_Y0_SIZE  230

uint8_t HFlip = 0;
uint8_t VFlip = 0;
uint16_t g_x0_width;
uint16_t g_y0_height;
uint16_t g_lcd_x0_origin;
uint16_t g_lcd_y0_origin;
uint32_t lcd_offset = 0;
uint8_t  uiProcFrames;
void config_write_one_c(uint16_t DeviceAddr);

/* Private typedef --------------------------------------------------------*/
extern DCMI_HandleTypeDef  hdcmi;//hal
CAMERA_DrvTypeDef   *camera_drv;

/* Private define ---------------------------------------------------------*/
#define CAMERA_FRAME_BUFFER               CAMERA_IMAGE_BUFFER
#define LCD_FRAME_BUFFER                  LAYER_1_FRAMEBUFFER
#define LCD_TEMP_BUFFER                   TEMP_FRAMEBUFFER

//#define SCALE 6
#define Cr_R Cb_B
/* Private macro ----------------------------------------------------------*/
/* Private variables ------------------------------------------------------*/

static uint32_t CameraCurrentResolution; /* Camera current resolution naming (QQVGA, VGA, ...) */
static uint32_t CameraHwAddress; /* Camera module I2C HW address */
extern volatile uint8_t detected;
extern volatile uint16_t sReconnected;
extern volatile uint16_t sDisconnected;
extern volatile uint16_t lock_Counter;
extern volatile uint16_t lock_Counter1;
extern volatile uint16_t lock_Counter2;
extern volatile uint16_t lock_Counter3;

/* Private functions declaration ------------------------------------------*/
static uint32_t GetSize(uint32_t resolution);
extern void showDeviceLoading(uint32_t duration);

/* PUBLIC functions definition --------------------------------------------*/
uint32_t camera_loader_wait_counter=0;
uint32_t camera_loader_wait_offset =150;

/* PRIVATE functions definition -------------------------------------------*/

/** @brief PowerON the camera
 *
 *  This function is used for powering ON and initializing the ADV7180KCP32Z
 *
 *
 *  @param  none
 *
 *  @return none
 *
 */
void CAMERA_PowerON()
{
  BSP_CAMERA_PwrUp();

  /* Intializing the Video Decoder */
  adv7180_Init(CAMERA_I2C_READ_ADDRESS);
  adv7180_Config(CAMERA_I2C_WRITE_ADDRESS,0,0,0);
  HAL_Delay(10);
}

/****************************************************************************
Function Name: . BSP_CAMERA_Init()
Description: . Initializes the camera.
Parameters: .   Resolution : camera sensor requested resolution (x, y) : standard resolution
                naming QQVGA, QVGA, VGA ...
Return:         CAMERA_STATUS
Dependencies: .- stm32746g_discovery.c
- stm32f7xx_hal_dcmi.c
- stm32f7xx_hal_dma.c
- stm32f7xx_hal_gpio.c
- stm32f7xx_hal_cortex.c
- stm32f7xx_hal_rcc_ex.h
- ov9655.c
Assumptions: . .
Limitations: .
Exception: .
Handling: .
****************************************************************************/
uint8_t BSP_CAMERA_Init()
{

  DCMI_HandleTypeDef *phdcmi;
  HAL_StatusTypeDef   status;
  bLCD_Hor_Flip= LCD_HORZ_FILP;
  bLCD_Ver_Flip= LCD_VERT_FILP;
  Set_DL(CAMERA_VIDEO_FLIP_HORIZONTAL, &bLCD_Hor_Flip);
  Set_DL(CAMERA_VIDEO_FLIP_VERTICAL, &bLCD_Ver_Flip);

  /*** Get the DCMI handle structure ***/
  phdcmi = &hdcmi;
  /*** Configures the DCMI to interface with the camera module ***/
  phdcmi->Init.CaptureRate      = DCMI_CR_ALTERNATE_4_FRAME ;
  phdcmi->Init.HSPolarity       = DCMI_HSPOLARITY_LOW;
  phdcmi->Init.SynchroMode      = DCMI_SYNCHRO_HARDWARE;
  phdcmi->Init.VSPolarity       = DCMI_VSPOLARITY_HIGH;
  phdcmi->Init.ExtendedDataMode = DCMI_EXTEND_DATA_8B;
  phdcmi->Init.PCKPolarity      = DCMI_PCKPOLARITY_RISING;
  phdcmi->Instance              = DCMI;

  BSP_CAMERA_MspInit(&hdcmi, NULL);
  status = HAL_DCMI_Init(phdcmi);
  if (HAL_OK == status)
	  CAMERA_PowerON();
  return status;
}
/****************************************************************************
Function Name: . BSP_CAMERA_DeInit()
Description: .  DeInitializes the camera.
Parameters: . 
Dependencies: .
Assumptions: . .
Limitations: .
Exception: .
Handling: .
Return: . Camera status
****************************************************************************/
uint8_t BSP_CAMERA_DeInit(void)
{ 
  hdcmi.Instance              = DCMI;
  HAL_DCMI_DeInit(&hdcmi);
  BSP_CAMERA_MspDeInit(&hdcmi, NULL);
  return CAMERA_OK;
}

/****************************************************************************
Function Name: . BSP_CAMERA_ContinuousStart()
Description: . Starts the camera capture in continuous mode.
Parameters: .buff: pointer to the camera output buffer
Dependencies: .
Assumptions: . .
Limitations: .
Exception: .
Handling: .
****************************************************************************/
void BSP_CAMERA_ContinuousStart(uint8_t *buffer)
{
  /* Start the camera capture */
  HAL_DCMI_Start_DMA(&hdcmi, DCMI_MODE_CONTINUOUS, (uint32_t)buffer,
                     GetSize(CameraCurrentResolution));
}

/****************************************************************************
Function Name: . BSP_CAMERA_SnapshotStart()
Description: .  Starts the camera capture in snapshot mode.
Parameters: . buff: pointer to the camera output buffer
Dependencies: .
Assumptions: . .
Limitations: .
Exception: .
Handling: .
****************************************************************************/
void BSP_CAMERA_SnapshotStart(uint8_t *buff)
{
  /* Start the camera capture */
  HAL_DCMI_Start_DMA(&hdcmi, DCMI_MODE_SNAPSHOT,
                     (uint32_t)buff, GetSize(CameraCurrentResolution));
}

/****************************************************************************
Function Name: . BSP_CAMERA_Suspend()
Description: . Suspend the CAMERA capture
Parameters: .
Dependencies: .
Assumptions: . .
Limitations: .
Exception: .
Handling: .
****************************************************************************/
void BSP_CAMERA_Suspend(void)
{
  /* Suspend the Camera Capture */
  HAL_DCMI_Suspend(&hdcmi);
}

/****************************************************************************
Function Name: . BSP_CAMERA_Resume()
Description: . Resume the CAMERA capture 
Parameters: .
Dependencies: .
Assumptions: . .
Limitations: .
Exception: .
Handling: .
****************************************************************************/
void BSP_CAMERA_Resume(void) 
{
  /* Start the Camera Capture */
  HAL_DCMI_Resume(&hdcmi);
}

/****************************************************************************
Function Name: . BSP_CAMERA_Stop()
Description: . Stop the CAMERA capture 
Parameters: .
Dependencies: .
Assumptions: . .
Limitations: .
Exception: .
Handling: .
Return: . Camera status
****************************************************************************/
uint8_t BSP_CAMERA_Stop(void) 
{
  uint8_t status = CAMERA_ERROR;
  
  if(HAL_DCMI_Stop(&hdcmi) == HAL_OK)
  {
    status = CAMERA_OK;
  }
  
  /* Set Camera in Power Down */
  //BSP_CAMERA_PwrDown();
  
  return status;
}

/****************************************************************************
Function Name: . BSP_CAMERA_PwrUp()
Description: . CAMERA Reset sequence 
Parameters: . 
Dependencies: .
Assumptions: . .
Limitations: .
Exception: .
Handling: .
****************************************************************************/
void BSP_CAMERA_PwrUp(void)
{
  HAL_GPIO_WritePin(GPIOC,GPIO_PIN_13,(GPIO_PinState)BSP_IO_PIN_RESET);
  HAL_Delay(100);   /* RST and XSDN signals asserted during 100ms */
  HAL_GPIO_WritePin(GPIOC,GPIO_PIN_13,(GPIO_PinState)BSP_IO_PIN_SET);
  HAL_Delay(5);     /* RST de-asserted during 3ms */
}

/****************************************************************************
Function Name: . BSP_CAMERA_PwrDown()
Description: . CAMERA power down
Parameters: .
Dependencies: .
Assumptions: . .
Limitations: .
Exception: .
Handling: .
****************************************************************************/
void BSP_CAMERA_PwrDown(void)
{
	HAL_DMA_DeInit(hdcmi.DMA_Handle);
	HAL_DCMI_DeInit(&hdcmi);

	memset(&hdcmi, 0x00, sizeof(hdcmi));

	HAL_GPIO_WritePin(GPIOC,GPIO_PIN_13,(GPIO_PinState)BSP_IO_PIN_RESET);
	HAL_Delay(100);   /* RST and XSDN signals asserted during 100ms */
}

/****************************************************************************
Function Name: . BSP_CAMERA_ContrastBrightnessConfig()
Description: . Configures the camera contrast and brightness.
Parameters: . contrast_level: Contrast level
*          This parameter can be one of the following values:
*          CAMERA_CONTRAST_LEVEL4: for contrast +2
*          CAMERA_CONTRAST_LEVEL3: for contrast +1
*          CAMERA_CONTRAST_LEVEL2: for contrast  0
*          CAMERA_CONTRAST_LEVEL1: for contrast -1
*          CAMERA_CONTRAST_LEVEL0: for contrast -2
* @param  brightness_level: Contrast level
*          This parameter can be one of the following values:
*          CAMERA_BRIGHTNESS_LEVEL4: for brightness +2
*          CAMERA_BRIGHTNESS_LEVEL3: for brightness +1
*          CAMERA_BRIGHTNESS_LEVEL2: for brightness  0
*          CAMERA_BRIGHTNESS_LEVEL1: for brightness -1
*          CAMERA_BRIGHTNESS_LEVEL0: for brightness -2  
Dependencies: .
Assumptions: . .
Limitations: .
Exception: .
Handling: .
****************************************************************************/
void BSP_CAMERA_ContrastBrightnessConfig(uint32_t contrast_level, 
                                         uint32_t brightness_level)
{
  if(camera_drv->Config != NULL)
  {
    camera_drv->Config(CameraHwAddress, CAMERA_CONTRAST_BRIGHTNESS, 
                       contrast_level, brightness_level);
  }  
}

/****************************************************************************
Function Name: . BSP_CAMERA_BlackWhiteConfig()
Description: . Configures the camera white balance.
Parameters: . Mode: black_white mode
*          This parameter can be one of the following values:
*             CAMERA_BLACK_WHITE_BW
*              CAMERA_BLACK_WHITE_NEGATIVE
*             CAMERA_BLACK_WHITE_BW_NEGATIVE
*             CAMERA_BLACK_WHITE_NORMAL  
Dependencies: .
Assumptions: . .
Limitations: .
Exception: .
Handling: .
****************************************************************************/
void BSP_CAMERA_BlackWhiteConfig(uint32_t Mode)
{
  if(camera_drv->Config != NULL)
  {
    camera_drv->Config(CameraHwAddress, CAMERA_BLACK_WHITE, Mode, 0);
  }  
}

/****************************************************************************
Function Name: . BSP_CAMERA_ColorEffectConfig()
Description: . Configures the camera color effect.
Parameters: .  Effect: Color effect
*          This parameter can be one of the following values:
*            CAMERA_COLOR_EFFECT_ANTIQUE               
*              CAMERA_COLOR_EFFECT_BLUE        
*              CAMERA_COLOR_EFFECT_GREEN    
*             CAMERA_COLOR_EFFECT_RED  
Dependencies: .
Assumptions: . .
Limitations: .
Exception: .
Handling: .
****************************************************************************/
void BSP_CAMERA_ColorEffectConfig(uint32_t Effect)
{
  if(camera_drv->Config != NULL)
  {
    camera_drv->Config(CameraHwAddress, CAMERA_COLOR_EFFECT, 
                       Effect, 0);
  }  
}

/****************************************************************************
Function Name: . GetSize()
Description: . Get the capture size in pixels unit.
Parameters: . resolution: the current resolution.
Dependencies: .
Assumptions: . .
Limitations: .
Exception: .
Handling: .
Return: . capture size in pixels unit.
****************************************************************************/
static uint32_t GetSize(uint32_t resolution)
{
  uint32_t size = 0;
  /* Get capture size */
  switch (resolution)
  {
  case CAMERA_R160x120:
    {
      size =  0x2580;
    }
    break;
  case CAMERA_R320x240:
    {
      size =  0x9600;
    }
    break;
  case CAMERA_R480x272:
    {
      size =  ((CAMERA_FULL_X0_SIZE*CAMERA_FULL_Y0_SIZE)/2);
    }
    break;
  case CAMERA_R640x480:
    {
      size =  0x25800;
    }
    break;
  default:
    {
      break;
    }
  }
  return size;
}

/****************************************************************************
Function Name: . BSP_CAMERA_MspInit()
Description: . Initializes the DCMI MSP.
Parameters: . hdcmi: HDCMI handle 
  Params
Dependencies: .
Assumptions: . .
Limitations: .
Exception: .
Handling: .
****************************************************************************/
__weak void BSP_CAMERA_MspInit(DCMI_HandleTypeDef *hdcmi, void *Params)
{
  static DMA_HandleTypeDef hdma_eval;
  GPIO_InitTypeDef gpio_init_structure;
  
  /*** Enable peripherals and GPIO clocks ***/
  /* Enable DCMI clock */
  __HAL_RCC_DCMI_CLK_ENABLE();
  
  /* Enable DMA2 clock */
  __HAL_RCC_DMA2_CLK_ENABLE();
  
  /* Enable GPIO clocks */
  __HAL_RCC_GPIOA_CLK_ENABLE();
  __HAL_RCC_GPIOB_CLK_ENABLE();
  __HAL_RCC_GPIOC_CLK_ENABLE();
  __HAL_RCC_GPIOD_CLK_ENABLE();
  __HAL_RCC_GPIOI_CLK_ENABLE();
  __HAL_RCC_GPIOG_CLK_ENABLE();
  
  /* Enabling DCMI Peripheral*/  
  gpio_init_structure.Pin       = GPIO_PIN_4 |GPIO_PIN_6|  GPIO_PIN_9 | GPIO_PIN_10;
  gpio_init_structure.Mode      = GPIO_MODE_AF_PP;
  gpio_init_structure.Pull      = GPIO_PULLUP;
  gpio_init_structure.Speed     = GPIO_SPEED_FREQ_HIGH;
  gpio_init_structure.Alternate = GPIO_AF13_DCMI;
  HAL_GPIO_Init(GPIOA, &gpio_init_structure);

  gpio_init_structure.Pin       = GPIO_PIN_7;
  gpio_init_structure.Mode      = GPIO_MODE_AF_PP;
  gpio_init_structure.Pull      = GPIO_PULLUP;
  gpio_init_structure.Alternate = GPIO_AF13_DCMI;
  HAL_GPIO_Init(GPIOB, &gpio_init_structure);
  
  gpio_init_structure.Pin       = GPIO_PIN_8  | GPIO_PIN_9 | GPIO_PIN_11;
  gpio_init_structure.Mode      = GPIO_MODE_AF_PP;
  gpio_init_structure.Pull      = GPIO_PULLUP;
  gpio_init_structure.Alternate = GPIO_AF13_DCMI;
  HAL_GPIO_Init(GPIOC, &gpio_init_structure);
  
  gpio_init_structure.Pin       = GPIO_PIN_3;
  gpio_init_structure.Mode      = GPIO_MODE_AF_PP;
  gpio_init_structure.Pull      = GPIO_PULLUP;
  gpio_init_structure.Alternate = GPIO_AF13_DCMI;
  HAL_GPIO_Init(GPIOD, &gpio_init_structure);
  
  gpio_init_structure.Pin       = GPIO_PIN_6 | GPIO_PIN_7;
  gpio_init_structure.Mode      = GPIO_MODE_AF_PP;
  gpio_init_structure.Pull      = GPIO_PULLUP;
  gpio_init_structure.Alternate = GPIO_AF13_DCMI;
  HAL_GPIO_Init(GPIOI, &gpio_init_structure);
  
  /* video reset VID_RST Pin configuration */
  gpio_init_structure.Pin       = GPIO_PIN_13;     
  gpio_init_structure.Mode      = GPIO_MODE_OUTPUT_PP;
  HAL_GPIO_Init(GPIOC, &gpio_init_structure);
  
  /*** Configure the DMA ***/
  /* Set the parameters to be configured */
//  hdma_eval.Init.Channel             = DMA_CHANNEL_1;
  hdma_eval.Init.Direction           = DMA_PERIPH_TO_MEMORY;
  hdma_eval.Init.PeriphInc           = DMA_PINC_DISABLE; 
  hdma_eval.Init.MemInc              = DMA_MINC_ENABLE;  
  hdma_eval.Init.PeriphDataAlignment = DMA_PDATAALIGN_WORD;  
  hdma_eval.Init.MemDataAlignment    = DMA_MDATAALIGN_WORD;
  hdma_eval.Init.Mode                = DMA_CIRCULAR;   
  hdma_eval.Init.Priority            = DMA_PRIORITY_HIGH; 
  hdma_eval.Init.FIFOMode            = DMA_FIFOMODE_DISABLE; 
  
  //  hdma_eval.Init.FIFOMode            = DMA_FIFOMODE_ENABLE;         
  hdma_eval.Init.FIFOThreshold       = DMA_FIFO_THRESHOLD_FULL;
  hdma_eval.Init.MemBurst            = DMA_MBURST_SINGLE;
  hdma_eval.Init.PeriphBurst         = DMA_PBURST_SINGLE; 
  
  hdma_eval.Instance =DMA2_Stream7;// DMA2_Stream1;
  
  /* Associate the initialized DMA handle to the DCMI handle */
  __HAL_LINKDMA(hdcmi, DMA_Handle, hdma_eval);
  
  /*** Configure the NVIC for DCMI and DMA ***/
  /* NVIC configuration for DCMI transfer complete interrupt */
  HAL_NVIC_SetPriority(DCMI_IRQn, 0x05, 0);   
  HAL_NVIC_EnableIRQ(DCMI_IRQn);  
  
  /* NVIC configuration for DMA2D transfer complete interrupt */
  HAL_NVIC_SetPriority(DMA2_Stream7_IRQn, 0x05, 0);  
  HAL_NVIC_EnableIRQ(DMA2_Stream7_IRQn);
  
  /* Configure the DMA stream */
  HAL_DMA_Init(hdcmi->DMA_Handle);  
}

/****************************************************************************
Function Name: . BSP_CAMERA_MspDeInit()
Description: . DeInitializes the DCMI MSP.
Parameters: . hdcmi: HDMI handle 
Params
Dependencies: .
Assumptions: . .
Limitations: .
Exception: .
Handling: .
****************************************************************************/
__weak void BSP_CAMERA_MspDeInit(DCMI_HandleTypeDef *hdcmi, void *Params)
{
  /* Disable NVIC  for DCMI transfer complete interrupt */
  HAL_NVIC_DisableIRQ(DCMI_IRQn);  
  
  /* Disable NVIC for DMA2 transfer complete interrupt */
  HAL_NVIC_DisableIRQ(DMA2_Stream1_IRQn);
  
  /* Configure the DMA stream */
  HAL_DMA_DeInit(hdcmi->DMA_Handle);  
  
  /* Disable DCMI clock */
  __HAL_RCC_DCMI_CLK_DISABLE();
  
  /* GPIO pins clock and DMA clock can be shut down in the application 
  by surcharging this __weak function */ 
}

/****************************************************************************
Function Name: . HAL_DCMI_LineEventCallback()
Description: . Line event callback
Parameters: .  hdcmi: pointer to the DCMI handle  
Dependencies: .
Assumptions: . .
Limitations: .
Exception: .
Handling: .
****************************************************************************/
void HAL_DCMI_LineEventCallback(DCMI_HandleTypeDef *hdcmi)
{        
  BSP_CAMERA_LineEventCallback();
}


/****************************************************************************
Function Name: . BSP_CAMERA_LineEventCallback()
Description: . Line Event callback.
Parameters: .
Dependencies: .
Assumptions: . .
Limitations: .
Exception: .
Handling: .
****************************************************************************/
__weak void BSP_CAMERA_LineEventCallback(void)
{
  /* NOTE : This function Should not be modified, when the callback is needed,
  the HAL_DCMI_LineEventCallback could be implemented in the user file
  */
}

/****************************************************************************
Function Name: . HAL_DCMI_VsyncEventCallback()
Description: .  VSYNC event callback
Parameters: .  hdcmi: pointer to the DCMI handle  
Dependencies: .
Assumptions: . .
Limitations: .
Exception: .
Handling: .
****************************************************************************/
void HAL_DCMI_VsyncEventCallback(DCMI_HandleTypeDef *hdcmi)
{        
  BSP_CAMERA_VsyncEventCallback();
}

/****************************************************************************
Function Name: . BSP_CAMERA_VsyncEventCallback()
Description: . VSYNC Event callback.
Parameters: . 
Dependencies: .
Assumptions: . .
Limitations: .
Exception: .
Handling: .
****************************************************************************/
__weak void BSP_CAMERA_VsyncEventCallback(void)
{
  /* NOTE : This function Should not be modified, when the callback is needed,
  the HAL_DCMI_VsyncEventCallback could be implemented in the user file
  */
	lcd_offset =0;
}

/****************************************************************************
Function Name: . HAL_DCMI_FrameEventCallback()
Description: .  Frame event callback
Parameters: . hdcmi: pointer to the DCMI handle  
Dependencies: .
Assumptions: . .
Limitations: .
Exception: .
Handling: .
****************************************************************************/
void HAL_DCMI_FrameEventCallback(DCMI_HandleTypeDef *hdcmi)
{
	static uint8_t uiFrameProcCnt = 0;
	static uint8_t uiFrameDropCnt = 0;
	if (uiProcFrames >= uiFrameProcCnt++)
	{
		BSP_CAMERA_FrameEventCallback();
	} else
	if (DROP_FRAMES == uiFrameDropCnt++)
	{
		uiFrameProcCnt = 0;
		uiFrameDropCnt = 0;
	}
}

//PAL Scale Full Screen
/****************************************************************************
Function Name: . PALYCBCR422ToARGB()
Description: .  Converts a YCBCR422 to an ARGB8888 pixel format using PAL video input
*         4:4:4   ( 3 byte -> 1 pixel ) ( image from online )
*         4:2:2   ( 4 Byte 2 Y, 1 Cb 1 Cr --> 2 pixel )
                  ( 4 bytes --> 2 pixel  ) ( video )
Parameters: .in: Pointer to source buffer
out: pointer to Output color
pixels : number of pixels
is_test : 0 for camera output
                   1 for testing color output
Dependencies: .
Assumptions: . .
Limitations: .
Exception: .
Handling: .
****************************************************************************/
void PALYCBCR422ToARGBresize(uint8_t* out, const uint8_t* in, bool bHF)
{
  register int y, u, v, yy, vr, ug, vg, ub;
  register int r1, g1, b1,r2, g2, b2, iPointNext=1;
  int iBlendColPixel=0;
  uint16_t *tout = (uint16_t *)out;
  if (bHF)
  {
	*tout+= LCD_RES_X;
	iPointNext=-1;
  }

  void oddRGB(uint8_t* in)
  {
  	y = in[1];
  	yy = y << 8;
  	u = in[2] - 128;
  	ug =  88 * u;

  	ub = 454 * u;
  	v = in[0] - 128;
  	vg = 183 * v;
  	vr = 359 * v;

  	r1 = (yy + vr) >> 8;
  	g1 = (yy - ug - vg) >> 8;
  	b1 = (yy + ub) >> 8;

  	if (r1 < 0)
  		r1 = 0;
  	else
  	if (r1 > 255)
  		r1 = 255;

  	if (g1 < 0)
  		g1 = 0;
  		else
  		if (g1 > 255)
  		g1 = 255;

  	if (b1 < 0)
  		b1 = 0;
  	else
  	if (b1 > 255)
  	b1 = 255;
  }
  void pairRGB(uint8_t* in)
  {
  	y = in[3];
  	yy = y << 8;

  	r2 = (yy + vr) >> 8;
  	g2 = (yy - ug - vg) >> 8;
  	b2 = (yy + ub) >> 8;

  	if (r2 < 0)
  		r2 = 0;
  	else
  	if (r2 > 255)
  		r2 = 255;

  	if (g2 < 0)
  		g2 = 0;
  	else
  	if (g2 > 255)
  		g2 = 255;

  	if (b2 < 0)
  		b2 = 0;
  	else
  	if (b2 > 255)
  		b2 = 255;
  }

  for (int iLCDColCouple = 0; iLCDColCouple <= LCD_RES_X; iLCDColCouple+=LCD_PIXEL_SIZE)
  {
	  oddRGB((uint8_t*)in);
      tout[0] = (((uint16_t)r1>>3)<<11) | (((uint16_t)g1>>2)<<5) | (((uint16_t)b1>>3)<<0); //oddPixel
	  tout += iPointNext;
	  //Reduce video size close to LCD width by blending video input column pixel by using bilinear blending interpolation every Column defined by iBlendColPixel
	  if (iLCDColCouple>=iBlendColPixel) //This block merge (blend) two columns in one by average
	  {
		  pairRGB((uint8_t*)in); //Get Pair Pixel by Column
		  in += 2*LCD_PIXEL_SIZE;
		  oddRGB((uint8_t*)in); //Get Odd Pixel by Column
		  tout[0] = ((((uint16_t)((r1+r2)/2))>>3)<<11) | ((((uint16_t)((g1+g2)/2))>>2)<<5) | ((((uint16_t)((b1+b2)/2))>>3)<<0); //oddPixel
		  tout += iPointNext;
		  pairRGB((uint8_t*)in);
		  tout[0] = ((((uint16_t)((r1+r2)/2))>>3)<<11) | ((((uint16_t)((g1+g2)/2))>>2)<<5) | ((((uint16_t)((b1+b2)/2))>>3)<<0); //PairPixel
		  tout += iPointNext;
		  iLCDColCouple++; //Move forward half pair (1 Column)
		  iBlendColPixel+=9; //Skips 4.5 Pairs to then blend
	  } else //This else block copies pair column from video as it is to LCD
	  {
		  pairRGB((uint8_t*)in);
		  tout[0] = (((uint16_t)r2>>3)<<11) | (((uint16_t)g2>>2)<<5) | (((uint16_t)b2>>3)<<0); //pairPixel
		  tout += iPointNext;
	  }
	  in += 2*LCD_PIXEL_SIZE; //move to next 2 input video pixels
  }
}

//NTSC Scale Full Screen
/****************************************************************************
Function Name: . NTSCYCBCR422ToARGBresize()
Description: .  Converts a YCBCR422 to an ARG565 pixel format using NTSC video input
*         4:4:4   ( 3 byte -> 1 pixel ) ( image from online )
*         4:2:2   ( 4 Byte 2 Y, 1 Cb 1 Cr --> 2 pixel )
                  ( 4 bytes --> 2 pixel  ) ( video )
Parameters: .in: Pointer to source buffer
out: pointer to Output color
pixels : number of pixels
is_test : 0 for camera output
                   1 for testing color output
Dependencies: .
Assumptions: . .
Limitations: .
Exception: .
Handling: .
****************************************************************************/
void NTSCYCBCR422ToARGBresize(uint8_t* out, const uint8_t* in, bool bHF)
{
  const int ciaBlendColPixel[3] = {4,4,3};
  register int y, u, v, yy, vr, ug, vg, ub;
  register int r1, g1, b1,r2, g2, b2, iPointNext=1;
  int iBlendColPixel=0,blendcount=0;
  uint16_t *tout = (uint16_t *)out;
  if (bHF)
  {
	*tout+= LCD_RES_X;
	iPointNext=-1;
  }

  void oddRGB(uint8_t* in)
  {
  	y = in[1];
  	yy = y << 8;
  	u = in[2] - 128;
  	ug =  88 * u;

  	ub = 454 * u;
  	v = in[0] - 128;
  	vg = 183 * v;
  	vr = 359 * v;

  	r1 = (yy + vr) >> 8;
  	g1 = (yy - ug - vg) >> 8;
  	b1 = (yy + ub) >> 8;

  	if (r1 < 0)
  		r1 = 0;
  	else
  	if (r1 > 255)
  		r1 = 255;

  	if (g1 < 0)
  		g1 = 0;
  		else
  		if (g1 > 255)
  		g1 = 255;

  	if (b1 < 0)
  		b1 = 0;
  	else
  	if (b1 > 255)
  	b1 = 255;
  }
  void pairRGB(uint8_t* in)
  {
  	y = in[3];
  	yy = y << 8;

  	r2 = (yy + vr) >> 8;
  	g2 = (yy - ug - vg) >> 8;
  	b2 = (yy + ub) >> 8;

  	if (r2 < 0)
  		r2 = 0;
  	else
  	if (r2 > 255)
  		r2 = 255;

  	if (g2 < 0)
  		g2 = 0;
  	else
  	if (g2 > 255)
  		g2 = 255;

  	if (b2 < 0)
  		b2 = 0;
  	else
  	if (b2 > 255)
  		b2 = 255;
  }

  for (int iLCDColCouple = 1; iLCDColCouple <= (LCD_RES_X); iLCDColCouple+=LCD_PIXEL_SIZE)
  {
	  oddRGB((uint8_t*)in); //VCol 1
      tout[0] = (((uint16_t)r1>>3)<<11) | (((uint16_t)g1>>2)<<5) | (((uint16_t)b1>>3)<<0); //oddPixel //LCol 1
	  tout += iPointNext;
	  //Reduce video size close to LCD width by blending video input column pixel by using bilinear blending interpolation every Column defined by iBlendColPixel
	  if (iLCDColCouple>=iBlendColPixel) //This block merge (blend) two columns in one by average
	  	  {
	  		  pairRGB((uint8_t*)in); //VCol 4
	  		  in += 2*LCD_PIXEL_SIZE;
	  		  oddRGB((uint8_t*)in); //Get Odd Pixel by Column
	  		  tout[0] = (((uint16_t)r1>>3)<<11) | (((uint16_t)g1>>2)<<5) | (((uint16_t)b1>>3)<<0); //oddPixel //LCol 1
	 //		  tout[0] = ((((uint16_t)((r1+r2)/2))>>3)<<11) | ((((uint16_t)((g1+g2)/2))>>2)<<5) | ((((uint16_t)((b1+b2)/2))>>3)<<0); //oddPixel
	  		  tout += iPointNext;
	  		  pairRGB((uint8_t*)in);
	  		  tout[0] = (((uint16_t)r2>>3)<<11) | (((uint16_t)g2>>2)<<5) | (((uint16_t)b2>>3)<<0); //pairPixel
	 //		  tout[0] = ((((uint16_t)((r1+r2)/2))>>3)<<11) | ((((uint16_t)((g1+g2)/2))>>2)<<5) | ((((uint16_t)((b1+b2)/2))>>3)<<0); //oddPixel
	  		  tout += iPointNext;
	  		  iLCDColCouple++;
	  		  iBlendColPixel+= ciaBlendColPixel[blendcount];
	  		  if (++blendcount==3) blendcount=0;
	  	  } else //This else block copies pair column from video as it is to LCD
	  {
		  pairRGB((uint8_t*)in);
		  tout[0] = (((uint16_t)r2>>3)<<11) | (((uint16_t)g2>>2)<<5) | (((uint16_t)b2>>3)<<0); //pairPixel
		  tout += iPointNext;
	  }
	  in += 2*LCD_PIXEL_SIZE; //move to next 2 input video pixels
  }
}

//Video Crop Screen
/****************************************************************************
Function Name: . YCBCR422ToARGBcrop()
Description: .  Converts a YCBCR422 to an ARG565 pixel format using NTSC/PAL video input
*         4:4:4   ( 3 byte -> 1 pixel ) ( image from online )
*         4:2:2   ( 4 Byte 2 Y, 1 Cb 1 Cr --> 2 pixel )
                  ( 4 bytes --> 2 pixel  ) ( video )
Parameters: .in: Pointer to source buffer
out: pointer to Output color
pixels : number of pixels
is_test : 0 for camera output
                   1 for testing color output
Dependencies: .
Assumptions: . .
Limitations: .
Exception: .
Handling: .
****************************************************************************/
void YCBCR422ToARGBcrop(uint8_t* out, const uint8_t* in, bool bHF, uint16_t uiLCDOffset_X)
{
  register int y, u, v, yy, vr, ug, vg, ub;
  register int r1, g1, b1,r2, g2, b2, iPointNext=1;
  uint16_t *tout = (uint16_t *)out;
  if (bHF)
  {
	*tout+= LCD_RES_X;
	*tout-= uiLCDOffset_X;
	iPointNext=-1;
  } else
  {
	 *tout+= uiLCDOffset_X;
  };

  void oddRGB(uint8_t* in)
  {
  	y = in[1];
  	yy = y << 8;
  	u = in[2] - 128;
  	ug =  88 * u;

  	ub = 454 * u;
  	v = in[0] - 128;
  	vg = 183 * v;
  	vr = 359 * v;

  	r1 = (yy + vr) >> 8;
  	g1 = (yy - ug - vg) >> 8;
  	b1 = (yy + ub) >> 8;

  	if (r1 < 0)
  		r1 = 0;
  	else
  	if (r1 > 255)
  		r1 = 255;

  	if (g1 < 0)
  		g1 = 0;
  		else
  		if (g1 > 255)
  		g1 = 255;

  	if (b1 < 0)
  		b1 = 0;
  	else
  	if (b1 > 255)
  	b1 = 255;
  }
  void pairRGB(uint8_t* in)
  {
  	y = in[3];
  	yy = y << 8;

  	r2 = (yy + vr) >> 8;
  	g2 = (yy - ug - vg) >> 8;
  	b2 = (yy + ub) >> 8;

  	if (r2 < 0)
  		r2 = 0;
  	else
  	if (r2 > 255)
  		r2 = 255;

  	if (g2 < 0)
  		g2 = 0;
  	else
  	if (g2 > 255)
  		g2 = 255;

  	if (b2 < 0)
  		b2 = 0;
  	else
  	if (b2 > 255)
  		b2 = 255;
  }

  for (int iLCDColCouple = 1; iLCDColCouple <= (LCD_RES_X); iLCDColCouple+=LCD_PIXEL_SIZE)
  {
	  oddRGB((uint8_t*)in); //VCol 1
      tout[0] = (((uint16_t)r1>>3)<<11) | (((uint16_t)g1>>2)<<5) | (((uint16_t)b1>>3)<<0); //oddPixel //LCol 1
	  tout += iPointNext;
	  pairRGB((uint8_t*)in);
	  tout[0] = (((uint16_t)r2>>3)<<11) | (((uint16_t)g2>>2)<<5) | (((uint16_t)b2>>3)<<0); //pairPixel
	  tout += iPointNext;
	  in += 2*LCD_PIXEL_SIZE; //move to next 2 input video pixels
  }
}

void loadCameraToLCDFullScreen(uint16_t LCD_X0, uint16_t LCD_Y0,
		 uint16_t Cam_Cap_line, uint16_t Cam_Cap_Pixel)
{
	    uint32_t cam_offset = 0;
	    uint8_t bNTSC_LCD_Hor_Flip=!bLCD_Ver_Flip; //Horizontal information in PAL standard is reversed
			if (bVidDet_PAL)
			{
				if (bLCD_Hor_Flip) cam_offset = (PS_PAL_CAMERA_Y0_HEIGHT)*(PS_PAL_CAMERA_X0_WIDTH-2) * CAMERA_PIXEL_SIZE+PS_PAL_CAMERA_Y0_HEIGHT_LAST_MISSING_COLS;
				if (bLCD_Hor_Flip && bLCD_Ver_Flip) cam_offset -= 4;
			    for(uint32_t i=0;i<=(PS_PAL_CAMERA_Y0_HEIGHT-1);i++)
			    {
				uint32_t *pFSrc  = (uint32_t*)(CAMERA_FRAME_BUFFER+(cam_offset));
				uint32_t *pFDest = (uint32_t*)((LCD_FRAME_BUFFER + lcd_offset));/*defined in touchgfx*/
				PALYCBCR422ToARGBresize((uint8_t*)pFDest, (uint8_t*)pFSrc,bLCD_Ver_Flip);
				lcd_offset = lcd_offset + ( LCD_RES_X * LCD_PIXEL_SIZE);
				if (bLCD_Hor_Flip) cam_offset -= (PS_PAL_CAMERA_X0_WIDTH * CAMERA_PIXEL_SIZE );
				else cam_offset += (PS_PAL_CAMERA_X0_WIDTH * CAMERA_PIXEL_SIZE ) ;

			    }
			} else
			{
				uint16_t iDupPrevLine=0;
	#if (SDK_CAMERA_STOP_CAN == PS_ENABLE) //Enable SDK_CAMERA_STOP_CAN to permit process Stop Message,
				                        //10mS period time = 80% to catch exit message on osDelay(1)
				                        //Set a higher osDelay or stream exit message continously each 100ms by 1 Second to assure stop camera service
				                        //Adjust settings accordingly to required performance
				osDelay(1);
	#endif

				if (bLCD_Hor_Flip) cam_offset = (PS_NTSC_CAMERA_Y0_HEIGHT)*(PS_NTSC_CAMERA_X0_WIDTH-2) * CAMERA_PIXEL_SIZE+PS_NTSC_CAMERA_Y0_HEIGHT_LAST_MISSING_COLS;
				if (bLCD_Hor_Flip && bLCD_Ver_Flip) cam_offset -= 4+(PS_NTSC_CAMERA_X0_WIDTH-2) * CAMERA_PIXEL_SIZE;

				for(uint32_t i=0;i<=(LCD_RES_Y);i++)
				{
					uint32_t *pFSrc  = (uint32_t*)(CAMERA_FRAME_BUFFER+(cam_offset));
					uint32_t *pFDest = (uint32_t*)((LCD_FRAME_BUFFER + lcd_offset));/*defined in touchgfx*/
					NTSCYCBCR422ToARGBresize((uint8_t*)pFDest, (uint8_t*)pFSrc,bNTSC_LCD_Hor_Flip);
					if (iDupPrevLine++==5) //Duplicate Video Line each six lines to fit screen size
					{
						iDupPrevLine=0;
					} else
					{
						if (bLCD_Hor_Flip) cam_offset -= (PS_NTSC_CAMERA_X0_WIDTH * CAMERA_PIXEL_SIZE );
						else cam_offset += (PS_NTSC_CAMERA_X0_WIDTH * CAMERA_PIXEL_SIZE ) ;
					}
					lcd_offset = lcd_offset + ( LCD_RES_X * LCD_PIXEL_SIZE);
				}
			}
}

void FullScreenDisplay()
{
    /* Crop the image */
	HAL_DCMI_ConfigCrop(&hdcmi,
    		            CAMERA_X0_ORIGIN_AFTER_BLANK_PERIOD,
						CAMERA_Y0_ORIGIN_AFTER_BLANK_PERIOD,
						(CAMERA_FULL_X0_SIZE *2 ) - 1,
						(CAMERA_FULL_Y0_SIZE - 1));
    HAL_DCMI_EnableCROP(&hdcmi);

	/* Start the camera capture */
	HAL_DCMI_Start_DMA(&hdcmi, DCMI_MODE_CONTINUOUS, (uint32_t)CAMERA_IMAGE_BUFFER,
                              ((CAMERA_FULL_X0_SIZE*CAMERA_FULL_Y0_SIZE)/2));

	/* Configure the Alpha blending screen */
	SetWindowPosition(pv.hltdc, 0, 0, CAMERA_FULL_X0_SIZE, CAMERA_FULL_Y0_SIZE, 1);
	HAL_LTDC_SetAlpha_NoReload(pv.hltdc, 255, 1);
}
/* Crop Image and Display as it is */
void loadCameraToLCDAsIts(uint16_t LCD_X0, uint16_t LCD_Y0,
					 uint16_t Cam_Cap_line, uint16_t Cam_Cap_Pixel)
{
    uint32_t cam_offset = uiLCDOffset_Y;
    uint8_t bNTSC_LCD_Hor_Flip=!bLCD_Ver_Flip; //Horizontal information in PAL standard is reversed
		if (bVidDet_PAL)
		{
			if (bLCD_Hor_Flip) cam_offset = (PS_PAL_CAMERA_Y0_HEIGHT)*(PS_PAL_CAMERA_X0_WIDTH-2) * CAMERA_PIXEL_SIZE+PS_PAL_CAMERA_Y0_HEIGHT_LAST_MISSING_COLS;
			if (bLCD_Hor_Flip && bLCD_Ver_Flip) cam_offset -= 4;
		for(uint32_t i=0;i<=(PS_PAL_CAMERA_Y0_HEIGHT-1-uiLCDOffset_Y);i++)
		{
			uint32_t *pFSrc  = (uint32_t*)(CAMERA_FRAME_BUFFER+(cam_offset));
			uint32_t *pFDest = (uint32_t*)((LCD_FRAME_BUFFER + lcd_offset));/*defined in touchgfx*/
			YCBCR422ToARGBcrop((uint8_t*)pFDest, (uint8_t*)pFSrc,bLCD_Ver_Flip,uiLCDOffset_X);
			lcd_offset = lcd_offset + ( LCD_RES_X * LCD_PIXEL_SIZE);
			if (bLCD_Hor_Flip) cam_offset -= (PS_PAL_CAMERA_X0_WIDTH * CAMERA_PIXEL_SIZE );
			else cam_offset += (PS_PAL_CAMERA_X0_WIDTH * CAMERA_PIXEL_SIZE ) ;

		}
		} else
		{
			uint16_t iDupPrevLine=0;
#if (SDK_CAMERA_STOP_CAN ==PS_ENABLE)  //Enable SDK_CAMERA_STOP_CAN to permit process Stop Message,
                                     //10mS period time = 80% to catch exit message on osDelay(1)
                                     //Set a higher osDelay or stream exit message continously each 100ms by 1 Second to assure stop camera service
                                     //Adjust settings accordingly to required performance
			osDelay(1);
#endif

			if (bLCD_Hor_Flip) cam_offset = (PS_NTSC_CAMERA_Y0_HEIGHT)*(PS_NTSC_CAMERA_X0_WIDTH-2) * CAMERA_PIXEL_SIZE+PS_NTSC_CAMERA_Y0_HEIGHT_LAST_MISSING_COLS;
			if (bLCD_Hor_Flip && bLCD_Ver_Flip) cam_offset -= 4+(PS_NTSC_CAMERA_X0_WIDTH-2) * CAMERA_PIXEL_SIZE;
			for(uint32_t i=0;i<=(LCD_RES_Y);i++)
			{
				uint32_t *pFSrc  = (uint32_t*)(CAMERA_FRAME_BUFFER+(cam_offset));
				uint32_t *pFDest = (uint32_t*)((LCD_FRAME_BUFFER + lcd_offset));/*defined in touchgfx*/
				YCBCR422ToARGBcrop((uint8_t*)pFDest, (uint8_t*)pFSrc,bNTSC_LCD_Hor_Flip,uiLCDOffset_X);
				if (iDupPrevLine++==5) //Duplicate Video Line each six lines to fit screen size
				{
					iDupPrevLine=0;
				} else
				{
					if (bLCD_Hor_Flip) cam_offset -= (PS_NTSC_CAMERA_X0_WIDTH * CAMERA_PIXEL_SIZE );
					else cam_offset += (PS_NTSC_CAMERA_X0_WIDTH * CAMERA_PIXEL_SIZE ) ;
				}
				lcd_offset = lcd_offset + ( LCD_RES_X * LCD_PIXEL_SIZE);
			}
		}
}

/****************************************************************************
Function Name: . ConnectedVideo()
Description: . Detect video connected.
Parameters: . None
Dependencies: . None
Assumptions: . .
Limitations: .
Exception: .
Handling: .
****************************************************************************/
bool ConnectedVideo(void)
{
	uint16_t uiStatus_1;
	uint16_t uiStatus_3;
	uiStatus_1 = CAMERA_IO_Read(CAMERA_I2C_WRITE_ADDRESS,   STATUS1);
	uiStatus_1 = uiStatus_1&0x03; //Lock lost
	uiStatus_3 = CAMERA_IO_Read(CAMERA_I2C_WRITE_ADDRESS,   STATUS3);
	uiStatus_3=uiStatus_3&0x31; //FreeRun +Lock Lost
	return (uiStatus_1&0x01)&&(uiStatus_3&0x21);
}

/****************************************************************************
Function Name: . GetVideoStandard()
Description: . Detect video input standard.
Parameters: . None
Dependencies: .
Assumptions: . .
Limitations: .
Exception: .
Handling: .
****************************************************************************/
Detected_Standard GetVideoStandard(void)
{
	config_read_one(CAMERA_I2C_WRITE_ADDRESS);
	return (Detected_Standard)((lock_Counter1 >> 4)&0x0F);
}

/****************************************************************************
Function Name: . VideoStandardID()
Description: . Set video input flags according to video input
Parameters: . None
Dependencies: .
Assumptions: . .
Limitations: .
Exception: .
Handling: .
****************************************************************************/
void VideoStandardID(void)
{
	bVidDet_NTSC=false;
	bVidDet_PAL=false;
	bVidDet_SECAM=false;
	switch (GetVideoStandard())
	{
		case NTSC_M_J:
		case NTSC_443:
						bVidDet_NTSC=true;
						uiProcFrames=PROC_FRAMES;
						break;
		case PAL_M:
		case PAL_60:
		case PAL_B_G_H_I_D:
		case PAL_COMBINATION_N:
		                bVidDet_PAL=true;
		                uiProcFrames=255;
						break;
		case SECAM:
		case SECAM_525:
		               bVidDet_SECAM=true;
		               uiProcFrames=255;
					   break;
		default:
					   bVidDet_NTSC=true;
					   uiProcFrames=PROC_FRAMES;
					   break;
	}
}

/****************************************************************************
Function Name: . BSP_CAMERA_FrameEventCallback()
Description: . The BSP_CAMERA_FrameEventCallback menthod is a callback when frame is detected and captured.
Parameters: .
Dependencies: .
Assumptions: . .
Limitations: .
Exception: .
Handling: .
****************************************************************************/
void BSP_CAMERA_FrameEventCallback(void)
{
///////Measure FPS //
//	static uint32_t uiStopMil=0;
//	uint32_t uiStartMil;
//	uint32_t uiTotalMil;
//	float fFPS;
//	static float fAFPS;
//////Measure FPS
	static uint16_t uCheckVideoCon=0;
	static bool bVideoNotCon=false;
    #define CL_CAMERA_BG 0x520

	/* User Pressed any Camera Stop streaming key */
	if (1 == pv.stopstreaming)
		return;

	if(DISPLAY_CROPPED_IMAGE== pv.cameramode)
	{
		loadCameraToLCDAsIts(g_lcd_x0_origin, g_lcd_y0_origin,
				             g_y0_height, g_x0_width);
	}
	else
	{
		loadCameraToLCDFullScreen(g_lcd_x0_origin, g_lcd_y0_origin,
			             g_y0_height, g_x0_width);
	}
//	uiStartMil = HAL_GetTick();
//	uiTotalMil= uiStartMil-uiStopMil;
//	uiStopMil = uiStartMil;
//	if (uiTotalMil)
//	{
//    fFPS=1000/(float)uiTotalMil;
//    fAFPS=(fAFPS+fFPS)/2;
//	}
	if (uCheckVideoCon++>2)
	{
		if (bVideoNotCon)
		{
			if (ConnectedVideo())
			{
				if (HAL_OK == ReStart())
		        {
				    	  //Start VC
				    Camera_Set_Stream_ON();
				    HAL_DCMI_Stop(&hdcmi);
				    CM_Delay(10);
				    BSP_CAMERA_ContinuousStart((uint8_t *)CAMERA_FRAME_BUFFER);
		        }
			}
		}
		else
		{
			uCheckVideoCon=0;
		}
		bVideoNotCon=!ConnectedVideo();
	}

}
HAL_StatusTypeDef ReStart(void)
{
	BSP_CAMERA_Stop();
	CM_Delay(1);
	StopStreaming();
	//Power Down IC
	HAL_DMA_DeInit(hdcmi.DMA_Handle);
	HAL_DCMI_DeInit(&hdcmi);
    memset(&hdcmi, 0x00, sizeof(hdcmi));
	HAL_GPIO_WritePin(GPIOC,GPIO_PIN_13,(GPIO_PinState)BSP_IO_PIN_RESET);
	CM_Delay(10);
	//Init registers
	DCMI_HandleTypeDef *phdcmi;
    HAL_StatusTypeDef   status;
    /*** Get the DCMI handle structure ***/
    phdcmi = &hdcmi;
	/*** Configures the DCMI to interface with the camera module ***/
	phdcmi->Init.CaptureRate      = DCMI_CR_ALTERNATE_4_FRAME ;
    phdcmi->Init.HSPolarity       = DCMI_HSPOLARITY_LOW;
    phdcmi->Init.SynchroMode      = DCMI_SYNCHRO_HARDWARE;
    phdcmi->Init.VSPolarity       = DCMI_VSPOLARITY_HIGH;
	phdcmi->Init.ExtendedDataMode = DCMI_EXTEND_DATA_8B;
    phdcmi->Init.PCKPolarity      = DCMI_PCKPOLARITY_RISING;
	phdcmi->Instance              = DCMI;

	BSP_CAMERA_MspInit(&hdcmi, NULL);
    status = HAL_DCMI_Init(phdcmi);

	if (HAL_OK == status)
   {
	  //BSP_CAMERA_PwrUp();
	  HAL_GPIO_WritePin(GPIOC,GPIO_PIN_13,(GPIO_PinState)BSP_IO_PIN_RESET);
	  CM_Delay(10);    /* RST and XSDN signals asserted during 100ms */
	  HAL_GPIO_WritePin(GPIOC,GPIO_PIN_13,(GPIO_PinState)BSP_IO_PIN_SET);
	  CM_Delay(10);;     /* RST de-asserted during 3ms */
	  //Config parameters
	  config_write_one_c(CAMERA_I2C_WRITE_ADDRESS);
   }
	return status;
}
/****************************************************************************
Function Name: . CAMERA_IO_Write_c(uint8_t Addr, uint16_t Reg, uint16_t Value)
Description: . Write Parameters to Camera no HAL Delay used
Parameters: . ADV7180 I2C Address, Control Register, Value to Assign
Dependencies: .
Assumptions: . .
Limitations: .
Exception: .
Handling: .
****************************************************************************/
void CAMERA_IO_Write_c(uint8_t Addr, uint16_t Reg, uint16_t Value)
{
  uint16_t tmp = Value;
  Value = ((uint16_t)(tmp) & 0xFF);
  I2Cx_WriteMultiple_c(Addr, Reg, I2C_MEMADD_SIZE_8BIT,(uint8_t*)&Value, 1);
  CM_Delay(20);
}

/****************************************************************************
Function Name: . config_write_one_c(uint16_t DeviceAddr)
Description: . Set Video Configuration only essential
Parameters: . ADV7180 I2C Address
Dependencies: .
Assumptions: . .
Limitations: .
Exception: .
Handling: .
****************************************************************************/
void config_write_one_c(uint16_t DeviceAddr)
{
  CAMERA_IO_Write_c(DeviceAddr,   INPUT_CONTROL,        BYTE(0,0));   /* '04' select ain 3, autodetect composite standart*/
  CAMERA_IO_Write_c(DeviceAddr,   VIDEO_SELECTION,      BYTE(0xC,0xC));   /* '04' select ain 3, autodetect composite standart*/
  CAMERA_IO_Write_c(DeviceAddr,   VS_FIELD_CTL_1,       BYTE(1,2));   /* '12'  *******default*/
  CAMERA_IO_Write_c(DeviceAddr,   VS_FIELD_CTL_2,       BYTE(0xC,1)); /* 'C1' default*/
  CAMERA_IO_Write_c(DeviceAddr,   VS_FIELD_CTL_3,       BYTE(0xC,0)); /* 'C0' default*/
  CAMERA_IO_Write_c(DeviceAddr,   VS_FIELD_PIN_CTL,     BYTE(0,5));   /*default*/    //0 for field 0x01 for vsync
  CAMERA_IO_Write_c(DeviceAddr,   MANDATE_TWO,          BYTE(8,1));
  CAMERA_IO_Write_c(DeviceAddr,   MANDATE_ONE,          BYTE(0,0));
  CAMERA_IO_Write_c(DeviceAddr,   MANDATE_THREE,        BYTE(0,0xD));
  CAMERA_IO_Write_c(DeviceAddr,   EXT_OUTPUT_CONTROL,   BYTE(0,1));
  CAMERA_IO_Write_c(DeviceAddr,   DEFAULT_VALUE_C,      BYTE(0x0,0x0));
  CAMERA_IO_Write_c(DeviceAddr,   CONTRAST,             BYTE(0x8,0xF));//Default BYTE(0x8,0x0)
  CAMERA_IO_Write_c(DeviceAddr,   BRIGHTNESS,           BYTE(0xF,0xD));//Default BYTE(0x0,0x0)
  CAMERA_IO_Write_c(DeviceAddr,   HUE,                  BYTE(0x0,0x7));//Default BYTE(0x0,0x0)
  CAMERA_IO_Write_c(DeviceAddr,   SD_SATURATION_CB,     BYTE(0xF,0xF));//Default BYTE(0x8,0x0)
  CAMERA_IO_Write_c(DeviceAddr,   SD_SATURATION_CR,     BYTE(0xF,0xF));//Default BYTE(0x8,0x0)
  CAMERA_IO_Read(DeviceAddr,DEFAULT_VALUE_C);
}

/****************************************************************************
Function Name: . overrun_callback(void)
Description: . Restore Lost Sync
Parameters: . none
Dependencies: .
Assumptions: . .
Limitations: .
Exception: .
Handling: .
****************************************************************************/

void overrun_callback(void)
{
  HAL_DCMI_Stop(&hdcmi);
  BSP_CAMERA_ContinuousStart((uint8_t *)CAMERA_FRAME_BUFFER);
}

/****************************************************************************
Function Name: . HAL_DCMI_ErrorCallback()
Description: . Error callback
Parameters: . hdcmi: pointer to the DCMI handle
Dependencies: .
Assumptions: . .
Limitations: .
Exception: .
Handling: .
****************************************************************************/
void HAL_DCMI_ErrorCallback(DCMI_HandleTypeDef *hdcmi)
{
  BSP_CAMERA_ErrorCallback();
}

/****************************************************************************
Function Name: . BSP_CAMERA_ErrorCallback()
Description: . Error callback.
Parameters: .
Dependencies: .
Assumptions: . .
Limitations: .
Exception: .
Handling: .
****************************************************************************/
__weak void BSP_CAMERA_ErrorCallback(void)
{
  /* NOTE : This function Should not be modified, when the callback is needed,
  the HAL_DCMI_ErrorCallback could be implemented in the user file
  */
}
void start_camera(void)
{
  #define XRESOLUTION  (480)//*2
#define YRESOLUTION  (272/2)
  HAL_DCMI_ConfigCROP(&hdcmi,           /* Crop in the middle of the VGA picture */
                       0,//x0 =0
                       108,//y0 =44
                       (XRESOLUTION*2)-1,//(CAMERA_480x272_RES_X * 2 ) - 1, // xSize = 480 pixels * 2 ( 4 Bytes == 2 pixels) = 960-1=959
                       YRESOLUTION-1);//ySize = 272/2 -1= 136-1=135
    HAL_DCMI_EnableCROP(&hdcmi);
  BSP_CAMERA_ContinuousStart((uint8_t *)CAMERA_FRAME_BUFFER);
}

/** @brief CropCameraImageResizetoLCDSize
 *
 *  This function crops the camera image and display the image to
 *  the LCD display size.
 *
 *  @param  x0_origin :  Camera X0 Origin
 *          y0_origin :  Camera Y0 Origin
 *          x0_width  :  Camera width to capture
 *          y0_height :  Camera height to capture
 *          lcd_x0_origin : LCD display x0 origin
 *          lcd_y0_origin : LCD display y0 origin
 *
 *  @return none
 *
 */
void CropCameraImageResizetoLCDSize(uint16_t x0_origin, uint16_t y0_origin,
						uint16_t x0_width, uint16_t y0_height,
						uint16_t lcd_x0_origin, uint16_t lcd_y0_origin)
{
    /* Crop the image */
	HAL_DCMI_ConfigCrop(&hdcmi,
    		            CAMERA_X0_ORIGIN_AFTER_BLANK_PERIOD + x0_origin,
						CAMERA_Y0_ORIGIN_AFTER_BLANK_PERIOD + y0_origin,
						(x0_width *2 ) - 1,
						(y0_height - 1));
    HAL_DCMI_EnableCROP(&hdcmi);

	/* Start the camera capture */
	HAL_DCMI_Start_DMA(&hdcmi, DCMI_MODE_CONTINUOUS, (uint32_t)CAMERA_IMAGE_BUFFER,
                              ((x0_width*y0_height)/2));

	/* Store the parameter to the global variable */
	g_x0_width      = x0_width;
	g_y0_height     = y0_height;
	g_lcd_x0_origin = lcd_x0_origin;
	g_lcd_y0_origin = lcd_y0_origin;

	/* Configure the Alpha blending screen */
    HAL_LTDC_SetAlpha_NoReload(pv.hltdc, 255, 1);

}
#endif
/**************** (C) COPYRIGHT 2012 Maxima Technologies *******************/
/***************************** END OF FILE *********************************/
