/*
* j1939_dynpgn.h - definitions for dynamic pgns
*
* Copyright (c) 2014-2021 emotas embedded communication GmbH
*
*-------------------------------------------------------------------
* $Id: j1939_dynpgn.h 36818 2021-06-11 12:34:35Z boe $
*-------------------------------------------------------------------
*
*
*/

/**
* \brief definitions for dynamic pgns
*
* \file j1939_dynpgn.h - definitions for dynamic pgns
*
* This header inludes defines for all services of the J1939 library.
*
*/

#ifndef J1939_DYNPGN_H
#define J1939_DYNPGN_H 1

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#include <co_datatype.h>
#include <co_cob.h>
#include <co_commtask.h>
#include <co_drv.h>
#include <co_timer.h>

#include <j1939_datatype.h>
#include <j1939_indication.h>


#define J1939_DIR_TRANSMIT CO_TRUE
#define J1939_DIR_RECEIVE  CO_FALSE

/* function prototypes */
EXTERN_DECL RET_T j1939DynPgnInit(CO_LINE_DECL);
EXTERN_DECL RET_T j1939DynPgnDeInit(CO_LINE_DECL);

EXTERN_DECL RET_T j1939DynPgnCreatePgn(CO_LINE_DECL, UNSIGNED32 pgn,
#ifdef J1939_MULTI_SRC_ADDR
		UNSIGNED8	srcAddr,
#endif /* J1939_MULTI_SRC_ADDR */
		DIRECTION_T direction, UNSIGNED32 spnCnt,
		UNSIGNED32 cycleTime, UNSIGNED16 inhibitTime, UNSIGNED8 priority);
EXTERN_DECL RET_T j1939DynPgnDeletePgn(CO_LINE_DECL, UNSIGNED32 pgn,
#ifdef J1939_MULTI_SRC_ADDR
		UNSIGNED8	srcAddr,
#endif /* J1939_MULTI_SRC_ADDR */
		DIRECTION_T direction);
EXTERN_DECL RET_T j1939DynPgnEnable(CO_LINE_DECL, UNSIGNED32 pgn, DIRECTION_T direction);
EXTERN_DECL RET_T j1939DynPgnDisable(CO_LINE_DECL, UNSIGNED32 pgn, DIRECTION_T direction);

EXTERN_DECL RET_T j1939DynPgnCreateSpn(CO_LINE_DECL, UNSIGNED32 pgn,
#ifdef J1939_MULTI_SRC_ADDR
		UNSIGNED8	srcAddr,
#endif /* J1939_MULTI_SRC_ADDR */
		UNSIGNED32 spn, DIRECTION_T direction, UNSIGNED16 size, void *pVar);
EXTERN_DECL RET_T j1939DynPgnDeleteSpn(CO_LINE_DECL, UNSIGNED32 pgn,
#ifdef J1939_MULTI_SRC_ADDR
		UNSIGNED8	srcAddr,
#endif /* J1939_MULTI_SRC_ADDR */
		UNSIGNED32 spn, DIRECTION_T direction);

EXTERN_DECL UNSIGNED32 j1939DynPgnComposePrio(CO_LINE_DECL, UNSIGNED32 pgn, UNSIGNED8 priority);
EXTERN_DECL RET_T j1939DynPgnConfigPgn(CO_LINE_DECL, UNSIGNED32 pgn, DIRECTION_T direction,
		UNSIGNED16 cycleTime, UNSIGNED16 inhibitTime, UNSIGNED8 priority);

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* J1939_DYNPGN_H */
