﻿/*
* ij1939_transmit.h - contains internal defines
*
* Copyright (c) 2014-2021 emotas embedded communication GmbH
*
*-------------------------------------------------------------------
* $Id: ij1939_transmit.h 38573 2021-10-15 08:39:19Z boe $
*
*-------------------------------------------------------------------
*
*
*
*/

/**
* \file
* \brief dataty type header
*/

#ifndef IJ1939_TRANSMIT_H
#define IJ1939_TRANSMIT_H 1

#ifdef J1939_NMEA2000_SUPPORTED
# include "ico_event.h"
#endif /* J1939_NMEA2000_SUPPORTED */

/* datatypes */

typedef enum {
	REC_STATE_ACTIVE,
	REC_STATE_FAILED
} REC_STATE_T;

typedef enum {
	FPT_STATE_FREE,
	FPT_STATE_TRANSMIT,
	FPT_STATE_RECEIVE
} FPT_STATE_T;

typedef struct {
	const J1939_PGN_CONFIG_T	*pCfg;
#ifdef CO_CAN_TIMESTAMP_SUPPORTED
	CO_CAN_TIMESTAMP_T			timeStamp;
#endif /* CO_CAN_TIMESTAMP_SUPPORTED */
	COB_REFERENZ_T	cob;
	CO_TIMER_T		timer;
	UNSIGNED8		dstAddress;
	UNSIGNED8		flags;
	REC_STATE_T		state;
#ifdef J1939_NMEA2000_SUPPORTED
	CO_EVENT_T		fptEvtHdl;
	FPT_STATE_T		fptState;
	UNSIGNED16		msgSize;
	UNSIGNED16		fptTransferSize;
	UNSIGNED8		fptFrameCnt;
	UNSIGNED8		fptSeqCnt;
	UNSIGNED8		fptSrcAddress;
#endif /* J1939_NMEA2000_SUPPORTED */
} PGN_CFG_T;

/* function prototypes */

RET_T	ij1939_transmit(CO_LINE_DECL, const J1939_PGN_CONFIG_T *pCfg,
			COB_REFERENZ_T cob, UNSIGNED8 flags);
RET_T	ij1939_transmitData(CO_LINE_DECL, const UNSIGNED8 *trData,
			COB_REFERENZ_T cob, UNSIGNED8 flags);
RET_T	ij1939_transmitMap(CO_LINE_DECL, const J1939_PGN_CONFIG_T *pCfg,
			UNSIGNED8 buf[], UNSIGNED32	bufOffs, UNSIGNED8 bufLen);
RET_T	ij1939_transmitPgnInit(CO_LINE_DECL, UNSIGNED16 cnt,
			const J1939_PGN_CONFIG_T *pCfg);
void	ij1939_transmitUpdateCobs(CO_LINE_DECL, UNSIGNED8	nodeAddress);
const	J1939_PGN_CONFIG_T	*ij1939_transmitGetPgnCfg(CO_LINE_DECL,
			UNSIGNED32 pgn);
RET_T	ij1939_transmitUpdatePgnCob(CO_LINE_DECL, UNSIGNED32 pgn,
			UNSIGNED8	destNode);

#ifdef J1939_RV_C_SUPPORTED
void	ij1939_transmitAcknowledgement(CO_LINE_DECL, UNSIGNED32 pgn,
			UNSIGNED8 destNode, UNSIGNED8 acq, UNSIGNED8  instance,
			UNSIGNED8 instanceBank);
#else /* J1939_RV_C_SUPPORTED */
void	ij1939_transmitAcknowledgement(CO_LINE_DECL, UNSIGNED32 pgn,
			UNSIGNED8 destNode, UNSIGNED8 acq);
#endif /* J1939_RV_C_SUPPORTED */

RET_T	ij1939_transmitPgnVarInit(const UNSIGNED16	*pTrCnt
			);

RET_T	ij1939_transmitPgnEnDisable(CO_LINE_DECL, UNSIGNED32 pgn, BOOL_T enable);
RET_T	ij1939_transmitFinished(CO_LINE_DECL, UNSIGNED32 pgn, UNSIGNED8 dstNode, RET_T error);

PGN_CFG_T *ij1939_transmitFindPgn(CO_LINE_DECL, UNSIGNED32 pgn);
RET_T	ij1939_transmitIncrDynPgnCnt(CO_LINE_DECL);
RET_T	ij1939_transmitDecrDynPgnCnt(CO_LINE_DECL);


#endif /* IJ1939_TRANSMIT_H */

